#include "BuildDataOrganizer.h"

void BuildDataOrganizer::BuildDataOrganizer_Manager(DataOrganizer* organizer, Inputs* input, string Directory_Input_CLArg)
{
	//If Model is SpatialTemperatureHydro then
	if (input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {
		//BuildSpatialDataOrganizer function is called
		BuildSpatialDataOrganizer(organizer, input, Directory_Input_CLArg);
		//Calling BuildDataOrganizer::SetGIMissingParams to add variables and values to non BulkArea folders, e.g., GI folders
		BuildDataOrganizer::SetGIMissingParams(organizer, input);
	}
	//Else if Model is StatisticalHydro then
	else if (input->SimulationStringParams["Model_Selection"] == "StatisticalHydro"|| input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {

		//BuildStatisticalDataOrganizer function is called
		BuildStatisticalDataOrganizer(organizer, input, Directory_Input_CLArg);
		//Calling BuildDataOrganizer::SetGIMissingParams to add variables and values to non BulkArea folders, e.g., GI folders
		BuildDataOrganizer::SetGIMissingParams(organizer, input);
	}
	//If Flag_GI_Simulated equals 1 then GI simulated and compute variable averages
	if (input->RepoDict["Flag_GI_Simulated"] == 1) {
		//Compute_CoverType_WeightedAverage_frac function computes GI variable weighted averages
		Compute_CoverType_WeightedAverage_frac(organizer, input);
	}
}

//BuildSpatialDataOrganizer function called for Model = SpatialTemperatureHydro
//Note: For model = SpatialTemperatureHydro, DataDrawer_ID has maximum = maxRows * maxCols, and DataFolder_ID typically has maximum = 1
void BuildDataOrganizer::BuildSpatialDataOrganizer(DataOrganizer* organizer, Inputs* input, string Directory_Input_CLArg)
{
	//Catchment_Type_Area_m2 initialized to size 2
	//Note: Model SpatialTemperatureHydro will sum catchment area into Catchment_Type_Area_m2
	input->RepoVecDict["Catchment_Type_Area_m2"] = { 0,0 };
	//initialize placeholder for DataDrawer_ID counter, set to zero, used in Compute_CoverType_WeightedAverage_m2 function
	int DataDrawer_ID = 0;
	int Count_ArrayItem = 0;
	//Folder_Type_ID will track Folder as BulkArea of GreenInfrastructure
	int Folder_Type_ID = 0;
	//input->RepoDict["Catchment_PerviousArea_AffectingStorageDeficit_m2"] reset to zero
	input->RepoDict["Catchment_PerviousArea_AffectingStorageDeficit_m2"] = 0;
	//Flag_GI_Simulated flag set = 0, ensuring no transfer of water between GI and BulkArea
	//Note: SpatialTemeperatureHydro model may be refactored to support GI, and this flag could be set to 1
	input->RepoDict["Flag_GI_Simulated"] = 0;
	
	//If RefWeatherLocationElevation_m <> 0 m then use HydroPlusConfig.xml value, else input->DEM defined by dem.asc in Inputs::ProcessTopographyData
	//Note: HydroPlusConfig.xml element RefWeatherLocationElevation_m is set to "DEM" or Value <> 0 m
	if (input->TemperatureCalculationParams["RefWeatherLocationElevation_m"] != 0) {
		//Note: MapPixel_ReferenceStation_ID is defined by HydroPlusConfig.xml element RefWeatherLocationRowCol by converting row, col to MapPixel_ID in Inputs::ProcessTemperatureExecutionParams
		input->DEM[input->MapPixel_ReferenceStation_ID] = input->TemperatureCalculationParams["RefWeatherLocationElevation_m"];
	}
	//If input->DEM[input->MapPixel_ReferenceStation_ID] equals NODATA then the user may have entered 0 into HydroPlusConfig.xml
	//Note: HydroPlusConfig.xml element RefWeatherLocationElevation_m when 'DEM' will generate a value of 0, causing this issue
	if (input->DEM[input->MapPixel_ReferenceStation_ID] == input->LocationParams["NODATA"]) {
		//Write output to notify user of faulty RefWeatherLocationElevation_m value
		cout << "Caution: DataOrganizer found issue with RefWeatherLocationElevation_m value in HydroPlusConfig.xml" << endl;
		cout << "Caution: If it was intended as 0 then fine, but this may be an error as few stations are at 0 m." << endl;
		cout << "Caution: If it was intended as DEM then make RefWeatherLocationRowCol something other than NODATA." << endl;
		input->DEM[input->MapPixel_ReferenceStation_ID] = input->TemperatureCalculationParams["RefWeatherLocationElevation_m"];
	}
	//If RefWeatherLocationAspect_deg > 0 then use HydroPlusConfig.xml value, else input->AspectGround_N_0_rad defined by dem.asc in Inputs::ProcessTopographyData
	//Note: HydroPlusConfig.xml element RefWeatherLocationAspect_deg is set to "DEM" or Value 1 to 360 degrees
	if (input->TemperatureCalculationParams["RefWeatherLocationAspect_deg"] > 0) {
		//Note: MapPixel_ReferenceStation_ID is defined by HydroPlusConfig.xml element RefWeatherLocationRowCol by converting row, col to MapPixel_ID in Inputs::ProcessTemperatureExecutionParams
		input->AspectGround_N_0_rad[input->MapPixel_ReferenceStation_ID] = input->TemperatureCalculationParams["RefWeatherLocationAspect_deg"] * Ratio_Radian_to_Degree;
	}
	//If RefWeatherLocationSlope_deg > 0 then use HydroPlusConfig.xml value, else input->SlopeGround_rad defined by dem.asc in Inputs::ProcessTopographyData
	//Note: HydroPlusConfig.xml element RefWeatherLocationSlope_deg is set to "DEM" or Value > 0.001 degrees
	if (input->TemperatureCalculationParams["RefWeatherLocationSlope_deg"] > 0) {
		//Note: MapPixel_ReferenceStation_ID is defined by HydroPlusConfig.xml element RefWeatherLocationRowCol by converting row, col to MapPixel_ID in Inputs::ProcessTemperatureExecutionParams
		input->SlopeGround_rad[input->MapPixel_ReferenceStation_ID] = input->TemperatureCalculationParams["RefWeatherLocationSlope_deg"] * Ratio_Radian_to_Degree;
	}
	//If RefWeatherLocationTopographicIndex == 1 then use TI_Value_Average, else input->TI_Value defined by dem.asc in Inputs::ProcessTopographyData 
	//Note: HydroPlusConfig.xml element RefWeatherLocationTopographicIndex is set to "DEM" or "Avg"; Avg converted to 1 Inputs::ProcessTemperatureExecutionParams function
	if (input->TemperatureCalculationParams["RefWeatherLocationTopographicIndex"] == 1) {
		//Note: MapPixel_ReferenceStation_ID is defined by HydroPlusConfig.xml element RefWeatherLocationRowCol by converting row, col to MapPixel_ID in Inputs::ProcessTemperatureExecutionParams
		input->TI_Value[input->MapPixel_ReferenceStation_ID] = input->TI_Value_Average;
	}
	//If Flag_CoolAir_AnthropogenicHeat == 1 & RefWeatherLocationAnthropogenicHeatMax_Wpm2 >= 0 then use value from HydroPlusConfig.xml, else use value from ah_max_w_p_m2.asc in Input folder
	//Note: HydroPlusConfig.xml element RefWeatherLocationAnthropogenicHeatMax_Wpm2 is intended to avoid need for a reference station in the map domain
	if (input->SimulationStringParams["Flag_CoolAir_AnthropogenicHeat"] == "1" && input->TemperatureCalculationParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"] >= 0) {
		//Note: MapPixel_ReferenceStation_ID is defined by HydroPlusConfig.xml element RefWeatherLocationRowCol by converting row, col to MapPixel_ID in Inputs::ProcessTemperatureExecutionParams
		input->AnthropogenicHeat_Max_W_p_m2[input->MapPixel_ReferenceStation_ID] = input->TemperatureCalculationParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"];
	}

	//Error handling for CoolAir simulations
	//maxRows and maxCols set to 300 based on typical RAM limitations for CoolAir runs
	double maxRows = 300; double maxCols = 300;
	if (int(input->LocationParams["NumberOfLocations"]) > maxRows * maxCols) {
		cout << endl << "Please note: Input map Rows and Columns exceed 300 x 300 and may overwhelm the computer RAM." << endl;
		cout << "Explanation: If RAM is exceeded, then the simulation will likely hang when creating DataOrganizer folders." << endl;
		cout << "Solution: If simulation hangs, then use smaller map arrays, either with coarser pixels or with post-run mosaics." << endl;
	}

	//For Loop through the NumberOfLocations, where the upper bound is based on the nrows * ncols, number of map pixels
	//Note: NumberOfLocations is number of map pixels in dem.asc, equal to nrows * ncols for SpatialTemperatureHydro; 1 for StatisticalHydro 
	//Note: Convert between row, column, and MapPixel_ID
	//Note: Conversion between MapPixel_ID and row & column pair, with MapPixel_ID starting at 0, row & col starting at 1
	//Note: Conversion Eq: row=(int)(MapPixel_ID/input->ncols+1); col=(MapPixel_ID%(int)(input->ncols)+1); MapPixel_ID = (row * nCols) + col
	for (int MapPixel_ID = 0; MapPixel_ID < input->LocationParams["NumberOfLocations"]; ++MapPixel_ID) {
		//row derived from MapPixel_ID
		//row = (int)(MapPixel_ID / input->ncols + 1), derived from MapPixel_ID, first row indexed to 1
		int row = (int)(MapPixel_ID / (int)(input->LocationParams["nCols"]) + 1);
		//col derived from MapPixel_ID
		//col = (MapPixel_ID % (int)(input->ncols) + 1), derived from MapPixel_ID, first col indexed to 1
		int col = (MapPixel_ID % (int)(input->LocationParams["nCols"]) + 1);
		//Create DataDrawer vector with DataFolder structure properties
		vector<DataFolder*> DataDrawer;
		//initialize DataFolder_ID counter for DataFolders
		int DataFolder_ID = 0;

		//While DataFolder_ID is less than DataDrawers[0].size, which typically is a size 1
		//Note: DataDrawers[0] contains the first DataFolder in vector, which is the BulkArea in HydroPlusConfig.xml file
		//Note: DataDrawers vector size is determined when reading in the HydroPlusConfig.xml and other input data
		while (DataFolder_ID < input->DataDrawers[0].size()) {

			//Assign folderInfo the DataFolder DataFolder_ID number
			unordered_map<string, double> folderInfo = input->DataDrawers[0][DataFolder_ID];

			//If input-> NLCDLandCoverData, TreeCoverData, ImperCoverData, DEM or BlockGroupData at [MapPixel_ID] equals NODATA then continue with loop
			//Note: Checking 4 required land cover types is intended for error handling, so computations do not occur for cases when NODATA is different across maps
			if (int(input->NLCDLandCoverData[MapPixel_ID]) == input->LocationParams["NODATA"] ||
				int(input->TreeCoverData[MapPixel_ID]) == input->LocationParams["NODATA"] ||
				int(input->ImperCoverData[MapPixel_ID]) == input->LocationParams["NODATA"] ||
				int(input->DEM[MapPixel_ID]) == input->LocationParams["NODATA"]) {

				//NLCDLandCoverData, TreeCoverData, ImperCoverData, and DEM at [MapPixel_ID] equals input->LocationParams["NODATA"] to force congruence
				input->NLCDLandCoverData[MapPixel_ID] = input->LocationParams["NODATA"];
				input->TreeCoverData[MapPixel_ID] = input->LocationParams["NODATA"];
				input->ImperCoverData[MapPixel_ID] = input->LocationParams["NODATA"];
				input->DEM[MapPixel_ID] = input->LocationParams["NODATA"];
				//If BlockGroupData.size > 0 then set BlockGroupData to NODATA
				if (input->BlockGoupData.size() > 0){ input->BlockGoupData[MapPixel_ID] = input->LocationParams["NODATA"];}

				//break breaks from while loop and returns to next item in for loop
				break;
			}			
			//Else If input vector NLCDLandCoverData at MapPixel_ID is not equal to NODATA, then enter for analysis
			else if (input->NLCDLandCoverData[MapPixel_ID] != input->LocationParams["NODATA"]) {

				//folder is pointer to new DataFolder class for SpatialTemperatureHydro model
				DataFolder* folder = new DataFolder;
				//Folder ParamDict variable set to folderInfo, or input->DataDrawers[0][DataFolder_ID]
				folder->ParamDict = folderInfo;
				//Folder ParamStringDict variable set to input->StringDataDrawers[0][DataFolder_ID]
				folder->ParamStringDict = input->StringDataDrawers[0][DataFolder_ID];

				//Set Type to equal BulkArea
				folder->ParamStringDict["Type"] = "BulkArea";
				//Folder_Type_ID defined as zero for BulkArea
				Folder_Type_ID = 0;
				//Maprow defined for folder
				//Catchment_Type_Area_m2 (m2) contains entire catchment area of this folder type
				input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID] += folder->ParamDict["Area_m2"];
				//Folder StorageDeficit_VadoseZone_m variable set to the initial average soil moisture deficit, AveSMD (m), which regulates TOPMODEL theory
				input->RepoDict["StorageDeficit_VadoseZone_m"] = input->AveSMD;

				//CheckHydroPlusConfigInputData function performs quality assurance on HydroPlusConfig.xml inputs for each folder
				CheckHydroPlusConfigInputData(folder, input, Directory_Input_CLArg);

				//Call AddFolderAreaSpatialCoverData function to get BulkArea parameters, sending pointers to folder, input, and MapPixel_ID
				AddFolderAreaSpatialCoverData(folder, input, MapPixel_ID, Folder_Type_ID);

				//Call Compute_CoverType_WeightedAverage_m2 function to compute weighted areas across catchment
				Compute_CoverType_WeightedAverage_m2(folder, input, DataDrawer_ID, DataFolder_ID, MapPixel_ID, Count_ArrayItem);

				//DataDrawer vector stores the folder
				DataDrawer.push_back(folder);

				//add 1 to Count_ArrayItem 
				Count_ArrayItem = Count_ArrayItem + 1;
			}

			//Else Type not equal to BulkArea
			//Note: Refactor to allow for GreenInfrastructure Type which Folder_Type_ID = 1
			else {
				//folder is pointer to new DataFolder class for SpatialTemperatureHydro model
				DataFolder* folder = new DataFolder;
				//Folder ParamDict variable set to folderInfo, or input->DataDrawers[0][DataFolder_ID]
				folder->ParamDict = folderInfo;
				//Folder_Type_ID defined as one for GreenInfrastructure
				Folder_Type_ID = 1;
				//Catchment_Type_Area_m2 (m2) contains entire catchment area of this folder type
				input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID] += folder->ParamDict["Area_m2"];
				//If HydroPlusConfig Type is not BulkArea or another known Type, then abort program
				cout << "Warning: The HydroPlusConfig.xml has a variable for the element Type = " << input->StringDataDrawers[0][DataFolder_ID]["Type"] << ", which is not compatible with Model = SpatialTemperatureHydro." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: When i-Tree Cool Air model does not find a known Type, it cannot allocate a water balance." << endl;
				cout << "Correction: In the HydroPlusConfig.xml file, set the DataFolder Type = BulkArea." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}

			//DataFolder_ID incremented by 1. 
			//Note: For Model SpatialTemperatureHydro, typical maximum value of DataFolder_ID = 1
			DataFolder_ID = DataFolder_ID + 1;
		}
		//DataDrawer_ID incremented by 1. 
		//Note: For Model SpatialTemperatureHydro, typical maximum value of DataDrawer_ID = maxRows * maxCols = map array size
		DataDrawer_ID = DataDrawer_ID + 1;

		//Store DataDrawer to DataDrawers vector
		organizer->DataDrawers.push_back(DataDrawer);
	}

	//Algorithm at end of loop ensures catchment_ImperviousCover_noTreeCanopy_frac and other catchment covers have correct precision and sum to 1.0
	//Note: Consider refactor to place this in function BuildDataOrganizer::Compute_CoverType_WeightedAverage_frac for Spatial and Statistical Hydro

	//Folder_Type_ID defined as 0 for BulkArea
	Folder_Type_ID = 0;
	//If input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID] > 0 then enter for division
	//Note: WaterBalance_Summary::writeWaterBalanceFile uses same calculations, so consider refactor to consolidate
	if (input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID] > 0) {
		//catchment_TotalPerviousCover_frac (frac) = int(((TotalPerviousCover_Type_m2 / Catchment_Type_Area_m2) * 1000) + 0.5) / 1000.0
		//Note: RHS function divides by 1000 after taking integer of the term that takes product with 1000 and sums 0.5 to round up
		input->RepoDict["catchment_TotalPerviousCover_frac"] = int(((input->RepoVecDict["TotalPerviousCover_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_TotalImperviousCover_frac"] = int(((input->RepoVecDict["TotalImperviousCover_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_TotalTreeCover_frac"] = int(((input->RepoVecDict["TotalTreeCover_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"] = int(((input->RepoVecDict["TreeCanopyCover_overImpervious_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] = int(((input->RepoVecDict["TreeCanopyCover_overPervious_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] = int(((input->RepoVecDict["ImperviousCover_noTreeCanopy_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] = int(((input->RepoVecDict["ShortVegCover_noTreeCanopy_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] = int(((input->RepoVecDict["SoilCover_noTreeCanopy_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"] = int(((input->RepoVecDict["PermeablePavementCover_noTreeCanopy_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"] = int(((input->RepoVecDict["WaterCover_noTreeCanopy_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_InfiltExcessGovernedArea_frac"] = int(((input->RepoVecDict["InfiltExcessGovernedArea_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_DCIA_frac"] = int(((input->RepoVecDict["DCIA_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_MacroPore_frac"] = int(((input->RepoVecDict["MacroPore_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
		input->RepoDict["catchment_MacroPore_frac"] = int(((input->RepoVecDict["MacroPore_Type_m2"][Folder_Type_ID] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID]) * 1000) + 0.5) / 1000.0;
	}
	//catchment_TotalPerviousCover_frac (frac) = catchment_TreeCanopyCover_overPervious_frac + catchment_ShortVegCover_noTreeCanopy_frac + catchment_SoilCover_noTreeCanopy_frac + catchment_PermeablePavementCover_noTreeCanopy_frac
	input->RepoDict["catchment_TotalPerviousCover_frac"] = input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] + input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] + input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] + input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"];

	//catchment_TotalImperviousCover_frac (frac) = catchment_ImperviousCover_noTreeCanopy_frac + catchment_TreeCanopyCover_overImpervious_frac
	input->RepoDict["catchment_TotalImperviousCover_frac"] = input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] + input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"];

	//catchment_TotalTreeCover_frac (frac) = catchment_TreeCanopyCover_overPervious_frac + catchment_TreeCanopyCover_overImpervious_frac
	input->RepoDict["catchment_TotalTreeCover_frac"] = input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] + input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"];

	//TotalCover_frac (frac) initialized to equal sum of catchment_ImperviousCover_noTreeCanopy_frac (frac), catchment_TreeCanopyCover_overImpervious_frac (frac), catchment_TreeCanopyCover_overPervious_frac (frac), catchment_ShortVegCover_noTreeCanopy_frac (frac), catchment_SoilCover_noTreeCanopy_frac (frac), catchment_PermeablePavementCover_noTreeCanopy_frac (frac), catchment_WaterCover_noTreeCanopy_frac (frac)
	double TotalCover_frac = input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] + input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"] + input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] + input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] + input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] + input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"] + input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"];

	//TotalCover_Remainder_frac (frac) initialized to equal 1.0 - TotalCover_frac
	double TotalCover_Remainder_frac = 1.0 - TotalCover_frac;

	//If TotalCover_Remainder_frac != 0.0 then add the remaining gap in a sequence that lastly updates tree cover, presumed correct
	//Note: Algorithm could be improved to search for smallest unit of area (frac) before setting anything to zero but ...
	//Note: ... Algorithm will never likely set anything to zero due to (1.0 - TotalCover_frac) naturally positive, without need for abs
	if (TotalCover_Remainder_frac != 0.0) {
		if (input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"] > 0.0) {
			//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - catchment_PermeablePavementCover_noTreeCanopy_frac)
			double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"]);
			//catchment_PermeablePavementCover_noTreeCanopy_frac (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
			input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
			//If catchment_PermeablePavementCover_noTreeCanopy_frac < 0 then set to zero
			if (input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"] < 0) { input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"] = 0.0; }
		}
		else if (input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] > 0.0) {
			//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - catchment_SoilCover_noTreeCanopy_frac)
			double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"]);
			//catchment_SoilCover_noTreeCanopy_frac (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
			input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
			//If catchment_SoilCover_noTreeCanopy_frac < 0 then set to zero
			if (input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] < 0) { input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] = 0.0; }
		}
		else if (input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"] > 0.0) {
			//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - catchment_WaterCover_noTreeCanopy_frac)
			double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"]);
			//catchment_WaterCover_noTreeCanopy_frac (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
			input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
			//If catchment_WaterCover_noTreeCanopy_frac < 0 then set to zero
			if (input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"] < 0) { input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"] = 0.0; }
		}
		else if (input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] > 0.0) {
			//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - catchment_ShortVegCover_noTreeCanopy_frac)
			double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"]);
			//catchment_ShortVegCover_noTreeCanopy_frac (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
			input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
			//If catchment_ShortVegCover_noTreeCanopy_frac < 0 then set to zero
			if (input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] < 0) { input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] = 0.0; }
		}
		else if (input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] > 0.0) {
			//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - catchment_ImperviousCover_noTreeCanopy_frac)
			double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"]);
			//catchment_ImperviousCover_noTreeCanopy_frac (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
			input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
			//If catchment_ImperviousCover_noTreeCanopy_frac < 0 then set to zero
			if (input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] < 0) { input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] = 0.0; }
		}
		else if (input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"] > 0.0) {
			//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - catchment_TreeCanopyCover_overImpervious_frac)
			double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"]);
			//catchment_TreeCanopyCover_overImpervious_frac (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
			input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
			//If catchment_TreeCanopyCover_overImpervious_frac < 0 then set to zero
			if (input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"] < 0) { input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"] = 0.0; }
		}
		else if (input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] > 0.0) {
			//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - catchment_TreeCanopyCover_overPervious_frac)
			double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"]);
			//catchment_TreeCanopyCover_overPervious_frac (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
			input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
			//If catchment_TreeCanopyCover_overPervious_frac < 0 then set to zero
			if (input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] < 0) { input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] = 0.0; }
		}
		//catchment_TotalPerviousCover_frac (frac) = catchment_TreeCanopyCover_overPervious_frac + catchment_ShortVegCover_noTreeCanopy_frac + catchment_SoilCover_noTreeCanopy_frac + catchment_PermeablePavementCover_noTreeCanopy_frac
		input->RepoDict["catchment_TotalPerviousCover_frac"] = input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] + input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] + input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] + input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"];

		//catchment_TotalImperviousCover_frac (frac) = catchment_ImperviousCover_noTreeCanopy_frac + catchment_TreeCanopyCover_overImpervious_frac
		input->RepoDict["catchment_TotalImperviousCover_frac"] = input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] + input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"];

		//catchment_TotalTreeCover_frac (frac) = catchment_TreeCanopyCover_overPervious_frac + catchment_TreeCanopyCover_overImpervious_frac
		input->RepoDict["catchment_TotalTreeCover_frac"] = input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] + input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"];
	}
}	

//BuildStatisticalDataOrganizer function called for Model = StatisticalHydro
void BuildDataOrganizer::BuildStatisticalDataOrganizer(DataOrganizer* organizer, Inputs* input, string Directory_Input_CLArg)
{
	//DataFolder_counter is initialized to zero
	int DataFolder_counter = 0;
	int Count_ArrayItem = 0;
	//Folder_Type_ID will track Folder as BulkArea of GreenInfrastructure
	int Folder_Type_ID = 0;
	//input->RepoDict["Catchment_PerviousArea_AffectingStorageDeficit_m2"] reset to zero
	input->RepoDict["Catchment_PerviousArea_AffectingStorageDeficit_m2"] = 0;

	//Loop through the DataDrawers, where the upper bound is based on the general DataDrawer attribute .size()
	//Note: Typically, DataDrawers.size() is equivalent to nrows * ncols, the number of map pixels
	//Note: DataDrawers vector size is determined when reading in the HydroPlusConfig.xml and other input data
	for (int DataDrawer_ID = 0; DataDrawer_ID < input->DataDrawers.size(); ++DataDrawer_ID) {
		//Create DataDrawer vector with DataFolder struct properties
		vector<DataFolder*> DataDrawer;
		//Initialize GI index for mapping contributing areas
		size_t gi_index = 0;
		//Loop through the DataFolders in each DataDrawer[DataDrawer_ID], upper bound indicated by .size()
		//Note: Typically, each DataDrawers[] has size 1, containing only 1 DataFolder
		//Note: DataDrawers[0] contains the first DataFolder in vector, which is the BulkArea in HydroPlusConfig.xml file
		for (int DataFolder_ID = 0; DataFolder_ID < input->DataDrawers[DataDrawer_ID].size(); ++DataFolder_ID) {
			//folderInfo map double created
			unordered_map<string, double> folderInfo = input->DataDrawers[DataDrawer_ID][DataFolder_ID];
			//folder is pointer to new DataFolder class for StatisticalHydro model
			DataFolder* folder = new DataFolder;
			//DataDrawer vector stores folder
			DataDrawer.push_back(folder);
			//ParamDict structure is defined for specific DataDrawers DataDrawer and DataFolder 
			folder->ParamDict = folderInfo;
			//ParamStringDict structure is defined for specific StringDataDrawers DataDrawer and DataFolder
			folder->ParamStringDict = input->StringDataDrawers[DataDrawer_ID][DataFolder_ID];

			//If 1st DataDrawer and 1st DataFolder then 
			if (DataDrawer_ID == 0 && DataFolder_ID == 0) {
				//Catchment_Type_Area_m2 initialized to size 2
				input->RepoVecDict["Catchment_Type_Area_m2"] = { 0,0 };
			}

			//If Type is BulkArea 
			if (folder->ParamStringDict["Type"] == "BulkArea") {
				//Folder_Type_ID defined as zero for BulkArea
				Folder_Type_ID = 0;
				if (input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
					//compute gi area and it;s contributing area, save to a vector
					double gi_area = input->BufferGIAreaCalc(gi_index);
					double gi_c_area = input->BufferGIContributingAreaCalc(gi_index);
					input->gi_area.push_back(gi_area);
					input->gi_c_area.push_back(gi_c_area);
					//compute the bulkarea
					folder->ParamDict["Area_m2"] = gi_c_area - gi_area;
				}
				//Catchment_Type_Area_m2 (m2) contains entire catchment area of this folder type
				input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID] += folder->ParamDict["Area_m2"];
			}
			//Else Type is not BulkArea and is likely GreenInfrastructure
			else {
				//Folder_Type_ID defined as one for GreenInfrastructure
				Folder_Type_ID = 1;
				if (input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
					//update the area of GI folder
					folder->ParamDict["Area_m2"] = input->BufferGIAreaCalc(gi_index);
					++gi_index; // Move to the next GI contributing area
				}
				//Catchment_Type_Area_m2 (m2) contains entire catchment area of this folder type
				input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID] += folder->ParamDict["Area_m2"];
				//Catchment_GIArea_m2 (m2) is increased by the GI Area_m2 (m2) for all devices in Folder
				input->RepoDict["Catchment_GIArea_m2"] += folder->ParamDict["Area_m2"];
				//adding GI type to GI_list (for SpatialBuffer)
				input->GI_list.push_back(folder->ParamStringDict["Type"]);
			}

			//StorageDeficit_VadoseZone_m (m) set as single reference value for entire catchment
			input->RepoDict["StorageDeficit_VadoseZone_m"] = input->AveSMD;

			//CheckHydroPlusConfigInputData function performs quality assurance on HydroPlusConfig.xml inputs for each folder
			CheckHydroPlusConfigInputData(folder, input, Directory_Input_CLArg);

			//AddFolderAreaStatisticalCoverData function determines land cover calculations for folder
			AddFolderAreaStatisticalCoverData(folder, input, DataDrawer_ID, DataFolder_ID, Folder_Type_ID);

			//Call Compute_CoverType_WeightedAverage_m2 function to compute weighted areas across catchment
			Compute_CoverType_WeightedAverage_m2(folder, input, DataDrawer_ID, DataFolder_ID, DataFolder_ID, Count_ArrayItem);

			//DataFolder_counter is advanced to loop through all DataDrawers and DataFolders
			DataFolder_counter = DataFolder_counter + 1;
			//add 1 to Count_ArrayItem 
			Count_ArrayItem = Count_ArrayItem + 1;
		}
		organizer->DataDrawers.push_back(DataDrawer);
	}
}

//AddFolderAreaSpatialCoverData function called for Model = SpatialTemperatureHydro
void BuildDataOrganizer::AddFolderAreaSpatialCoverData(DataFolder* folder, Inputs* input, int MapPixel_ID, int Folder_Type_ID)
{
	//MapPixel_ID defined by MapPixel_ID
	folder->ParamDict["MapPixel_ID"] = MapPixel_ID;
	//NLCDLandCoverData (#) is the NLCD class value.g. 11 = water, https://www.mrlc.gov/data/legends/national-land-cover-database-2019-nlcd2019-legend
	folder->ParamDict["LandCover_Class_NLCD"] = input->NLCDLandCoverData[MapPixel_ID];

	if (input->NLCDLandCoverData[MapPixel_ID] == input->LocationParams["NODATA"]) {
		folder->ParamStringDict["Type"] = "NODATA";
		return;
	}

	//TreeCanopyCover_overImpervious_frac (fraction) is derived from treecover.asc and imperviouscover.asc, only occurs if TC+IC > 1 
	folder->ParamDict["TreeCanopyCover_overImpervious_frac"] = input->TreeOnImpervious[MapPixel_ID];
	//TreeCanopyCover_overPervious_frac (fraction) is derived from treecover.asc and imperviouscover.asc, defaults to all TC 
	folder->ParamDict["TreeCanopyCover_overPervious_frac"] = input->TreeOnPervious[MapPixel_ID];

	//ShortVegCover_noTreeCanopy_frac (fraction) is derived when TC+IC < 1, and is function of NLCD value; see Inputs::calcNLCDLandCoverData
	folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] = input->ShortVegCover[MapPixel_ID];
	//SoilCover_noTreeCanopy_frac (fraction) is derived when TC+IC < 1, and is function of NLCD value; see Inputs::calcNLCDLandCoverData
	folder->ParamDict["SoilCover_noTreeCanopy_frac"] = input->SoilCover[MapPixel_ID];
	//PermeablePavementCover_noTreeCanopy_frac (fraction) is derived from HydroPlusConfig.xml and is not function of NLCD value
	//Note: Consider refactor to allow this option within the Inputs::calcNLCDLandCoverData function or set to zero 
	folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"] = input->PermeablePavementCover[MapPixel_ID];
	//WaterCover_noTreeCanopy_frac (fraction) is derived when TC+IC < 1, and is function of NLCD value; see Inputs::calcNLCDLandCoverData
	folder->ParamDict["WaterCover_noTreeCanopy_frac"] = input->WaterCover[MapPixel_ID];

	//ImperviousCover_noTreeCanopy_frac (fraction) is derived from treecover.asc and imperviouscover.asc, defaults to all IC 
	folder->ParamDict["ImperviousCover_noTreeCanopy_frac"] = input->ImperviousNoTree[MapPixel_ID];

	//ImperviousCover_frac (fraction) combines TreeCanopyCover_overImpervious_frac and ImperviousCover_noTreeCanopy_frac
	folder->ParamDict["ImperviousCover_frac"] = folder->ParamDict["TreeCanopyCover_overImpervious_frac"] + folder->ParamDict["ImperviousCover_noTreeCanopy_frac"];

	//PerviousCover_frac (fraction) combines TreeCanopyCover_overPervious_frac, ShortVegCover_noTreeCanopy_frac, SoilCover_noTreeCanopy_frac, PermeablePavementCover_noTreeCanopy_frac
	folder->ParamDict["PerviousCover_frac"] = folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"];

	//TreeCover_frac (fraction) combines TreeCanopyCover_overImpervious_frac, TreeCanopyCover_overPervious_frac
	folder->ParamDict["TreeCover_frac"] = folder->ParamDict["TreeCanopyCover_overImpervious_frac"] +
		folder->ParamDict["TreeCanopyCover_overPervious_frac"];

	//Section to scale soil terms based on vegetation cover based on this literature:
	//Aubertin, G. M. (1971). Nature and extent of macropores in forest soils and their influence on subsurface water movement (Vol. 192): Northeastern Forest Experiment Station. 
	//Endreny, T. A., Kwon, P., Williamson, T. N., & Evans, R. (2019). Reduced Soil Macropores and Forest Cover Reduce Warm-Season Baseflow below Ecological Thresholds in the Upper Delaware River Basin. Journal of the American Water Resources Association, 55(5), 1268-1287. doi:10.1111/1752-1688.12777
	//Harman, C. J., Lohse, K. A., Troch, P. A., & Sivapalan, M. (2014). Spatial patterns of vegetation, soils, and microtopography from terrestrial laser scanning on two semiarid hillslopes of contrasting lithology. Journal of Geophysical Research: Biogeosciences, 119(2), 163-180. doi:https://doi.org/10.1002/2013JG002507
	//Holdo, R. M., Nippert, J. B., & Mack, M. C. (2018). Rooting depth varies differentially in trees and grasses as a function of mean annual rainfall in an African savanna. Oecologia, 186(1), 269-280. doi:10.1007/s00442-017-4011-4
	//Pitt, R., Chen, S. E., Clark, S., Lantrip, J., Ong, C. K., & Voorhees, J. (2003). Infiltration Through Compacted Urban Soils and Effects on Biofiltration Design. In W. James (Ed.), Practical Modeling of Urban Water Systems (Vol. 11, pp. 217-252). Guelph, Ontario: CHI.
	//Rossman, L. A. (2015). Storm Water Management Model User's Manual Version 5.1. US Environmental Protection Agency. Cincinnati, HO. EPA/600/R-14/413b: 352. 
	//ASCE/WEF. (1992). Design and Construction of Urban Stormwater Management Systems. New York: American Society of Civil Engineers and the Water Environment Federation.

	//If sum of TreeCanopyCover_overPervious_frac, ShortVegCover_noTreeCanopy_frac, SoilCover_noTreeCanopy_frac, PermeablePavementCover_noTreeCanopy_frac > 0 then enter for division
	if ((folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] +	folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) > 0) {
		//Soil_Macropore_frac from HydroPlusConfig.xml is scaled values based on land cover as in Endreny et al. (2019), Aubertin (1971), Pitt et al. (2003)
		//Note: Soil_Macropore_frac value decreases from trees to short vegetation to soil, increases with age; not simulated below impervious area
		//Note: Refactor option to define Soil_Macropore_PermeablePavementCover_frac for PermeablePavementCover_noTreeCanopy_frac
		folder->ParamDict["Soil_Macropore_frac"] =
			(folder->ParamDict["Soil_Macropore_TreeCover_frac"] * folder->ParamDict["TreeCanopyCover_overPervious_frac"] +
				folder->ParamDict["Soil_Macropore_SVegCover_frac"] * folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] +
				folder->ParamDict["Soil_Macropore_SoilCover_frac"] * folder->ParamDict["SoilCover_noTreeCanopy_frac"] +
				folder->ParamDict["Soil_Macropore_SoilCover_frac"] * folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) /
			(folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] +
				folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]);
	}
	//Else If set to zero
	else { folder->ParamDict["Soil_Macropore_frac"] = 0.0; }

	//If sum of TreeCover_frac, ShortVegCover_noTreeCanopy_frac, SoilCover_noTreeCanopy_frac, PermeablePavementCover_noTreeCanopy_frac > 0 then enter for division
	if ((folder->ParamDict["TreeCover_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) > 0) {
		//Evapotranspiration_Depth_m from HydroPlusConfig.xml is scaled values based on land cover as in Holdo et al. (2018) and other studies
		//Note: Evapotranspiration_Depth_m value decreases from trees to short vegetation to soil, increases with age; simulated below impervious area
		//Note: Refactor Option: Create EvapotranspirationDepth_PermeablePavementCover_m for PermeablePavementCover_noTreeCanopy_frac 
		folder->ParamDict["Evapotranspiration_Depth_m"] =
			(folder->ParamDict["EvapotranspirationDepth_TreeCover_m"] * folder->ParamDict["TreeCover_frac"] + folder->ParamDict["EvapotranspirationDepth_SVegCover_m"] * folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["EvapotranspirationDepth_SoilCover_m"] * folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["EvapotranspirationDepth_SoilCover_m"] * folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) /	(folder->ParamDict["TreeCover_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]);
	}
	//Else If set to a default minimum value to avoid division by zero
	else { folder->ParamDict["Evapotranspiration_Depth_m"] = 0.0; }

	//If sum of TreeCanopyCover_overPervious_frac, ShortVegCover_noTreeCanopy_frac, SoilCover_noTreeCanopy_frac, PermeablePavementCover_noTreeCanopy_frac > 0 then enter for division
	if ((folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) > 0) {
		//PerviousDepressionStorage_mm from HydroPlusConfig.xml is scaled values based on land cover as in ASCE/WEF (1992), Rossman (2015) Table A5, Harman et al. (2014)
		//Note: PerviousDepressionStorage_mm value decreases from trees to short vegetation to soil, increases with age
		//Note: Refactor Option: Create PermeablePavementStorage_noTreeCover_mm for PermeablePavementCover_noTreeCanopy_frac
		folder->ParamDict["PerviousDepressionStorage_mm"] =
			(folder->ParamDict["PerviousDepressionStorage_TreeCover_mm"] * folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["PerviousDepressionStorage_SVegCover_mm"] * folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["PerviousDepressionStorage_SoilCover_mm"] * folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["ImperviousDepressionStorage_noTreeCover_mm"] * folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) / (folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]);
	}
	//Else If set to zero
	else { folder->ParamDict["PerviousDepressionStorage_mm"] = 0.0; }

	//If sum of TreeCanopyCover_overImpervious_frac and ImperviousCover_noTreeCanopy_frac > 0 then enter for division
	if ((folder->ParamDict["TreeCanopyCover_overImpervious_frac"] + folder->ParamDict["ImperviousCover_noTreeCanopy_frac"]) > 0) {
		//ImperviousDepressionStorage_mm from HydroPlusConfig.xml is scaled values based on land cover as in ASCE/WEF (1992), Rossman (2015) Table A5, Harman et al. (2014)
		//Note: ImperviousDepressionStorage_mm value increases with trees, due to leaf and organic litter, root generated undulations, increases w age
		folder->ParamDict["ImperviousDepressionStorage_mm"] =
			(folder->ParamDict["ImperviousDepressionStorage_noTreeCover_mm"] * folder->ParamDict["TreeCanopyCover_overImpervious_frac"] +
				folder->ParamDict["ImperviousDepressionStorage_TreeCover_mm"] * folder->ParamDict["ImperviousCover_noTreeCanopy_frac"]) /
			(folder->ParamDict["TreeCanopyCover_overImpervious_frac"] + folder->ParamDict["ImperviousCover_noTreeCanopy_frac"]);
	}
	//Else If set to zero
	else { folder->ParamDict["ImperviousDepressionStorage_mm"] = 0.0; }

	double DCIA_frac = 0;
	//If for BulkArea the PerviousCover_frac = 0, all impervious area drains to outlet and DCIA_frac must equal 1, despite HydroPlusConfig.xml inputs
	//If for GI Types, there are cases such as Roof Disconnect when PerviousCover_frac = 0 and DCIA should remain < 1
	if (folder->ParamDict["PerviousCover_frac"] <= 0 && folder->ParamStringDict["Type"] == "BulkArea") {
		//DCIA_frac = 1 forces all impervious area to drain to outlet given no pervious area
		DCIA_frac = 1;
	}
	else {
		//DCIA_frac = HydroPlusConfig.xml input, and allows either for some impervious area to drain to pervious area or GI
		DCIA_frac = input->ConnectedImpervious[MapPixel_ID];
	}

	//ImperviousCover_DrainsToOutlet_frac (fraction) is product of DCIA_frac and ImperviousCover_frac, including water and IC under TC
	folder->ParamDict["ImperviousCover_DrainsToOutlet_frac"] = DCIA_frac * folder->ParamDict["ImperviousCover_frac"];
	//ImperviousCover_DrainsToPervious_frac (fraction) is product of (1-DCIA_frac) and ImperviousCover_frac, including water and IC under TC
	folder->ParamDict["ImperviousCover_DrainsToPervious_frac"] = (1.0 - DCIA_frac) * folder->ParamDict["ImperviousCover_frac"];

	//Porosity_Drainable_m3pm3 (m3/m3) is gravitational water for the catchment, used to compute catchment average Depth_to_Groundwater_Table_m (m); constant for catchment 
	input->RefParams["Porosity_Drainable_m3pm3"] = input->RefParams["Soil_SaturationPoint_m3pm3"] - input->RefParams["Soil_FieldCapacity_m3pm3"];
	//StorageDeficit_FieldCapacity_SoilEvapZone_Max_m (m) is water available for evapotranspiration, not available for gravitational drainage, which can vary between folders across catchment
	folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_Max_m"] = folder->ParamDict["Evapotranspiration_Depth_m"] * (folder->ParamDict["Soil_FieldCapacity_m3pm3"] -
		folder->ParamDict["Soil_WiltingPoint_m3pm3"]);

	//If Soil_MoistureInitial_m3pm3 > Soil_FieldCapacity_m3pm3 then
	if (folder->ParamDict["Soil_MoistureInitial_m3pm3"] > folder->ParamDict["Soil_FieldCapacity_m3pm3"]) {
		//StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m (m) equals zero, soil moisture is beyond Soil_FieldCapacity_m3pm3 and no capacity remains for tension held water
		folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m"] = 0.0;
	}
	//Else Soil_MoistureInitial_m3pm3 (m3/m3) is less than Soil_FieldCapacity_m3pm3 then
	else {
		//StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m (m) equals Soil_FieldCapacity_m3pm3 - Soil_MoistureInitial_m3pm3 to compute tension held water available for evapotranspiration
		folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m"] = folder->ParamDict["Evapotranspiration_Depth_m"] * (folder->ParamDict["Soil_FieldCapacity_m3pm3"] - folder->ParamDict["Soil_MoistureInitial_m3pm3"]);
	}
	//Storage_SoilEvapZone_TS_0th_m (m) computed using HydroPlusConfig.xml inputs to obtain exact initial condition, not end of time step condition
	folder->VarDict["Storage_SoilEvapZone_TS_0th_m"] = (folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_Max_m"] - folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m"]);

	//If (Type is BulkArea) or (Type is GI and Flag_Exfiltration_to_Catchment equals 1), then enter to sum area draining to vadose zone shared by catchment
	//Note: Consider refactor for cases with GI, to add subsurface Vault storage area rather than surface Pervious cover area, given vault area can be larger than pervious area
	if ((folder->ParamStringDict["Type"] == "BulkArea") || (folder->ParamStringDict["Type"] != "BulkArea" && folder->ParamDict["Flag_Exfiltration_to_Catchment"] == 1)) {
		//Catchment_PerviousArea_AffectingStorageDeficit_m2 (m2) contains entire catchment pervious area draining to vadose zone
		//Note: Catchment_PerviousArea_AffectingStorageDeficit_m2 influences the catchment variable, StorageDeficit_VadoseZone_m
		input->RepoDict["Catchment_PerviousArea_AffectingStorageDeficit_m2"] += folder->ParamDict["Area_m2"] * folder->ParamDict["PerviousCover_frac"];
	}
	
	//Create parameters for catchment average land cover fraction for SpatialTemperatureHydro simulations
	//If MapPixel_ID == 0 then set all parameters to zero
	if (MapPixel_ID == 0) {

		input->RepoDict["catchment_LeafAreaIndex_Tree_m2m2"] = 0.0;
		input->RepoDict["catchment_LeafAreaIndex_SVeg_m2m2"] = 0.0;
	}

	double 	ratio_FolderArea_to_CatchmentArea = folder->ParamDict["Area_m2"] / input->SimulationNumericalParams["CatchmentArea_m2"];

	input->RepoDict["catchment_LeafAreaIndex_Tree_m2m2"] = input->RefParams["LeafAreaIndex_Tree_m2pm2"];
	input->RepoDict["catchment_LeafAreaIndex_SVeg_m2m2"] = input->RefParams["LeafAreaIndex_SVeg_m2pm2"];
}

//AddFolderAreaStatisticalCoverData function called for Model = StatisticalHydro
void BuildDataOrganizer::AddFolderAreaStatisticalCoverData(DataFolder* folder, Inputs* input, int DataDrawer_ID, int DataFolder_ID, int Folder_Type_ID)
{
	//If HydroPlusConfig.xml has DataFolder Type for GI then define RepoDict variables related to water transfer between GI and BulkArea
	if (folder->ParamStringDict["Type"] == "BioRetention" || folder->ParamStringDict["Type"] == "RainGarden" || folder->ParamStringDict["Type"] == "InfilTrench" ||
		folder->ParamStringDict["Type"] == "Swale" || folder->ParamStringDict["Type"] == "PermeablePavement" || folder->ParamStringDict["Type"] == "GreenRoof" ||
		folder->ParamStringDict["Type"] == "RoofDisconnect" || folder->ParamStringDict["Type"] == "RainBarrel") {

		//Flag_GI_Simulated flag set = 1, ensuring transfer of water between GI and BulkArea
		input->RepoDict["Flag_GI_Simulated"] = 1;
		//PerAreaTreated_frac (fraction) is fraction of BulkArea pervious cover that drains to GI
		input->RepoDict["BulkAreaPervious_DrainingToGI_frac"] = folder->ParamDict["PerAreaTreated_frac"];
		//ImpAreaTreated_frac (fraction) is fraction of BulkArea impervious cover that drains to GI
		input->RepoDict["BulkAreaImpervious_DrainingToGI_frac"] = folder->ParamDict["ImpAreaTreated_frac"];
		//BulkAreaWater_DrainingToGI_frac (fraction) is fraction of BulkArea water cover that drains to GI
		input->RepoDict["BulkAreaWater_DrainingToGI_frac"] = folder->ParamDict["WaterAreaTreated_frac"];
	}
	else {
		//Flag_GI_Simulated flag set = 0, ensuring no transfer of water between GI and BulkArea
		input->RepoDict["Flag_GI_Simulated"] = 0;
	}

	//ImperviousCover_frac (fraction) combines TreeCanopyCover_overImpervious_frac and ImperviousCover_noTreeCanopy_frac
	folder->ParamDict["ImperviousCover_frac"] = folder->ParamDict["TreeCanopyCover_overImpervious_frac"] + folder->ParamDict["ImperviousCover_noTreeCanopy_frac"];

	//PerviousCover_frac (fraction) combines TreeCanopyCover_overPervious_frac, ShortVegCover_noTreeCanopy_frac, SoilCover_noTreeCanopy_frac
	folder->ParamDict["PerviousCover_frac"] = folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"];

	//TreeCover_frac (fraction) combines TreeCanopyCover_overImpervious_frac, TreeCanopyCover_overPervious_frac
	folder->ParamDict["TreeCover_frac"] = folder->ParamDict["TreeCanopyCover_overImpervious_frac"] + folder->ParamDict["TreeCanopyCover_overPervious_frac"];

	//Section to scale soil terms based on vegetation cover based on this literature:
	//Aubertin, G. M. (1971). Nature and extent of macropores in forest soils and their influence on subsurface water movement (Vol. 192): Northeastern Forest Experiment Station. 
	//Endreny, T. A., Kwon, P., Williamson, T. N., & Evans, R. (2019). Reduced Soil Macropores and Forest Cover Reduce Warm-Season Baseflow below Ecological Thresholds in the Upper Delaware River Basin. Journal of the American Water Resources Association, 55(5), 1268-1287. doi:10.1111/1752-1688.12777
	//Harman, C. J., Lohse, K. A., Troch, P. A., & Sivapalan, M. (2014). Spatial patterns of vegetation, soils, and microtopography from terrestrial laser scanning on two semiarid hillslopes of contrasting lithology. Journal of Geophysical Research: Biogeosciences, 119(2), 163-180. doi:https://doi.org/10.1002/2013JG002507
	//Holdo, R. M., Nippert, J. B., & Mack, M. C. (2018). Rooting depth varies differentially in trees and grasses as a function of mean annual rainfall in an African savanna. Oecologia, 186(1), 269-280. doi:10.1007/s00442-017-4011-4
	//Pitt, R., Chen, S. E., Clark, S., Lantrip, J., Ong, C. K., & Voorhees, J. (2003). Infiltration Through Compacted Urban Soils and Effects on Biofiltration Design. In W. James (Ed.), Practical Modeling of Urban Water Systems (Vol. 11, pp. 217-252). Guelph, Ontario: CHI.
	//Rossman, L. A. (2015). Storm Water Management Model User's Manual Version 5.1. US Environmental Protection Agency. Cincinnati, HO. EPA/600/R-14/413b: 352. 
	//ASCE/WEF. (1992). Design and Construction of Urban Stormwater Management Systems. New York: American Society of Civil Engineers and the Water Environment Federation.

	//If sum of TreeCanopyCover_overPervious_frac, ShortVegCover_noTreeCanopy_frac, SoilCover_noTreeCanopy_frac, PermeablePavementCover_noTreeCanopy_frac > 0 then enter for division
	if ((folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] +	folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) > 0) {
		//Soil_Macropore_frac from HydroPlusConfig.xml is scaled values based on land cover as in Endreny et al. (2019), Aubertin (1971), Pitt et al. (2003)
		//Note: Soil_Macropore_frac value decreases from trees to short vegetation to soil, increases with age; not simulated below impervious area
		//Note: Refactor Option: Create Soil_Macropore_PermeablePavementCover_frac for PermeablePavementCover_noTreeCanopy_frac
		folder->ParamDict["Soil_Macropore_frac"] =
			(folder->ParamDict["Soil_Macropore_TreeCover_frac"] * folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["Soil_Macropore_SVegCover_frac"] * folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["Soil_Macropore_SoilCover_frac"] * folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["Soil_Macropore_SoilCover_frac"] * folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) / (folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]);
	}
	//Else If set to zero
	else { folder->ParamDict["Soil_Macropore_frac"] = 0.0; }

	//If sum of TreeCover_frac, ShortVegCover_noTreeCanopy_frac, SoilCover_noTreeCanopy_frac, PermeablePavementCover_noTreeCanopy_frac > 0 then enter for division
	if ((folder->ParamDict["TreeCover_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) > 0) {
		//Evapotranspiration_Depth_m from HydroPlusConfig.xml is scaled values based on land cover as in Holdo et al. (2018) and other studies
		//Note: Evapotranspiration_Depth_m value decreases from trees to short vegetation to soil, increases with age; simulated below impervious area
		//Note: Refactor Option: Create EvapotranspirationDepth_PermeablePavementCover_m for PermeablePavementCover_noTreeCanopy_frac
		folder->ParamDict["Evapotranspiration_Depth_m"] =
			(folder->ParamDict["EvapotranspirationDepth_TreeCover_m"] * folder->ParamDict["TreeCover_frac"] + folder->ParamDict["EvapotranspirationDepth_SVegCover_m"] * folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["EvapotranspirationDepth_SoilCover_m"] * folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["EvapotranspirationDepth_SoilCover_m"] * folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) / (folder->ParamDict["TreeCover_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]);
	}
	//Else If set to a default minimum value to avoid division by zero
	else { folder->ParamDict["Evapotranspiration_Depth_m"] = 0.0; }

	//If sum of TreeCanopyCover_overPervious_frac, ShortVegCover_noTreeCanopy_frac, SoilCover_noTreeCanopy_frac, PermeablePavementCover_noTreeCanopy_frac > 0 then enter for division
	if ((folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["SoilCover_noTreeCanopy_frac"] +	folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) > 0) {
		//PerviousDepressionStorage_mm from HydroPlusConfig.xml is scaled values based on land cover as in ASCE/WEF (1992), Rossman (2015) Table A5, Harman et al. (2014)
		//Note: PerviousDepressionStorage_mm value decreases from trees to short vegetation to soil, increases with age
		//Note: Refactor Option: Create PermeablePavementDepressionStorage_noTreeCover_mm for PermeablePavementCover_noTreeCanopy_frac
		folder->ParamDict["PerviousDepressionStorage_mm"] =
			(folder->ParamDict["PerviousDepressionStorage_TreeCover_mm"] * folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["PerviousDepressionStorage_SVegCover_mm"] * folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] + folder->ParamDict["PerviousDepressionStorage_SoilCover_mm"] * folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["ImperviousDepressionStorage_noTreeCover_mm"] * folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]) /	(folder->ParamDict["TreeCanopyCover_overPervious_frac"] + folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] +	folder->ParamDict["SoilCover_noTreeCanopy_frac"] + folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"]);
	}
	//Else If set to zero
	else { folder->ParamDict["PerviousDepressionStorage_mm"] = 0.0; }

	//If sum of TreeCanopyCover_overImpervious_frac and ImperviousCover_noTreeCanopy_frac > 0 then enter for division
	if ((folder->ParamDict["TreeCanopyCover_overImpervious_frac"] + folder->ParamDict["ImperviousCover_noTreeCanopy_frac"]) > 0) {
		//ImperviousDepressionStorage_mm from HydroPlusConfig.xml is scaled values based on land cover as in ASCE/WEF (1992), Rossman (2015) Table A5, Harman et al. (2014)
		//Note: ImperviousDepressionStorage_mm value increases with trees, due to leaf and organic litter, root generated undulations, increases w age
		folder->ParamDict["ImperviousDepressionStorage_mm"] =
			(folder->ParamDict["ImperviousDepressionStorage_noTreeCover_mm"] * folder->ParamDict["TreeCanopyCover_overImpervious_frac"] + folder->ParamDict["ImperviousDepressionStorage_TreeCover_mm"] * folder->ParamDict["ImperviousCover_noTreeCanopy_frac"]) / (folder->ParamDict["TreeCanopyCover_overImpervious_frac"] + folder->ParamDict["ImperviousCover_noTreeCanopy_frac"]);
	}
	//Else If set to zero
	else { folder->ParamDict["ImperviousDepressionStorage_mm"] = 0.0; }

	double DCIA_frac = 0;
	//If for BulkArea the PerviousCover_frac = 0, all impervious area drains to outlet and DCIA_frac must equal 1, despite HydroPlusConfig.xml inputs
	//If for GI Types, there are cases such as Roof Disconnect when PerviousCover_frac = 0 and DCIA should remain < 1
	if (folder->ParamDict["PerviousCover_frac"] <= 0 && folder->ParamStringDict["Type"] == "BulkArea") {
		//DCIA_frac = 1 forces all impervious area to drain to outlet given no pervious area
		DCIA_frac = 1;
	}
	else {
		//DCIA_frac = HydroPlusConfig.xml input, and allows either for some impervious area to drain to pervious area or GI
		DCIA_frac = folder->ParamDict["DirectlyConnectedImperviousArea_frac"];
	}

	//ImperviousCover_DrainsToOutlet_frac (fraction) is product of DCIA_frac and ImperviousCover_frac, including water and IC under TC
	folder->ParamDict["ImperviousCover_DrainsToOutlet_frac"] = DCIA_frac * folder->ParamDict["ImperviousCover_frac"];
	//ImperviousCover_DrainsToPervious_frac (fraction) is product of (1-DCIA_frac) and ImperviousCover_frac, including water and IC under TC
	folder->ParamDict["ImperviousCover_DrainsToPervious_frac"] = (1.0 - DCIA_frac) * folder->ParamDict["ImperviousCover_frac"];

	double lc_sum_frac = 0.0;
	double lc_gap_frac = 0.0;
	//lc_sum_frac is sum of ImperviousCover_frac, PerviousCover_frac, and WaterCover_noTreeCanopy_frac, which combine to account for all land cover
	//Note: This section will abort program if (ImperviousCover_frac + PerviousCover_frac + WaterCover_noTreeCanopy_frac) != 1
	lc_sum_frac = folder->ParamDict["ImperviousCover_frac"] + folder->ParamDict["PerviousCover_frac"] + folder->ParamDict["WaterCover_noTreeCanopy_frac"];
	//lc_gap_frac is absolute value of lc_sum_frac minus 1
	lc_gap_frac = abs(lc_sum_frac - 1.0);
	//If lc_gap_frac < Constant_1E_negative5 then lc_gap_frac equals zero
	if (lc_gap_frac < Constant_1E_negative5) {
		lc_gap_frac = 0.0;
	}

	//If lc_gap_frac > Constant_1E_negative5 then
	if (lc_gap_frac > Constant_1E_negative5) {
		//Explain why program aborted
		cout << endl << endl << "Program aborted due to " << folder->ParamStringDict["Type"] << " total land cover fraction != 1." << endl;
		//Determine gap in desired land cover fraction of 1.0
		lc_gap_frac = lc_sum_frac - 1;
		//Determine if the land cover has to be lowered or raised
		if (lc_gap_frac > 0) {
			cout << endl << "	Lower a " << folder->ParamStringDict["Type"] << " land cover value below by:	" << lc_gap_frac << endl << endl;
		}
		else {
			cout << endl << "	Raise a " << folder->ParamStringDict["Type"] << " land cover value below by:	" << abs(lc_gap_frac) << endl << endl;
		}
		//List all land cover types and their fractions
		cout << "	TreeCanopyCover_overPervious_frac:		" << folder->ParamDict["TreeCanopyCover_overPervious_frac"] << endl;
		cout << "	TreeCanopyCover_overImpervious_frac:		" << folder->ParamDict["TreeCanopyCover_overImpervious_frac"] << endl;
		cout << "	ShortVegCover_noTreeCanopy_frac:		" << folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] << endl;
		cout << "	SoilCover_noTreeCanopy_frac:			" << folder->ParamDict["SoilCover_noTreeCanopy_frac"] << endl;
		cout << "	PermeablePavementCover_noTreeCanopy_frac:	" << folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"] << endl;
		cout << "	ImperviousCover_noTreeCanopy_frac:		" << folder->ParamDict["ImperviousCover_noTreeCanopy_frac"] << endl;
		cout << "	WaterCover_noTreeCanopy_frac:			" << folder->ParamDict["WaterCover_noTreeCanopy_frac"] << endl;

		//List total land cover fraction
		cout << "	Total Land cover fraction:			" << lc_sum_frac << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//Porosity_Drainable_m3pm3 (m3/m3) is gravitational water for the catchment, used to compute catchment average Depth_to_Groundwater_Table_m (m); constant for catchment 
	input->RefParams["Porosity_Drainable_m3pm3"] = input->RefParams["Soil_SaturationPoint_m3pm3"] - input->RefParams["Soil_FieldCapacity_m3pm3"];
	//StorageDeficit_FieldCapacity_SoilEvapZone_Max_m (m) is water available for evapotranspiration, not available for gravitational drainage, which can vary between folders across catchment
	folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_Max_m"] = folder->ParamDict["Evapotranspiration_Depth_m"] * (folder->ParamDict["Soil_FieldCapacity_m3pm3"] - folder->ParamDict["Soil_WiltingPoint_m3pm3"]);
	//If Soil_MoistureInitial_m3pm3 > Soil_FieldCapacity_m3pm3 then
	if (folder->ParamDict["Soil_MoistureInitial_m3pm3"] > folder->ParamDict["Soil_FieldCapacity_m3pm3"]) {
		//StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m (m) equals zero, soil moisture is beyond Soil_FieldCapacity_m3pm3 and no capacity remains for tension held water
		folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m"] = 0.0;
	}
	//Else Soil_MoistureInitial_m3pm3 (m3/m3) is less than Soil_FieldCapacity_m3pm3 then
	else {
		//StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m (m) equals Soil_FieldCapacity_m3pm3 - Soil_MoistureInitial_m3pm3 to compute tension held water available for evapotranspiration
		folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m"] = folder->ParamDict["Evapotranspiration_Depth_m"] * (folder->ParamDict["Soil_FieldCapacity_m3pm3"] - folder->ParamDict["Soil_MoistureInitial_m3pm3"]);
	}
	//Storage_SoilEvapZone_TS_0th_m (m) computed using HydroPlusConfig.xml inputs to obtain exact initial condition, not end of time step condition
	folder->VarDict["Storage_SoilEvapZone_TS_0th_m"] = (folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_Max_m"] - folder->VarDict["StorageDeficit_FieldCapacity_SoilEvapZone_TS_first_m"]);

	//If (Type is BulkArea) or (Type is GI and Flag_Exfiltration_to_Catchment equals 1), then enter to sum area draining to vadose zone shared by catchment
	//Note: Consider refactor for cases with GI, to add subsurface Vault storage area rather than surface Pervious cover area, given vault area can be larger than pervious area
	if ((folder->ParamStringDict["Type"] == "BulkArea") || (folder->ParamStringDict["Type"] != "BulkArea" && folder->ParamDict["Flag_Exfiltration_to_Catchment"] == 1)) {
		//Catchment_PerviousArea_AffectingStorageDeficit_m2 (m2) contains entire catchment pervious area draining to vadose zone
		//Note: Catchment_PerviousArea_AffectingStorageDeficit_m2 influences the catchment variable, StorageDeficit_VadoseZone_m
		input->RepoDict["Catchment_PerviousArea_AffectingStorageDeficit_m2"] += folder->ParamDict["Area_m2"] * folder->ParamDict["PerviousCover_frac"];
	}

	//Create parameters for catchment average land cover fraction for StatisticalHydro simulations
	//If DataDrawer_ID == 0 && DataFolder_ID == 0 then set all parameters to zero
	if (DataDrawer_ID == 0 && DataFolder_ID == 0) {

		input->RepoDict["catchment_LeafAreaIndex_Tree_m2m2"] = 0.0;
		input->RepoDict["catchment_LeafAreaIndex_SVeg_m2m2"] = 0.0;
		input->RepoDict["catchment_TotalPerviousCover_frac"] = 0.0;
		input->RepoDict["catchment_TotalImperviousCover_frac"] = 0.0;
		input->RepoDict["catchment_TotalTreeCover_frac"] = 0.0;
		input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] = 0.0;
		input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"] = 0.0;
		input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] = 0.0;
		input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"] = 0.0;
		input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] = 0.0;
		input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] = 0.0;
		input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"] = 0.0;
		input->RepoDict["catchment_DCIA_frac"] = 0.0;
		input->RepoDict["catchment_InfiltExcessGovernedArea_frac"] = 0.0;
		input->RepoDict["catchment_MacroPore_frac"] = 0.0;
	}
	//update CatchmentArea_m2 for SpatialBuffer with GI
	if (input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
			input->SimulationNumericalParams["CatchmentArea_m2"] = input->RepoVecDict["Catchment_Type_Area_m2"][0] + input->RepoVecDict["Catchment_Type_Area_m2"][1];
	}
	double 	ratio_FolderArea_to_CatchmentArea = folder->ParamDict["Area_m2"] / input->SimulationNumericalParams["CatchmentArea_m2"];

	input->RepoDict["catchment_LeafAreaIndex_Tree_m2m2"] = input->RefParams["LeafAreaIndex_Tree_m2pm2"];
	input->RepoDict["catchment_LeafAreaIndex_SVeg_m2m2"] = input->RefParams["LeafAreaIndex_SVeg_m2pm2"];
	input->RepoDict["catchment_TotalPerviousCover_frac"] += folder->ParamDict["PerviousCover_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_TotalImperviousCover_frac"] += folder->ParamDict["ImperviousCover_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_TotalTreeCover_frac"] += folder->ParamDict["TreeCover_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_TreeCanopyCover_overPervious_frac"] += folder->ParamDict["TreeCanopyCover_overPervious_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_TreeCanopyCover_overImpervious_frac"] += folder->ParamDict["TreeCanopyCover_overImpervious_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_ImperviousCover_noTreeCanopy_frac"] += folder->ParamDict["ImperviousCover_noTreeCanopy_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_WaterCover_noTreeCanopy_frac"] += folder->ParamDict["WaterCover_noTreeCanopy_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_ShortVegCover_noTreeCanopy_frac"] += folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_SoilCover_noTreeCanopy_frac"] += folder->ParamDict["SoilCover_noTreeCanopy_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_PermeablePavementCover_noTreeCanopy_frac"] += folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_DCIA_frac"] += folder->ParamDict["DirectlyConnectedImperviousArea_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_InfiltExcessGovernedArea_frac"] += folder->ParamDict["InfiltExcessGovernedArea_frac"] * ratio_FolderArea_to_CatchmentArea;
	input->RepoDict["catchment_MacroPore_frac"] += folder->ParamDict["Soil_Macropore_frac"] * ratio_FolderArea_to_CatchmentArea;
}


void BuildDataOrganizer::CheckHydroPlusConfigInputData(DataFolder* folder, Inputs* input, string Directory_Input_CLArg)
{
	//If HydroPlusConfig Type is not BulkArea or another known Type, then abort program
	if ((folder->ParamStringDict["Type"] != "BulkArea") && (folder->ParamStringDict["Type"] != "RainGarden") &&
		(folder->ParamStringDict["Type"] != "BioRetention") && (folder->ParamStringDict["Type"] != "GreenRoof") &&
		(folder->ParamStringDict["Type"] != "RainBarrel") && (folder->ParamStringDict["Type"] != "PermeablePavement") &&
		(folder->ParamStringDict["Type"] != "Swale") && (folder->ParamStringDict["Type"] != "InfilTrench") &&
		(folder->ParamStringDict["Type"] != "RoofDisconnect")) {

		cout << "Warning: The HydroPlusConfig.xml has a variable for the Type element not known to the HydroPlus model." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When i-Tree Hydro model does not find a known Type, it cannot allocate a water balance." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, set the 1st DataFolder Type = BulkArea, and for any GI folders to Type: BioRetention, GreenRoof, IniltTrench, PermeablePavement, RainGarden, RainBarrel, RoofDisconnect, or Swale." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//If BulkArea T0_m2ph and Q0_mph are less than zero, then abort program
	//Note: Consider refactor to place these terms in SimulationNumericalParams for catchment to have single value 
	if (folder->ParamStringDict["Type"] == "BulkArea" && (input->RefParams["Discharge_Subsurface_Initial_mph"] <= 0 || 
		input->RefParams["VadoseZone_Transmissivity_Max_m2ph"] <= 0 || input->RefParams["Parameter_m_KsatExpDecay"] <= 0 || 
		input->RefParams["Parameter_n_KsatPowerDecay"] <= 0)) {
		cout << "Warning: HydroPlusConfig.xml has values <= 0 Discharge_Subsurface_Initial_mph, VadoseZone_Transmissivity_Max_m2ph, Parameter_m_KsatExpDecay, or Parameter_n_KsatPowerDecay." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When any of these parameters are zero or less there is unrealistic subsurface flow in the catchment." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, modify parameters such that they are greater than zero." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//Note: Control HydroPlusConfig.xml has DataFolder with Area_m2 = 0 or Type not BulkArea and Count_GI_Units = 0 then model will abort
	if (input->SimulationNumericalParams["CatchmentArea_m2"] <= 0.0 || folder->ParamDict["Area_m2"] <= 0.0 || folder->ParamStringDict["Type"] != "BulkArea" && folder->ParamDict["Count_GI_Units"] <= 0.0) {
		if (input->SimulationStringParams["Model_Selection"] != "SpatialBufferwGI") {
			cout << "Warning: HydroPlusConfig.xml has parameter CatchmentArea_m2 = 0 m2, Area_m2 = 0 m2, or Area_m2 * Count_GI_Units = 0 m2." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the HydroPlusConfig.xml parameters provide no CatchmentArea_m2, Area_m2, or Area_m2 * Count_GI_Units it cannot balance water." << endl;
			cout << "Correction: In the HydroPlusConfig.xml file, modify parameters such that CatchmentArea_m2 and Area_m2 are > 0." << endl;
			cout << "Correction: In the HydroPlusConfig.xml file, if using green infrastructure, then ensure Count_GI_Units > 0." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
	}

	//Note: Control HydroPlusConfig.xml has DataFolder Type for GI with keys Surface_Berm_Height_m >0 & Surface_Porosity_m3pm3 = 0, then model will abort
	//Note: May need to relax this constraint for GI types without the use of surface layer Surface_Berm_Height_m
	if (folder->ParamStringDict["Type"] != "BulkArea" && folder->ParamDict["Surface_Berm_Height_m"] > 0.0 && folder->ParamDict["Surface_Porosity_m3pm3"] <= 0.0) {
		cout << "Warning: HydroPlusConfig.xml contains DataFolder Type for Green Infrastructure (GI), but parameter Surface_Berm_Height_m > 0 and parameter Surface_Porosity_m3pm3 = 0." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the HydroPlusConfig.xml parameter Surface_Porosity_m3pm3 = 0, there are no voids in the GI layer to receive runoff." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, either remove the DataFolder Type GI, or modify parameters such that Surface_Porosity_m3pm3 > 0." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//Note: Control HydroPlusConfig.xml has DataFolder Type for GI with keys Soil_Thickness_m > 0 & soil porosity = 0, then model will abort
	//Note: This control is addressed by control in this section ensuring Soil_SaturationPoint_m3pm3 < Soil_WiltingPoint_m3pm3

	//Note: Control HydroPlusConfig.xml has DataFolder Type for GI with keys Vault_Thickness_m > 0 & Vault_Porosity_m3pm3 = 0, then model will abort
	if (folder->ParamStringDict["Type"] != "BulkArea" && folder->ParamDict["Vault_Thickness_m"] > 0 && folder->ParamDict["Vault_Porosity_m3pm3"] <= 0.0) {
		cout << "Warning: HydroPlusConfig.xml contains DataFolder Type for Green Infrastructure (GI) with parameter Vault_Thickness_m > 0, but parameter Vault_Porosity_m3pm3 = 0." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the HydroPlusConfig.xml parameter Vault_Thickness_m > 0 and Vault_Porosity_m3pm3 = 0, there are no voids in the GI layer to receive runoff." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, either remove the DataFolder Type GI, or modify parameters such that Vault_Thickness_m = 0 or Vault_Porosity_m3pm3 > 0." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//Note: Control HydroPlusConfig.xml has DataFolder Type for GI PermeablePavement with keys Pavement_Thickness_m > 0 & Pavement_Porosity_m3pm3 = 0, then model will abort
	if (folder->ParamStringDict["Type"] == "PermeablePavement") {
		if (folder->ParamDict["Pavement_Thickness_m"] <= 0 || folder->ParamDict["Pavement_Porosity_m3pm3"] <= 0 ||
			folder->ParamDict["Pavement_Drainage_Limit_m"] <= 0) {
			cout << "Warning: HydroPlusConfig.xml contains DataFolder Type for Permeable Pavement with parameters Pavement_Thickness_m, Pavement_Porosity_m3pm3, or Pavement_Drainage_Limit_m = 0." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the HydroPlusConfig.xml parameter Pavement_Thickness_m, Pavement_Porosity_m3pm3, or Pavement_Drainage_Limit_m = 0, the model cannot balance water." << endl;
			cout << "Correction: In the HydroPlusConfig.xml file, modify parameters such that Pavement_Thickness_m, Pavement_Porosity_m3pm3, and Pavement_Drainage_Limit_m > 0." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
		//InfiltExcessGovernedArea_frac defined to 1 for PermeablePavement to ensure GreenAmpt routine used with Pavement_HydraulicConductivity_mph
		//Note: Pavement_HydraulicConductivity_mph is reduced by clogging, and can go to zero; if InfiltExcessGovernedArea_frac = 0 this constraint bypassed
		folder->ParamDict["InfiltExcessGovernedArea_frac"] = 1;
	}

	//If Control HydroPlusConfig.xml has Type GreenRoof, InfilTrench, RainBarrel, RoofDisconnect, but Vault_Thickness_m or Vault_Porosity_m3pm3 <= 0, then model will abort
	if (folder->ParamStringDict["Type"] == "GreenRoof" || folder->ParamStringDict["Type"] == "InfilTrench" ||
		folder->ParamStringDict["Type"] == "RainBarrel" || folder->ParamStringDict["Type"] == "RoofDisconnect") {
		//Note: Control HydroPlusConfig.xml has DataFolder Type for GI with storage layer and keys Vault_Thickness_m <= 0, then model will abort
		if (folder->ParamDict["Vault_Thickness_m"] <= 0 || folder->ParamDict["Vault_Porosity_m3pm3"] <= 0.0) {
			cout << "Warning: HydroPlusConfig.xml contains DataFolder Type " << folder->ParamStringDict["Type"] << "with parameter Vault_Thickness_m or parameter Vault_Porosity_m3pm3 = 0." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the HydroPlusConfig.xml parameter Vault_Thickness_m or Vault_Porosity_m3pm3 = 0, the model cannot balance water." << endl;
			cout << "Correction: In the HydroPlusConfig.xml file, modify parameters such that Vault_Thickness_m and Vault_Porosity_m3pm3 > 0." << endl;
			if (folder->ParamStringDict["Type"] == "RoofDisconnect") { cout << "... Vault layer receives discharge from downspout." << endl; }
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
	}

	//Note: Control HydroPlusConfig.xml parameters for Swale; swales are expected to have Surface_Outlet_BottomWidth_m > 0 for drainage 
	if (folder->ParamStringDict["Type"] == "Swale") {
		//Surface_BottomWidth_Min_m (m) set to a minimum surface width, overriding HydroPlusConfig.xml inputs; if used create cout statement
		double Surface_BottomWidth_Min_m = 0.05;
		if (folder->ParamDict["Surface_Outlet_BottomWidth_m"] < Surface_BottomWidth_Min_m) {
			cout << "Warning: For GI device Swale the HydroPlusConfig.xml input value of Surface_Outlet_BottomWidth_m was changed." << endl;
			cout << "Warning adjustment: Surface_Outlet_BottomWidth_m was increased from " << folder->ParamDict["Surface_Outlet_BottomWidth_m"] << " to " << Surface_BottomWidth_Min_m << "." << endl;
			cout << "Warning explained: The GI device Swale should have a Surface_Outlet_BottomWidth_m > 0 to allow outflow." << endl;

			//Surface_Outlet_BottomWidth_m for Swale is redefined to maximum of HydroPlusConfig.xml input or Surface_BottomWidth_Min_m to allow outflow
			folder->ParamDict["Surface_Outlet_BottomWidth_m"] = MAX(folder->ParamDict["Surface_Outlet_BottomWidth_m"], Surface_BottomWidth_Min_m);
		}
	}

	//Note: Control HydroPlusConfig.xml parameter for exfiltration to catchment soils and possible upwelling of groundwater
	if (folder->ParamStringDict["Type"] != "BulkArea" && folder->ParamDict["Flag_Exfiltration_to_Catchment"] != 0 &&
		folder->ParamDict["Flag_Exfiltration_to_Catchment"] != 1) {
		cout << "Warning: HydroPlusConfig.xml contains an illogical Flag_Exfiltration_to_Catchment parameter." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the HydroPlusConfig.xml Flag_Exfiltration_to_Catchment parameter is illogical, the model cannot balance water." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, ensure Flag_Exfiltration_to_Catchment is 0 or 1." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//Note: Control HydroPlusConfig.xml parameter for exfiltration to catchment soils and possible decay of exfiltration rate
	if (folder->ParamStringDict["Type"] != "BulkArea" && folder->ParamDict["Flag_Exfiltration_to_Catchment"] == 1 &&
		folder->ParamDict["Exfiltration_Limit_m"] <= 0) {
		cout << "Warning: HydroPlusConfig.xml contains an illogical Exfiltration_Limit_m parameter." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the HydroPlusConfig.xml Flag_Exfiltration_to_Catchment = 1 and Exfiltration_Limit_m parameter = 0, the model cannot balance water." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, ensure Exfiltration_Limit_m is > 0." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//Note: Control HydroPlusConfig.xml parameters for soil moisture, ensuring they are in proper sequence
	//Note: Consider refactor to allow Soil_WiltingPoint_m3pm3 > Soil_MoistureInitial_m3pm3, updating calculation of Storage_GI_Soil_m3, Storage_GI_Soil_Max_m3, Storage_GI_Soil_Max_Potential_m3
	//Note: Users can set Soil_WiltingPoint_m3pm3 to 0 to allow Soil_MoistureInitial_m3pm3 to reach 0
	if (folder->ParamDict["Soil_WiltingPoint_m3pm3"] > folder->ParamDict["Soil_FieldCapacity_m3pm3"] ||
		folder->ParamDict["Soil_FieldCapacity_m3pm3"] > folder->ParamDict["Soil_SaturationPoint_m3pm3"] ||
		folder->ParamDict["Soil_MoistureInitial_m3pm3"] > folder->ParamDict["Soil_SaturationPoint_m3pm3"] ||
		folder->ParamDict["Soil_WiltingPoint_m3pm3"] > folder->ParamDict["Soil_MoistureInitial_m3pm3"]) {
			
		cout << "Warning: HydroPlusConfig.xml contains illogical soil moisture parameters." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the HydroPlusConfig.xml soil moisture parameters are illogical, the model cannot balance water." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, ensure Soil_WiltingPoint_m3pm3 < Soil_FieldCapacity_m3pm3 < Soil_SaturationPoint_m3pm3 and Soil_WiltingPoint_m3pm3 <= Soil_MoistureInitial_m3pm3 <= Soil_SaturationPoint_m3pm3." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	//If BlockGoupData is zero and Flag_BlockGroupsProcessed is one or StartingBlockIdsToPrintTempOutput > 0 then missing blockgroup.asc
	if (input->BlockGoupData.size() <= 0 && (input->Flag_BlockGroupsProcessed == 1 || input->StartingBlockIdsToPrintTempOutput.size() > 0)) {
		cout << "Warning: HydroPlusConfig.xml element PrintBlockGroupRange# has values but input folder has no blockgroup.asc." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the HydroPlusConfig.xml indicates block group analysis, the model needs a blockgroup.asc." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, remove the PrintBlockGroupRange# element, or provide a blockgroup.asc." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	//If HydroPlusConfig.xml parameter Flag_CompareToObservedRunoff is not set to NoCalibration, then check if Runoff_Observed_for_Calibration.csv is provided
	//Note: HydroPlusConfig.xmls element Flag_CompareToObservedRunoff can equal 0, 1 (routed runoff), or 2 (unrouted runoff)
	if (input->SimulationStringParams["Flag_CompareToObservedRunoff"] != "0") {
		//Qobs_file created as string combination of Directory_Input_CLArg + "Runoff_Observed_for_Calibration.csv, where double backslash needed to get beyond escape character
		string Qobs_file = Directory_Input_CLArg + "Runoff_Observed_for_Calibration.csv";
		//readQobs created as part of fstream, defined as the Directory_Input_CLArg and file name for Runoff_Observed_for_Calibration.csv
		ifstream readQobs(Qobs_file);
		//if readQobs does not exist, alert user
		if (!readQobs) {
			cout << "Warning: The input file Runoff_Observed_for_Calibration.csv is missing." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: Without observed flow data, the model cannot calibrate." << endl;
			cout << "Correction: Provide a Runoff_Observed_for_Calibration.csv file or set the HydroPlusConfig.xml element Flag_CompareToObservedRunoff = 0." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
	}

	//If Height_Sensor_Wind_m is < zero then working with unusual data
	if (input->RefParams["Height_Sensor_Wind_m"] < 2) {
		cout << "Warning: HydroPlusConfig.xml element Height_Sensor_Wind_m is not appropriate." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model is designed for Height_Sensor_Wind_m of 2 to 10 m." << endl;
		cout << "Correction: Provide a reasonable Height_Sensor_Wind_m value in the DataFolder." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//If Height_Sensor_Wind_m is < zero then working with unusual data
	if (input->RefParams["Height_Avg_TreeCover_m"] <= 0 || input->RefParams["Height_Avg_SVegCover_m"] <= 0) {
		cout << "Warning: HydroPlusConfig.xml elements Height_Avg_TreeCover_m and/or Height_Avg_SVegCover_m are <= 0." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model requires Height_Avg_TreeCover_m and Height_Avg_SVegCover_m > 0." << endl;
		cout << "Correction: Set Height_Avg_TreeCover_m and Height_Avg_SVegCover_m > 0 in the DataFolder." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
}

//Compute_CoverType_WeightedAverage_m2 function computes GI variable weighted averages for areas
//Note: Area weighted averages require use of folder fractional terms and Area_m2, which can be computed during initial loop through folders
void BuildDataOrganizer::Compute_CoverType_WeightedAverage_m2(DataFolder* folder, Inputs* input, int DataDrawer_ID, int DataFolder_ID, int MapPixel_ID, int Count_ArrayItem)
{
	//If Count_ArrayItem equals zero then at first data folder and initialize vector size to 2
	if (Count_ArrayItem == 0) {
		input->RepoVecDict["TotalPerviousCover_Type_m2"] = { 0,0 };
		input->RepoVecDict["TotalImperviousCover_Type_m2"] = { 0,0 };
		input->RepoVecDict["TotalTreeCover_Type_m2"] = { 0,0 };
		input->RepoVecDict["TreeCanopyCover_overPervious_Type_m2"] = { 0,0 };
		input->RepoVecDict["TreeCanopyCover_overImpervious_Type_m2"] = { 0,0 };
		input->RepoVecDict["ImperviousCover_noTreeCanopy_Type_m2"] = { 0,0 };
		input->RepoVecDict["WaterCover_noTreeCanopy_Type_m2"] = { 0,0 };
		input->RepoVecDict["ShortVegCover_noTreeCanopy_Type_m2"] = { 0,0 };
		input->RepoVecDict["SoilCover_noTreeCanopy_Type_m2"] = { 0,0 };
		input->RepoVecDict["PermeablePavementCover_noTreeCanopy_Type_m2"] = { 0,0 };
		input->RepoVecDict["DCIA_Type_m2"] = { 0,0 };
		input->RepoVecDict["InfiltExcessGovernedArea_Type_m2"] = { 0,0 };
		//MacroPore_Type_m2 (m2) is converted to MacroPore_Type_frac in WaterBalance_Summary.cpp
		input->RepoVecDict["MacroPore_Type_m2"] = { 0,0 };
	}

	double Folder_Area_m2 = folder->ParamDict["Area_m2"];
	//Folder_Type_ID will track Folder as BulkArea of GreenInfrastructure
	int Folder_Type_ID;
	//If HydroPlusCong.xml element Type is BulkArea or Model is SpatialTemperatureHydro then
	if (folder->ParamStringDict["Type"] == "BulkArea" || input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {
		//Folder_Type_ID defined as zero for BulkArea
		Folder_Type_ID = 0;
	}
	//Else HydroPlusCong.xml element Type is not BulkArea
	else {
		//Folder_Type_ID defined as one for GreenInfrastructure
		Folder_Type_ID = 1;
	}
	//TotalPerviousCover_Type_m2 (m2) += PerviousCover_frac (frac) * Folder_Area_m2 (m2); accumulating the amount in each folder
	//Note: RepoVecDict variable contains catchment total of Folder_Type_ID specific variables, e.g., BulkArea or GreenInfrastructure
	input->RepoVecDict["TotalPerviousCover_Type_m2"][Folder_Type_ID] += folder->ParamDict["PerviousCover_frac"] * Folder_Area_m2;
	input->RepoVecDict["TotalImperviousCover_Type_m2"][Folder_Type_ID] += folder->ParamDict["ImperviousCover_frac"] * Folder_Area_m2;
	input->RepoVecDict["TotalTreeCover_Type_m2"][Folder_Type_ID] += folder->ParamDict["TreeCover_frac"] * Folder_Area_m2;
	input->RepoVecDict["TreeCanopyCover_overPervious_Type_m2"][Folder_Type_ID] += folder->ParamDict["TreeCanopyCover_overPervious_frac"] * Folder_Area_m2;
	input->RepoVecDict["TreeCanopyCover_overImpervious_Type_m2"][Folder_Type_ID] += folder->ParamDict["TreeCanopyCover_overImpervious_frac"] * Folder_Area_m2;
	input->RepoVecDict["ImperviousCover_noTreeCanopy_Type_m2"][Folder_Type_ID] += folder->ParamDict["ImperviousCover_noTreeCanopy_frac"] * Folder_Area_m2;
	input->RepoVecDict["WaterCover_noTreeCanopy_Type_m2"][Folder_Type_ID] += folder->ParamDict["WaterCover_noTreeCanopy_frac"] * Folder_Area_m2;
	input->RepoVecDict["ShortVegCover_noTreeCanopy_Type_m2"][Folder_Type_ID] += folder->ParamDict["ShortVegCover_noTreeCanopy_frac"] * Folder_Area_m2;
	input->RepoVecDict["SoilCover_noTreeCanopy_Type_m2"][Folder_Type_ID] += folder->ParamDict["SoilCover_noTreeCanopy_frac"] * Folder_Area_m2;
	input->RepoVecDict["PermeablePavementCover_noTreeCanopy_Type_m2"][Folder_Type_ID] += folder->ParamDict["PermeablePavementCover_noTreeCanopy_frac"] * Folder_Area_m2;
	input->RepoVecDict["DCIA_Type_m2"][Folder_Type_ID] += folder->ParamDict["DirectlyConnectedImperviousArea_frac"] * Folder_Area_m2;
	input->RepoVecDict["InfiltExcessGovernedArea_Type_m2"][Folder_Type_ID] += folder->ParamDict["InfiltExcessGovernedArea_frac"] * Folder_Area_m2;
	input->RepoVecDict["MacroPore_Type_m2"][Folder_Type_ID] += folder->ParamDict["Soil_Macropore_frac"] * Folder_Area_m2;
}

//Compute_CoverType_WeightedAverage_frac function computes GI variable weighted averages for fractions
//Note: Fractional weighted averages require use of Catchment_Type_Area_m2, which is first created by summing Area_m2 for all folders
void BuildDataOrganizer::Compute_CoverType_WeightedAverage_frac(DataOrganizer* organizer, Inputs* input)
{


	string Name_Removal_Rate_Pollutant_frac;
	string Name_Removal_Rate_Pollutant_percent;
	//For Loop through pollutant names to write output file header; length of For Loop is Pollutant_Name.size()
	//Note: Pollutant_Name vector used below are from Pollutants.csv, read in Inputs.cpp into vector 
	for (int pollutant_ID = 0; pollutant_ID < input->Pollutant_Name.size(); pollutant_ID++) {
		//Create string to represent variable associated with Pollutant_Name
		Name_Removal_Rate_Pollutant_frac = "Removal_Rate_" + input->Pollutant_Name[pollutant_ID] + "_Avg_frac";
		//Name_Removal_Rate_Pollutant_frac reset to zero at start of simulation, MapPixel_ID equal to zero
		input->RepoDict[Name_Removal_Rate_Pollutant_frac] = 0;
	}

	//Loop through the DataDrawers, where the upper bound is based on the general DataDrawer attribute .size()
	//Note: DataDrawers[0] is first DataFolder in vector, which is the BulkArea in HydroPlusConfig.xml file
	//Note: DataDrawers vector size is determined when reading in the HydroPlusConfig.xml and other input data
	for (int DataDrawer_ID = 0; DataDrawer_ID < input->DataDrawers.size(); ++DataDrawer_ID) {
		//Create DataDrawer vector with DataFolder struct properties
		vector<DataFolder*> DataDrawer;

		//Loop through the DataFolders in each DataDrawer[DataDrawer_ID], upper bound indicated by .size()
		for (auto DataFolder_ID = 0; DataFolder_ID < input->DataDrawers[DataDrawer_ID].size(); ++DataFolder_ID) {
			//folder is pointer to organizer->DataDrawers[DataDrawer_ID][DataFolder_ID]
			DataFolder* folder = organizer->DataDrawers[DataDrawer_ID][DataFolder_ID];

			//If HydroPlusConfig.xml Type not equal to BulkArea then prepare GreenInfrastructure Name_Removal_Rate_Pollutant_frac
			if (folder->ParamStringDict["Type"] != "BulkArea" || input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {
				//Folder_Type_ID defined as one for GreenInfrastructure
				int Folder_Type_ID = 1;
				//Ratio_FolderTypeArea_to_CatchmentTypeArea (fraction) is ratio of folder Type Area_m2 and catchment area of folders with that Type
				double Ratio_FolderTypeArea_to_CatchmentTypeArea = folder->ParamDict["Area_m2"] / input->RepoVecDict["Catchment_Type_Area_m2"][Folder_Type_ID];
				//For Loop through pollutant names to write output file header; length of For Loop is Pollutant_Name.size()
				//Note: Pollutant_Name vector used below are from Pollutants.csv, read in Inputs.cpp into vector 
				for (int pollutant_ID = 0; pollutant_ID < input->Pollutant_Name.size(); pollutant_ID++) {
					//Create string to represent variable associated with Pollutant_Name
					Name_Removal_Rate_Pollutant_frac = "Removal_Rate_" + input->Pollutant_Name[pollutant_ID] + "_Avg_frac";
					//Create string to represent variable associated with Pollutant_Name
					Name_Removal_Rate_Pollutant_percent = input->Pollutant_Name[pollutant_ID] + "_RemovalEfficiency_percent";
					//Removal_Rate_Pollutant_frac (fraction) is pollutant removal rate (%) multiplied by Ratio_Decimal_to_Percent; removal rate from HydroPlusConfig.xml
					double Removal_Rate_Pollutant_frac = folder->ParamDict[Name_Removal_Rate_Pollutant_percent] * Ratio_Decimal_to_Percent;
					//RepoDict variable Name_Removal_Rate_Pollutant_frac is sum of Removal_Rate_Pollutant_frac * ratio_GIArea_to_CatchmentGIArea
					input->RepoDict[Name_Removal_Rate_Pollutant_frac] += Removal_Rate_Pollutant_frac * Ratio_FolderTypeArea_to_CatchmentTypeArea;
				}
			}
		}
	}
}

//BuildDataOrganizer::SetGIMissingParams function is for Folder Type other than BulkArea 
void BuildDataOrganizer::SetGIMissingParams(DataOrganizer *organizer, Inputs *input)
{ 
	//If Model is not SpatialTemperatureHydro or StatisticalHydro, then use C++ return function to leave SetGIMissingParams function
	if (input->SimulationStringParams["Model_Selection"] == "CoolRiver") return;
	if (input->SimulationStringParams["Model_Selection"] == "ECDynamic") return;
	if (input->SimulationStringParams["Model_Selection"] == "SpatialBuffer"|| input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") return;

	//For loop with DataFolder_ID incrementing through all DataDrawer[0].size, which is the number of folders in the 1st DataDrawer
	for (int DataFolder_ID = 0; DataFolder_ID < organizer->DataDrawers[0].size(); ++DataFolder_ID) { 

		//giFolder is pointer for DataFolder to organizer->DataDrawers[0][DataFolder_ID]
		DataFolder* giFolder = organizer->DataDrawers[0][DataFolder_ID];	
		
		//If giFolder ParamStringDict["Type"] is BulkArea then use C++ continue function to return to the for loop and not proceed to next statement
		if (giFolder->ParamStringDict["Type"] == "BulkArea") {
			//continue function will return to next item in upper for loop
			continue;
		}
		
		//Note: If program arrives here then Type is not BulkArea

		//For loop using const auto &myPair address through all input->RefParams values, which are all Reference Folder HydroPlusConfig.xml elements 
		//Note: &myPair is the address for the input->RefParams read from the HydroPlusConfig.xml for the Feference Folder
		//Note: HydroPlusConfig element RefParamFolder defines the location of the Reference Folder, which is typically the 1st BulkArea 
		for (const auto &myPair : input->RefParams) {

			//If giFolder->ParamDict.find(myPair.first) is equal to giFolder->ParamDict.end(), then myPair.first is empty or has no value
			//Note: When myPair.first is zero, the non-Reference folder does not have a reference folder element
			//Note: giFolder->ParamDict.find(myPair.first) references all the HydroPlusConfig.xml elements assigned to the Reference Folder
			if (giFolder->ParamDict.find(myPair.first) == giFolder->ParamDict.end()) {

				//giFolder->ParamDict[myPair.first] is created, defined as myPair.second, which is the value of the Reference Folder
				//Note: Values assigned by myPair.second are the element values in the HydroPlusConfig.xml for the Reference Folder
				giFolder->ParamDict[myPair.first] = myPair.second;
			}
		}
	}
}