#ifndef BuildDataOrganizer_H
#define BuildDataOrganizer_H
/*
* HydroPlus is source code and models developed and managed by Theodore Endreny and his students at SUNY ESF, te@esf.edu
* The US Forest Service and Davey Tree have provided strategic software development support through funding and personnel
* Attribution for use of software and code is requested. Please see Executive.cpp for documentation.
*/

//_USE_MATH_DEFINES gives access to Pi as M_PI, and must precede #include <cmath> or <math.h>
#define _USE_MATH_DEFINES
#include <cmath>
#include "Inputs\Inputs.h"
#include "DataOrganizer.h"

//Define constant effective zero value
#define Constant_1E_negative5 1.E-5
//Conversion between fraction and percent
#define Ratio_Decimal_to_Percent 0.01
//Conversion m to mm
#define Ratio_m_to_mm 1.0/1000.0

//MAX function for finding minimum of x_var and y_var using conditional ternary operator (?:)
//Note: Ternary operator explained: For (x_var<=y_var?x_var:y_var), if x_var<=y_var is true, then x_var is taken, otherwise y_var taken.
#define MAX(x_var,y_var) (((x_var)>=(y_var)) ? (x_var) : (y_var))        

class BuildDataOrganizer
{
	public:
		static void BuildDataOrganizer_Manager(DataOrganizer *organizer, Inputs *input, string Directory_Input_CLArg);
		static void BuildStatisticalDataOrganizer(DataOrganizer *organizer, Inputs *input, string Directory_Input_CLArg);
		static void BuildSpatialDataOrganizer(DataOrganizer *organizer, Inputs *input, string Directory_Input_CLArg);
		static void AddFolderAreaSpatialCoverData(DataFolder *folder, Inputs *input, int MapPixel_ID, int Folder_Type_ID);
		static void AddFolderAreaStatisticalCoverData(DataFolder *folder, Inputs* input, int DataDrawer_ID, int DataFolder_ID, int Folder_Type_ID);
		static void CheckHydroPlusConfigInputData(DataFolder* folder, Inputs* input, string Directory_Input_CLArg);
		static void SetGIMissingParams(DataOrganizer *organizer, Inputs *input);
		static void Compute_CoverType_WeightedAverage_m2(DataFolder* folder, Inputs* input, int DataDrawer_ID, int DataFolder_ID, int MapPixel_ID, int Count_ArrayItem);
		static void Compute_CoverType_WeightedAverage_frac(DataOrganizer* organizer, Inputs* input);
};

#endif