#ifndef Drainage_StormwaterDevice_H
#define Drainage_StormwaterDevice_H
/*
* HydroPlus is source code and models developed and managed by Theodore Endreny and his students at SUNY ESF, te@esf.edu
* The US Forest Service and Davey Tree have provided strategic software development support through funding and personnel
* Attribution for use of software and code is requested. Please see Executive.cpp for documentation.
*/
#include "../Inputs/Inputs.h"
#include "../DataFolder.h"

//Define constants effective maximum 
#define Constant_1E_positive10 1.E10
//Define constant effective zero value
#define Constant_1E_negative10 1.E-10        
// Value for TRUE state
#define TRUE 1              
// Value for FALSE state
#define FALSE 0              
//Conversion seconds to hour
#define Ratio_s_to_hr 3600
//Conversion hour to seconds
#define Ratio_Hour_to_Second 1.0/3600.0
//Conversion hour to day
#define Ratio_Hour_to_Day 24
//Conversion meters to feet
#define Ratio_m_to_ft 0.3048
//Conversion m to mm
#define Ratio_m_to_mm 1.0/1000.0

//MIN function for finding minimum of x_var and y_var using conditional ternary operator (?:)
//Note: Ternary operator explained: For (x_var<=y_var?x_var:y_var), if x_var<=y_var is true, then x_var is taken, otherwise y_var taken.
#define MIN(x_var,y_var) (((x_var)<=(y_var)) ? (x_var) : (y_var))        

//MAX function for finding minimum of x_var and y_var using conditional ternary operator (?:)
//Note: Ternary operator explained: For (x_var<=y_var?x_var:y_var), if x_var<=y_var is true, then x_var is taken, otherwise y_var taken.
#define MAX(x_var,y_var) (((x_var)>=(y_var)) ? (x_var) : (y_var))        

class Drainage_StormwaterDevice {
public:

	//Percolation_Soil function gets percolation rate through GI soil layer; Hydro SoilEvapZoneQ in GI area = percolation
	static double Percolation_Soil(Inputs* input, DataFolder* folder, int timeStep);
	//Percolation_Pavement function computes reduced permeability of a pavement layer due to clogging.
	static void Percolation_Pavement(Inputs* input, DataFolder* folder, int timeStep);
	//Exfiltration_Vault function gets exfiltration rate from vault zone into the native soil
	static double Exfiltration_Vault(Inputs* input, DataFolder* folder, int timeStep);
	//Drainage_VadoseZone function determines if percolation replaces exfiltration as drainage to the native soil
	//Note: Drainage_VadoseZone raises the groundwater table, and creates temporary groundwater mounding
	static void Drainage_VadoseZone(Inputs* input, DataFolder* folder, int timeStep);

};

#endif