#include "SolarCalculator.h"

//WetBulbGlobeTempCalc::WetBulbGlobeTempCalc defined constructor for use here with input pointer
SolarCalculator::SolarCalculator(Inputs* input) {
    //Initialize variables
    Latitude_NH_pos_dd = 0;
    Longitude_WH_pos_dd = 0;
    DeclinationAngle_Solar_rad = 0;
    Day_Angle_rad = 0;
    Equation_of_Time_HH_point_Mn = 0;
    Standard_Meridian_deg = 0;
    TrueSolarTime_HH_point_Mn = 0;

    HourAngle_Solar_deg = 0;
    HourAngle_Solar_rad = 0;

    Cos_ZenithAngle_Solar = 0;
    ZenithAngle_Solar_rad = 0;
    ZenithAngle_Solar_deg = 0;
    ElevationAngle_Solar_deg = 0;
    Correction_ZenithAngle_deg = 0;

    Sin_AltitudeAngle_Solar = 0;
    AltitudeAngle_Solar_rad = 0;

    Sin_AzimuthAngle_Solar = 0;
    Cos_AzimuthAngle_Solar = 0;
    AzimuthAngle_Solar_S_0_rad = 0;
    AzimuthAngle_Solar_N_0_rad = 0;
    AzimuthAngle_Solar_N_0_deg = 0;

    Cos_IncidenceAngle_Yang = 0;
    Cos_IncidenceAngle_Solar = 0;
}

//WetBulbGlobeTemperature_Inputs function will manage call to calculate the wet bulb globe temperature
void SolarCalculator::SolarCalculator_Inputs(Inputs* input, int MapPixel_ID, int timeStep) {

    //YYYYMMDD is input->SimulationDate_GDH[timeStep]
    int Date_YYYYMMDD = input->SimulationDate_GD[timeStep];
    //Date_JulianDay = input->Date_YYYYMMDD_to_JulianDay(YYYYMMDD)
    int Date_JulianDay = input->Date_YYYYMMDD_to_JulianDay(Date_YYYYMMDD);

    //Day_Angle_rad (radians)
    //cte 2024 Determine why subtraction of 1; check Satoshi and my formula reference 
    Day_Angle_rad = (360 * (Date_JulianDay - 1) / 365.0) * Ratio_Radian_to_Degree;
    //Equation_of_Time_HH_point_Mn (decimal minutes), equation of time
    //cte 2024 Get source of equation
    Equation_of_Time_HH_point_Mn = (0.000075 + 0.001868 * cos(Day_Angle_rad) - 0.032077 * sin(Day_Angle_rad)
        - 0.014615 * cos(2 * Day_Angle_rad) - 0.040849 * sin(2 * Day_Angle_rad)) * 229.18;
    //DeclinationAngle_Solar_rad (radians), solar declination angle
    DeclinationAngle_Solar_rad = (0.006918 - 0.399912 * cos(Day_Angle_rad) + 0.070257 * sin(Day_Angle_rad)
        - 0.006758 * cos(2 * Day_Angle_rad) + 0.000907 * sin(2 * Day_Angle_rad)
        - 0.002697 * cos(3 * Day_Angle_rad) + 0.00148 * sin(3 * Day_Angle_rad));

    //row = (int)(MapPixel_ID / input->ncols + 1), derived from MapPixel_ID, first row indexed to 1
    //Note: Conversion between MapPixel_ID and row & column pair, with MapPixel_ID starting at 0, row & col starting at 1
    //Note: Conversion Eq: row=(int)(MapPixel_ID/input->ncols+1); col=(MapPixel_ID%(int)(input->ncols)+1); MapPixel_ID = (row * nCols) + col
    int row = (int)(MapPixel_ID / input->ncols + 1);
    //col = (MapPixel_ID % (int)(input->ncols) + 1), derived from MapPixel_ID, first col indexed to 1
    int col = (MapPixel_ID % (int)(input->ncols) + 1);
    //Latitude_NH_pos_dd (dd) = Latitude_Longitude_vectorPair_dd[row - 1][col - 1].first
    //Note: Latitude_Longitude_vectorPair_dd indexed to 0, row and col indexed to 1, so subtract 1 from row and col
    Latitude_NH_pos_dd = input->Latitude_Longitude_vectorPair_dd[row - 1][col - 1].first;
    //Longitude_WH_pos_dd (dd) = Latitude_Longitude_vectorPair_dd[row - 1][col - 1].second * -1; Negative sign makes Western Hemisphere +
    //Note: Latitude_Longitude_vectorPair_dd indexed to 0, row and col indexed to 1, so subtract 1 from row and col
    Longitude_WH_pos_dd = input->Latitude_Longitude_vectorPair_dd[row - 1][col - 1].second * -1;

    //To handle both Northern/Southern hemisphere, accept negative latitude (for Southern)
    //Latitude_NH_pos_rad (radians), latitude
    double Latitude_NH_pos_rad = Latitude_NH_pos_dd * Ratio_Radian_to_Degree;
    double Longitude_WH_neg_dd = -1 * Longitude_WH_pos_dd;
    //GMT_Offset_int = Longitude_WH_neg_dd / 15, negative values in western hemisphere
    //Note: Earth divided into 24 time zones, each separated into 15 degrees, where 24 * 15 deg = 360 deg
    int GMT_Offset_int = int(Longitude_WH_neg_dd / 15.0);

    //Standard_Meridian_deg (degrees) takes 15 deg for each GMT offset hour
    Standard_Meridian_deg = 15.0 * GMT_Offset_int;

    //HH_MM_SS is input->SimulationTime_HMS[timeStep]
    string HH_MM_SS = input->SimulationTime_HMS[timeStep];
    //Date_Hour_HH extracted from HH_MM_SS; values must be 1 to 24
    int Date_Hour_HH = stoi(HH_MM_SS.substr(0, 2)) + 1;

    //TrueSolarTime_HH_point_Mn (hour with decimal minutes), true solar time
    TrueSolarTime_HH_point_Mn = Date_Hour_HH + (4 * (abs(Standard_Meridian_deg) - abs(Longitude_WH_pos_dd)) + Equation_of_Time_HH_point_Mn) / 60.0;
    //hourAngle (degrees)
    HourAngle_Solar_deg = 15.0 * TrueSolarTime_HH_point_Mn - 180;
    //hourAngle (radians)
    HourAngle_Solar_rad = HourAngle_Solar_deg * Ratio_Radian_to_Degree;

    //Cos_ZenithAngle_Solar from Eq 9-12 Vanek et al (2012), which shows equivalence of Cos_ZenithAngle_Solar and Sin_AltitudeAngle_Solar
    Cos_ZenithAngle_Solar = sin(DeclinationAngle_Solar_rad) * sin(Latitude_NH_pos_rad) + cos(DeclinationAngle_Solar_rad) * cos(Latitude_NH_pos_rad) * cos(HourAngle_Solar_rad);

    if (Cos_ZenithAngle_Solar > 1)
    {
        Cos_ZenithAngle_Solar = 1.0;
    }
    else if (Cos_ZenithAngle_Solar < -1)
    {
        Cos_ZenithAngle_Solar = -1.0;
    }

    //ZenithAngle_Solar_deg is acos(Cos_ZenithAngle_Solar) * Ratio_Degree_to_Radian, acosine function, convert to deg
    ZenithAngle_Solar_deg = acos(Cos_ZenithAngle_Solar) * Ratio_Degree_to_Radian;
    //ElevationAngle_Solar_deg is 90.0 - ZenithAngle_Solar_deg, the complement of ZenithAngle_Solar_deg
    ElevationAngle_Solar_deg = 90.0 - ZenithAngle_Solar_deg;

    //Correction_ZenithAngle_deg initialized to zero
    Correction_ZenithAngle_deg = 0.0;
    //If ElevationAngle_Solar_deg  >= -1 && ElevationAngle_Solar_deg  < 15 then enter
    if (-1 <= ElevationAngle_Solar_deg && ElevationAngle_Solar_deg < 15) {
        //Correction_ZenithAngle_deg is AtmosphericPressure_SeaLevel_hPa / AtmosphericTemperature_Avg_K * (0.1594 + 0.0196 * ElevationAngle_Solar_deg + 0.00002 * pow(ElevationAngle_Solar_deg, 2)) / (1 + 0.505 * ElevationAngle_Solar_deg + 0.0845 * pow(ElevationAngle_Solar_deg, 2));
        Correction_ZenithAngle_deg = AtmosphericPressure_SeaLevel_hPa / AtmosphericTemperature_Avg_K * (0.1594 + 0.0196 * ElevationAngle_Solar_deg + 0.00002 * pow(ElevationAngle_Solar_deg, 2)) / (1 + 0.505 * ElevationAngle_Solar_deg + 0.0845 * pow(ElevationAngle_Solar_deg, 2));
    }
    //Else If ElevationAngle_Solar_deg >= 15 && ElevationAngle_Solar_deg < 90 then enter
    else if (15 <= ElevationAngle_Solar_deg && ElevationAngle_Solar_deg < 90) {
        //Correction_ZenithAngle_deg is 0.00452 * AtmosphericPressure_SeaLevel_hPa / AtmosphericTemperature_Avg_K / tan(ElevationAngle_Solar_deg * Ratio_Radian_to_Degree)
        Correction_ZenithAngle_deg = 0.00452 * AtmosphericPressure_SeaLevel_hPa / AtmosphericTemperature_Avg_K / tan(ElevationAngle_Solar_deg * Ratio_Radian_to_Degree);
    }
    //ZenithAngle_Solar_deg is ZenithAngle_Solar_deg - Correction_ZenithAngle_deg
    ZenithAngle_Solar_deg = ZenithAngle_Solar_deg - Correction_ZenithAngle_deg;
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //Sin_AltitudeAngle_Solar from Eq 2 of Kumar et al. (1997)
    Sin_AltitudeAngle_Solar = sin(DeclinationAngle_Solar_rad) * sin(Latitude_NH_pos_rad) + cos(DeclinationAngle_Solar_rad) * cos(Latitude_NH_pos_rad) * cos(HourAngle_Solar_rad);
    //ZenithAngle_Solar_rad is acosine of Cos_ZenithAngle_Solar
    ZenithAngle_Solar_rad = acos(Cos_ZenithAngle_Solar);
    //AltitudeAngle_Solar_rad is asine of Sin_AltitudeAngle_Solar
    AltitudeAngle_Solar_rad = asin(Sin_AltitudeAngle_Solar);

    //Algorithm from Reda et al. (2003) NREL SPA, with conversion from Equator or S=0 to Pole or N=0, 360 degree rotation clockwise
    //AzimuthAngle_Solar_S_0_rad is computed with Eq 45 of Reda et al. (2003), using non-prime RHS coefficients
    //Note: Eq 45 computes AzimuthAngle_Solar_S_0_rad as Equator or S=0, 360 degree rotation clockwise
    //Note: This Eq preferred over calculating Sin_AzimuthAngle_Solar from Eq 9-13 Vanek et al (2012) and Eq 3 Kumar et al. (1997)
    //Note: The algorithm based on NREL SPA by Reda et al. (2003) and is preferred to original PASATH approach.
    AzimuthAngle_Solar_S_0_rad = atan2(sin(HourAngle_Solar_rad), cos(HourAngle_Solar_rad) * sin(Latitude_NH_pos_rad) - tan(DeclinationAngle_Solar_rad) * cos(Latitude_NH_pos_rad));

    //AzimuthAngle_Solar_N_0_rad is computed with Eq 46 of Reda et al. (2003), with Pole or N=0, 360 degree rotation clockwise
    AzimuthAngle_Solar_N_0_rad = AzimuthAngle_Solar_S_0_rad + M_PI;
    //AzimuthAngle_Solar_N_0_deg is converted from radians as AzimuthAngle_Solar_N_0_rad * Ratio_Degree_to_Radian 
    AzimuthAngle_Solar_N_0_deg = AzimuthAngle_Solar_N_0_rad * Ratio_Degree_to_Radian;
    //AzimuthAngle_Solar_N_0_deg is constrained to 360 deg with algorithm Step 3.2.6 of Reda et al. (2003), with Pole or N=0, 360 degree rotation clockwise
    AzimuthAngle_Solar_N_0_deg = AzimuthAngle_Solar_N_0_deg / 360.0;
    AzimuthAngle_Solar_N_0_deg = 360.0 * (AzimuthAngle_Solar_N_0_deg - floor(AzimuthAngle_Solar_N_0_deg));
    if (AzimuthAngle_Solar_N_0_deg < 0.0) {
        AzimuthAngle_Solar_N_0_deg = AzimuthAngle_Solar_N_0_deg + 360.0;
    }
    //End of Algorithm Step 3.2.6 of Reda et al. (2003)
    //AzimuthAngle_Solar_N_0_rad is converted from degrees as AzimuthAngle_Solar_N_0_deg * Ratio_Radian_to_Degree
    AzimuthAngle_Solar_N_0_rad = AzimuthAngle_Solar_N_0_deg * Ratio_Radian_to_Degree;
    //Cos_IncidenceAngle_Solar_Reda from Eq 47 of Reda et al. (2003), so Pole or N=0, 360 degree rotation clockwise
    //Note: Reda et al. (2003) is  National Renewable Energy Laboratory (NREL) Solar Position Algorithm (SPA) Eq 47, equivalent to Iqbal (1983) Eq 1.6.5b
    //Note: AspectGround_N_0_rad and AzimuthAngle_Solar_N_0_rad use Pole or N=0, 360 degree clockwise rotation
    Cos_IncidenceAngle_Solar = cos(ZenithAngle_Solar_rad) * cos(input->SlopeGround_rad[MapPixel_ID]) + sin(ZenithAngle_Solar_rad) * sin(input->SlopeGround_rad[MapPixel_ID]) * cos(AzimuthAngle_Solar_N_0_rad - input->AspectGround_N_0_rad[MapPixel_ID]);
}