#ifndef SolarCalculator_H
#define SolarCalculator_H
/*
* HydroPlus is source code and models developed and managed by Theodore Endreny and his students at SUNY ESF, te@esf.edu
* The US Forest Service and Davey Tree have provided strategic software development support through funding and personnel
* Attribution for use of software and code is requested. Please see Executive.cpp for documentation.
*/

//_USE_MATH_DEFINES gives access to Pi as M_PI, and must precede #include <cmath> or <math.h>
#define _USE_MATH_DEFINES
#include <cmath>
#include <math.h>
#include <string>
#include <utility>
#include <map>
#include <vector>
#include <sstream>
#include "Inputs.h"
#include "..\DataFolder.h"


#define AtmosphericPressure_SeaLevel_hPa 1013.25
#define AtmosphericTemperature_Avg_K 288.15


class SolarCalculator
{

public:
	SolarCalculator(Inputs* input);
	//~SolarCalculator() {} destructor that ensures resources used by the object can be released 
	~SolarCalculator() {};

	//WetBulbGlobeTemperature_Inputs function will manage call to calculate the wet bulb globe temperature
	void SolarCalculator_Inputs(Inputs* input, int MapPixel_ID, int timeStep);

	double get_Cos_ZenithAngle_Solar() { return Cos_ZenithAngle_Solar; }
	double get_Cos_IncidenceAngle_Solar() { return Cos_IncidenceAngle_Solar; }
	double get_ZenithAngle_Solar_rad() { return ZenithAngle_Solar_rad; }
	double get_Cos_AzimuthAngle_Solar() { return Cos_AzimuthAngle_Solar; }
	double get_Sin_AzimuthAngle_Solar() { return Sin_AzimuthAngle_Solar; }
	double get_AzimuthAngle_Solar_N_0_rad() { return AzimuthAngle_Solar_N_0_rad; }
	double get_DeclinationAngle_Solar_rad() { return DeclinationAngle_Solar_rad; }
	double get_HourAngle_Solar_rad() { return HourAngle_Solar_rad; }
	double get_Latitude_NH_pos_dd() { return Latitude_NH_pos_dd; }
	double get_Longitude_WH_pos_dd() { return Longitude_WH_pos_dd; }

private:

	double Latitude_NH_pos_dd;
	double Longitude_WH_pos_dd;

    double Day_Angle_rad;
    double Equation_of_Time_HH_point_Mn;
    double Standard_Meridian_deg;
    double TrueSolarTime_HH_point_Mn;

	double DeclinationAngle_Solar_rad;
	double HourAngle_Solar_deg;
    double HourAngle_Solar_rad;
	
	double Cos_ZenithAngle_Solar;
	double ZenithAngle_Solar_rad;
	double ZenithAngle_Solar_deg;
	double ElevationAngle_Solar_deg;
	double Correction_ZenithAngle_deg;

	double Sin_AltitudeAngle_Solar;
	double AltitudeAngle_Solar_rad;

	double Sin_AzimuthAngle_Solar;
	double Cos_AzimuthAngle_Solar;
	double AzimuthAngle_Solar_S_0_rad;
	double AzimuthAngle_Solar_N_0_rad;
	double AzimuthAngle_Solar_N_0_deg;

	double Cos_IncidenceAngle_Yang;
	double Cos_IncidenceAngle_Solar;
	
};

#endif