#ifndef WeatherPreprocessor_H
#define WeatherPreprocessor_H
/*
* HydroPlus is source code and models developed and managed by Theodore Endreny and his students at SUNY ESF, te@esf.edu
* The US Forest Service and Davey Tree have provided strategic software development support through funding and personnel
* Attribution for use of software and code is requested. Please see Executive.cpp for documentation.
*/
#include <vector>
#include <iostream>
#include <numeric>
#include <string>
#define _USE_MATH_DEFINES
#include <math.h>
#include <algorithm> 
#include <numeric> 
#include "../DataOrganizer.h"
//#include "Projections.h"

//Forward declaration of class Inputs
class Inputs;

#define meterofdgree 111000; 
//Conversion km to m
#define Ratio_km_to_m 1.0/1000.0
//Constant environmental lapse rate dry (ELR) air (from Stull, 2000, estimated as Gamma_elr = 5.5 C/km), ...
//Note: ... there is no ELR equation but is often between 5 and 6 C/km, hence the estimate of 5.5 C.km
#define Gamma_EnvironmentalLapseRate_C_p_km 5.5
//Constant adiabatic lapse rate dry air (from Stull, 2000, defined after Eq 5.13, Gamma_d = 9.8 C/km)
//Note: Adiabatic lapse rate dry air is not used for estimating air 2 m above ground when summiting a mountain 
#define Gamma_dry_C_p_km 9.8
//Constant adiabatic lapse rate dew point air (from Stull, 2000, derived from Eq 5.8; 1/0.125 = 1/Z_lcl * (T - Td), if Z_lcl=1 km, T=9.8 then Td=1.8)
#define Gamma_dew_C_p_km 1.8


class WeatherPreprocessor
{
public:
	//cte WeatherPreprocessor(Inputs* input);
	static vector<long double>  Distance_Pixel_to_Station_vec_km, Weight_StationInverseDistance_vec_frac, Distance_InverseSquared_vec;
	static vector<string>  OBJECTID_shortest, USAF_shortest, WBAN_shortest, Path_folder_shortest, TI_option_shortest, NLCD_Class_shortest;
	static vector<long double> Easting_shortest, Northing_shortest, Elev_m_shortest, Distance_shortest, 
		Slope_deg_shortest, Aspect_deg_shortest, TC_per_shortest, IC_per_shortest, SVeg_per_shortest, Soil_per_shortest, Water_per_shortest;
	//static vector<long double> albersEasting_shortest, albersNorthing_shortest;
	
	static void CalcWeightsandRead(long double PixelCenter_easting_m, long double PixelCenter_northing_m, Inputs* input, int DataDrawer_ID);
	static void WeatherStationDistance_sort(Inputs* input, long double PixelCenter_easting_m, long double PixelCenter_northing_m);
	static void MapPixelNearestWeatherStationWriter( Inputs* input);
	static void WeatherStationInformationWriter(Inputs* input);
	static void MapPixelWeightedWeatherInputsWriter(Inputs* input);
	static void ApplyingWeights_to_pixel_at_timestep(Inputs* input, int pixelID, int timeStep);
	static void WeatherDataReadandProcessing(Inputs* input);
	//ComputeSpatiallyWeightedVariables function will compute the spatially weighted meteorological variables
	static void ComputeSpatiallyWeightedVariables(Inputs* input, int DataDrawer_ID, int timeStep);
	static void MapPixelCalcWeightsandRead(Inputs* input);
	static void MapCenterCalcWeightsandRead(Inputs* input);
	static bool readWeatherFile(Inputs* input);
	static bool readEvaporationFile(Inputs* input);
	static bool readRadiationFile(Inputs* input);
	static void VectorsOfWeatherData(Inputs* input, istream& lineWeatherFile, bool Flag_SingleStation, bool Flag_ProcessTimeStep);

	static vector<int> SimulationDate_GD_AllRecord, SimulationDate_GDH_AllRecord, SimulationDate_HH_AllRecord;
	static vector<string> SimulationTime_HMS_AllRecord;
	static vector<double> RadiationNet_Wpm2_AllRecord, Tair_F_AllRecord, Tdew_F_AllRecord, Rain_m_AllRecord, Snow_m_AllRecord, WindSpd_mps_AllRecord, WindDir_deg_AllRecord, AtmPres_kPa_AllRecord, Precip_mpdt_AllRecord;
	static int SimulationDate_Hottest_GD;
	static string SimulationTime_Hottest_HMS;
	static int SimulationDateStart_GDH, SimulationDateStop_GDH;

	//saving detailed info of stations for the hottest day
	static map<string, vector<double> > WeatherMap, EvaporationMap, RadiationMap;
	static map<int, map<int, vector<string> > > PixelNearestStation;
	//saving basic info of a station, including the hottest day
	static map<string, vector<string> > StationInfoMap;
	//StationMapID vector will contain string integers as ID to each weather station that has influence across maps used in simulation
	//static vector<string> StationMapID;
	
	static map<int, map<string, vector<double> > > WeightedInputMap;
	
	//Tair_Mesoscale_byStationID_K is 2D vector with (ID)(Tair_K)
	static map<int, vector<double>> Tair_Mesoscale_byStationID_K;
	//AbsHumidity_Mesoscale_byStationID_kg_p_m3 is 2D vector with (ID)(AbsHumidity_kg_p_m3)
	static map<int, vector<double>> AbsHumidity_Mesoscale_byStationID_kg_p_m3;
	//Tair_weighted_K and Tdew_weighted_K (K) are spatially weighted values of station air temperature, corrected for elevation
	static double Tair_weighted_K, Tdew_weighted_K;
	//Tair_mesoScale_weighted_K and AbsHumidity_mesoScale_weighted_kg_p_m3 are spatially weighted values of station mesoscale air temperature, corrected for elevation
	static double Tair_mesoScale_weighted_K, AbsHumidity_mesoScale_weighted_kg_p_m3;

	static int Date_YYYYMMDD, Time_HHMMSS, timeStepMetData, previous_YYYYMMDD, previous_HMS;
	static string Time_HHMMSS_str;

	//static bool Flag_SearchHottestDay;
	//static int Date_Hottest_YYYYMMDD;
	static string ReadStation_path, ReadStationID;
	static int Actual_hottest_GDH_index;

	static vector<long double> Map_Easting_X_m_vec;
	static vector<long double> Map_Northing_Y_m_vec;
	static int centerMapPixel_ID;

private:
	//cte Inputs* input;
	static long double toRadians(const long double degree);
	static long double Distance_Calc_LatLong(long double pixelcenter_lat, long double pixelcenter_long, long double station_lat, long double station_lon);
	static long double Distance_Calc_EastingNorthing(long double PixelCenter_x, long double PixelCenter_y, long double Station_easting_m, long double Station_northing_m);
	static void resizeWeatherVectors(Inputs* input);
	static void clearWeatherVectors(Inputs* input); 
	static long double PixelCenter_X_MapLowerLeft_m, PixelCenter_Y_MapLowerLeft_m;
	string Directory_Input_CLArg;

};

#endif