#include "TemperatureOutputWriter.h"
#include <fstream>
#include <iomanip>
#include <sstream>
#include <vector>
#include <string>
#include <algorithm>

//TemperatureOutputWriter::timeStepResultsFileProcessor function coordinates time-based map output, calling createTimeStepResultFile function
void TemperatureOutputWriter::timeStepResultsFileProcessor(int timeStep, DataOrganizer* organizer, string OutputDirectory_String , Inputs* input, WeatherPreprocessor* Weatherpre)
{
	int SimulationDate_GDH;

	//print is defined by value of TimeBasedOutputAllTimeSteps, 1 or 0, true or false, with value set in Inputs::ProcessTemperatureExecutionParams
	//Note: print will control whether printing time-based output maps for all time steps or specific time steps
	bool print = input->TemperatureCalculationParams["TimeBasedOutputAllTimeSteps"];

	//If Flag_HottestDaySimulated equal 1 then implement date for Hottest Hour based on searching Multiple Station
	//Note: If Flag_HottestDaySimulated = 1, then output dates are from HydroPlusConfig.xml StartDate_YYYYMMDD and TimeBasedOutputPeriod0 values
	//Note: While SimulationDateStart_GD is fixed to a single date, input->SimulationDate_HH can contain a time series of values
	if (input->SimulationStringParams["Flag_HottestDaySimulated"] == "1") {
		SimulationDate_GDH = input->SimulationNumericalParams["SimulationDateStart_GD"] * 100 + input->SimulationDate_HH[timeStep];
	}
	//Note: If Flag_HottestDaySimulated = 0, then output dates are from time series contained in SimulationDate_GDH, based on HydroPlusConfig.xml inputs
	else {
		//SimulationDate_GDH is string created from SimulationDate_GDH (YYYYMMDDHH) of timeStep, vector defined in readWeatherFile
		SimulationDate_GDH = input->SimulationDate_GDH[timeStep];
	}

	//SimulationDate_GDH_string is created from SimulationDate_GDH (YYYYMMDDHH)
	string SimulationDate_GDH_string = to_string(input->SimulationDate_GDH[timeStep]);

	//If print = 0, aka not 1 or true, then print time-based maps at specific time steps, not all time steps
	if (!print) {
		//for loop with iter counter through size of StartDatesForTimeBasedTempOutput vector, which contains start dates for time-based output
		for (int iter = 0; iter < input->StartDatesForTimeBasedTempOutput.size(); ++iter) {
			//if SimulationDate_GDH of time step is within the specific range of dates for printing time-based maps then
			if (SimulationDate_GDH >= input->StartDatesForTimeBasedTempOutput[iter] &&
				SimulationDate_GDH <= input->EndDatesForTimeBasedTempOutput[iter]) {

				//redefine print as true for this time step so it will pass to next conditional
				print = true;
				//break from for loop before passing to next specific date for time-based output
				break;
			}
		}
	}

	//Call createTimeStepResultFile function to create file and write six rows of header values for ArcGIS ASCII map type
	//If print is true and PrintAllTimeBasedResults not true, then only print the map type defined in HydroPlusConfig.xml by PrintTimeBasedResults element
	//Note: Any HydroPlus variable can be entered as the PrintTimeBasedResults variable for printing
	//Note: TemperatureExecutionParams string variables converted to TemperatureCalculationParams numeric variables
	if (print && input->TemperatureCalculationParams["PrintAllTimeBasedResults"] != 1) {

		string printTimeBasedResults = input->TemperatureExecutionParams["PrintTimeBasedResults"];
		vector<string> PrintTimeBasedResults_variableNames_vec;
		stringstream stringstream_all(printTimeBasedResults);
		string stringstream_item;

		// Split the string based on comma
		while (getline(stringstream_all, stringstream_item, ',')) {
			//stringstream_item erase to remove or trim any leading or trailing whitespace
			stringstream_item.erase(stringstream_item.begin(), find_if(stringstream_item.begin(), stringstream_item.end(), [](unsigned char ch) { return !isspace(ch); }));
			//stringstream_item erase to remove or trim any leading or trailing whitespace
			stringstream_item.erase(find_if(stringstream_item.rbegin(), stringstream_item.rend(), [](unsigned char ch) { return !isspace(ch); }).base(), stringstream_item.end());
			//PrintTimeBasedResults_variableNames_vec increased using push_back with stringstream_item
			PrintTimeBasedResults_variableNames_vec.push_back(stringstream_item);
		}

		//For loop to call createTimeStepResultFile for each PrintTimeBasedResults_variableName_item within PrintTimeBasedResults_variableNames_vec
		for (const string& PrintTimeBasedResults_variableName_item : PrintTimeBasedResults_variableNames_vec) {
			//createTimeStepResultFile function called with PrintTimeBasedResults_variableName_item, e.g., Tair_K, Tair_weighted_K
			createTimeStepResultFile(PrintTimeBasedResults_variableName_item, SimulationDate_GDH, input);
		}

		//call createTimeStepResultFile function to write header rows for 1st term in (), which is a variable in HydroPlus.sln
		//	Note: the PrintTimeBasedResults value must be identical to one of the 31 map types defined below in this timeStepResultsFileProcessor function
		//cte 202407 Refactor: createTimeStepResultFile(input->TemperatureExecutionParams["PrintTimeBasedResults"], SimulationDate_GDH, input);
	}

	//Call createTimeStepResultFile function to create file and write six rows of header values for ArcGIS ASCII map type
	//If print equals 1 and PrintAllTimeBasedResults equals 1, then call createTimeStepResultFile function for each time-based map type listed below
	//Note: Any HydroPlus variable can be entered as the PrintTimeBasedResults variable for printing
	if (print && input->TemperatureCalculationParams["PrintAllTimeBasedResults"] == 1) {

		//call createTimeStepResultFile function to write header rows for 1st term in (), which is a variable in HydroPlus.sln
		createTimeStepResultFile("Tair_K", SimulationDate_GDH, input);
		createTimeStepResultFile("Tdew_K", SimulationDate_GDH, input);
		createTimeStepResultFile("HeatIndex_K", SimulationDate_GDH, input);
		createTimeStepResultFile("Twetbulbglobe_K", SimulationDate_GDH, input);
		createTimeStepResultFile("AbsHumidity_kg_p_m3", SimulationDate_GDH, input);

		createTimeStepResultFile("ImpNR_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("ImpH_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("ImpLE_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("ImpDeltaQ_W_p_m2", SimulationDate_GDH, input);

		createTimeStepResultFile("TreeNR_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("TreeH_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("TreeLE_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("TreeLEE_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("TreeLET_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("TreeDeltaQ_W_p_m2", SimulationDate_GDH, input);

		createTimeStepResultFile("SVegNR_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("SVegH_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("SVegLE_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("SVegLEE_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("SVegLET_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("SVegDeltaQ_W_p_m2", SimulationDate_GDH, input);

		createTimeStepResultFile("SoilNR_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("SoilH_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("SoilLE_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("SoilDeltaQ_W_p_m2", SimulationDate_GDH, input);

		createTimeStepResultFile("WaterNR_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("WaterH_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("WaterLE_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("WaterDeltaQ_W_p_m2", SimulationDate_GDH, input);

		createTimeStepResultFile("NR_total_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("H_total_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("LE_total_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("DeltaQ_W_p_m2", SimulationDate_GDH, input);
		createTimeStepResultFile("AH_total_W_p_m2", SimulationDate_GDH, input);
	}

	//Call writeToTimeStepResultFile function to write variable values under six rows of header values for ArcGIS ASCII map type
	//If print is true and PrintAllTimeBasedResults not true, then only print the map type defined in HydroPlusConfig.xml by PrintTimeBasedResults element
	//Note: Any HydroPlus variable can be entered as the PrintTimeBasedResults variable for printing
	//Note: TemperatureExecutionParams string variables converted to TemperatureCalculationParams numeric variables
	if (print && input->TemperatureCalculationParams["PrintAllTimeBasedResults"] != 1) {

		string printTimeBasedResults = input->TemperatureExecutionParams["PrintTimeBasedResults"];
		vector<string> PrintTimeBasedResults_variableNames_vec;
		stringstream stringstream_all(printTimeBasedResults);
		string stringstream_item;

		// Split the string based on comma
		while (getline(stringstream_all, stringstream_item, ',')) {
			//stringstream_item erase to remove or trim any leading or trailing whitespace
			stringstream_item.erase(stringstream_item.begin(), find_if(stringstream_item.begin(), stringstream_item.end(), [](unsigned char ch) { return !isspace(ch); }));
			//stringstream_item erase to remove or trim any leading or trailing whitespace
			stringstream_item.erase(find_if(stringstream_item.rbegin(), stringstream_item.rend(), [](unsigned char ch) { return !isspace(ch); }).base(), stringstream_item.end());
			//PrintTimeBasedResults_variableNames_vec increased using push_back with stringstream_item
			PrintTimeBasedResults_variableNames_vec.push_back(stringstream_item);
		}

		//For loop to call writeToTimeStepResultFile for each PrintTimeBasedResults_variableName_item within PrintTimeBasedResults_variableNames_vec
		for (const string& PrintTimeBasedResults_variableName_item : PrintTimeBasedResults_variableNames_vec) {
			//writeToTimeStepResultFile function called to write values to map for 1st term in (), which is a variable in HydroPlus.sln
			writeToTimeStepResultFile(PrintTimeBasedResults_variableName_item, SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		}
	}
	
	//Call writeToTimeStepResultFile function to write variable values under six rows of header values for ArcGIS ASCII map type
	//If print equals 1 and PrintAllTimeBasedResults equals 1, then call writeToTimeStepResultFile function for each time-based map type listed below
	//Note: TemperatureExecutionParams string variables converted to TemperatureCalculationParams numeric variables
	if (print && input->TemperatureCalculationParams["PrintAllTimeBasedResults"]) {

		//call writeToTimeStepResultFile function to write values to map for 1st term in (), which is a variable in HydroPlus.sln
		writeToTimeStepResultFile("Tair_K", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("Tdew_K", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("AbsHumidity_kg_p_m3", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("NR_total_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("ImpNR_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("TreeNR_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SVegNR_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SoilNR_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("WaterNR_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("ImpH_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("ImpLE_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("TreeH_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("TreeLE_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("TreeLEE_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("TreeLET_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SVegH_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SVegLE_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SVegLEE_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SVegLET_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SoilH_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SoilLE_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("WaterH_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("WaterLE_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("H_total_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("LE_total_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("DeltaQ_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("ImpDeltaQ_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("TreeDeltaQ_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SVegDeltaQ_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("SoilDeltaQ_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
		writeToTimeStepResultFile("WaterDeltaQ_W_p_m2", SimulationDate_GDH, input->SimulationStringParams["OutputFolder_Path"], organizer, input);
	}
}

//Variables defined in TemperatureOutputWriter.h need to be declared in TemperatureOutputWriter.cpp; relates to static vs non-static function
int TemperatureOutputWriter::Folder_Type_ID;

//TemperatureOutputWriter::generateRowColFiles function to create cell-based values for Hydro and Heat variables
void TemperatureOutputWriter::generateRowColFiles(DataOrganizer* organizer, Inputs* input)
{
	//define folder type for any variables that vary between BulkArea and GI
	int Folder_BulkArea_ID = 0;
	int Folder_GI_ID = 1;
	int row, col;
	int folder; 

	//cte 202403 Refactor to use DataDrawer in place of locationIndex

	//Loop through the DataDrawers, where the upper bound is based on the general DataDrawer attribute .size()
	//Note: DataDrawers contain all pixels, hence all rows and columns, of ASCII map
	//Note: Conversion between DataDrawer_ID and row, column, with DataDrawer_ID starting at 0, row & col starting at 1
	//Note: Conversion Eq: DataDrawer_ID = (row * col_max) + col
	//Note: Conversion Eq: row = [int(DataDrawer_ID/col_max)+1]; col = [mod(DataDrawer_ID,col_max)+1]; 
	for (int locationIndex = 0; locationIndex < organizer->DataDrawers.size(); ++locationIndex) {

		//Loop through the DataFolders in each DataDrawer[DataDrawer_ID], upper bound indicated by .size()
		//Note: Each DataDrawer contains only 1 BulkArea DataFolder for in HydroPlusConfig.xml for Model = SpatialTemperatureHydro 
		for (int DataFolder_ID = 0; DataFolder_ID < organizer->DataDrawers[locationIndex].size(); ++DataFolder_ID) {

			//Note: Conversion Eq: row = [int(DataDrawer_ID/col_max)+1]; col = [mod(DataDrawer_ID,col_max)+1]; DataDrawer_ID = (row * col_max) + col
			//row equals integer [(locationIndex / nCols) + 1], where location DataDrawer_ID is counter through all map pixels
			row = (int)(locationIndex / (int)(input->LocationParams["nCols"])) + 1;
			//col equals [modulus (locationIndex / nCols) + 1], where location DataDrawer_ID is counter through all map pixels
			col = (locationIndex % (int)(input->LocationParams["nCols"])) + 1;

			//If map pixel at locationIndex is not NODATA (defined by HydroPlusConfig.xml element SimulationLocationParams) then enter
			//Note: ParamStringDict["Type"] is string, set to NODATA or BulkArea, etc. The value of NODATA defined w input->LocationParams["NODATA"]
			if (organizer->DataDrawers[locationIndex][DataFolder_ID]->ParamStringDict["Type"] != "NODATA") {

				//locationDataIsToBeOutputted boolean is defined as 1 for true if HydroPlusConfig.xml TemperatureCalculationParams = PrintAllRowColOutput
				bool locationDataIsToBeOutputted = input->TemperatureCalculationParams["PrintAllRowColOutput"];

				//if locationDataIsToBeOutputted is 0 or false, then do not print all row and column pairs
				if (!locationDataIsToBeOutputted) {
					locationDataIsToBeOutputted = input->LocationIdsToPrintTempOutput.find(locationIndex) != input->LocationIdsToPrintTempOutput.end();
				}

				//if locationDataIsToBeOutputted is	1 or true, then print all row and column pairs
				if (locationDataIsToBeOutputted) {

					//If HydroPlusConfig.xml Type equals BulkArea then
					//Note: organizer->DataDrawers[locationIndex][DataFolder_ID] is path to folder
					if (organizer->DataDrawers[locationIndex][DataFolder_ID]->ParamStringDict["Type"] == "BulkArea") {
						//Folder_Type_ID equals zero, Folder_BulkArea_ID = 0
						Folder_Type_ID = Folder_BulkArea_ID;
					}
					//Else Type is GreenInfrastructure
					else {
						//Folder_Type_ID equals 1, Folder_GI_ID = 1
						Folder_Type_ID = Folder_GI_ID;
					}

					//createHeatRowColFile function creates Row#Col#Heat.csv file for writing during time step loop, aka RowColHeat or Row#Col#Heat
					createHeatRowColFile(row, col, input->SimulationStringParams["OutputFolder_Path"], input);
					//createHeatRowColFile function creates Row#Col#Hydro.csv file for writing during time step loop, aka RowColHydro or Row#Col#Hydro
					createHydroRowColFile(row, col, input->SimulationStringParams["OutputFolder_Path"], input);
				}
			}
			//Else If map pixel at locationIndex is NODATA then 
			else {
				//continue statement will skip the next execution statements below and begins next iteration of for loop for new row col pair
				continue;
			}
		}
	}
}

//TemperatureOutputWriter::rowColFileBatchProcessor function to coordinate writing cell-based values for Hydro and Heat variables
void TemperatureOutputWriter::rowColFileBatchProcessor(int timeStep, int DataDrawer_ID,
	DataFolder* folder, string OutputDirectory_String , Inputs* input) {

	//locationDataIsToBeOutputted defined as input->TemperatureCalculationParams["PrintAllRowColOutput"]
	bool locationDataIsToBeOutputted = input->TemperatureCalculationParams["PrintAllRowColOutput"];

	//If not locationDataIsToBeOutputted then enter to set locationDataIsToBeOutputted to value
	if (!locationDataIsToBeOutputted) {
		//locationDataIsToBeOutputted defined as True of False based on comparison of LocationIdsToPrintTempOutput
		locationDataIsToBeOutputted = input->LocationIdsToPrintTempOutput.find(DataDrawer_ID) != input->LocationIdsToPrintTempOutput.end();
	}
	//If not locationDataIsToBeOutputted, then the above assignment failed and exit function
	if (!locationDataIsToBeOutputted) {
		return;
	}

	//Note: Below the code is converting between map vector MapDataFolder_ID and pixel row, column ID
	//Note: Conversion between DataDrawer_ID and row, column, with DataDrawer_ID starting at 0, row & col starting at 1
	//Note: Conversion Eq: DataDrawer_ID = (row * col_max) + col
	//Note: Conversion Eq: row = [int(DataDrawer_ID/col_max)+1]; col = [mod(DataDrawer_ID,col_max)+1]; 
	int row = (int)(DataDrawer_ID / (int)(input->LocationParams["nCols"]) + 1);
	int col = (DataDrawer_ID % (int)(input->LocationParams["nCols"]) + 1);
	bool printLocation = input->TemperatureCalculationParams["RowColOutputAllTimeSteps"];

	//control to write out 1st time step, connects with SimulationCoordinator.cpp statement
	if (timeStep >= 0) {

		int SimulationDate_GDH;

		//SimulationDate_GDH is string created from SimulationDate_GDH (YYYYMMDDHH) of timeStep, vector defined in readWeatherFile
		SimulationDate_GDH = input->SimulationDate_GDH[timeStep];

		//SimulationDate_GDH_string is created from SimulationDate_GDH (YYYYMMDDHH)
		string SimulationDate_GDH_string = to_string(input->SimulationDate_GDH[timeStep]);

		//If Not printLocation then enter and see if printLocation should be changed to True based on dates
		if (!printLocation) {
			for (int iter = 0; iter < input->StartDatesForRowColTempOutput.size(); ++iter) {
				if (SimulationDate_GDH >= input->StartDatesForRowColTempOutput[iter] && 
					SimulationDate_GDH <= input->EndDatesForRowColTempOutput[iter]) {
					printLocation = true;
					break;
				}
			}
		}
		//If printLocation then enter and call writeToHeatRowColFile and writeToHydroRowColFile
		if (printLocation) {
			//writeToHeatRowColFile function writes to Row#Col#Heat.csv file during time step loop, aka RowColHeat or Row#Col#Heat
			writeToHeatRowColFile(input, folder, input->SimulationStringParams["OutputFolder_Path"], timeStep, SimulationDate_GDH, row, col);
			//writeToHeatRowColFile function writes to Row#Col#Heat.csv file during time step loop, aka RowColHydro or Row#Col#Hydro
			writeToHydroRowColFile(input, folder, input->SimulationStringParams["OutputFolder_Path"], timeStep, SimulationDate_GDH, row, col);
		}
	}
}

//TemperatureOutputWriter::createHydroRowColFile function writes cell-based Hydro output, aka RowColHydro or Row#Col#Hydro
void TemperatureOutputWriter::createHydroRowColFile(int row, int col, string OutputDirectory_String , Inputs* input) {

	//row is decreased by one to account for C++ vectors originating at zero, not one  
	row = row - 1;
	//row is decreased by one to account for C++ vectors originating at zero, not one  
	col = col - 1;
	// Construct the filename using std::string and std::to_string
	std::string filename = "\\Row" + std::to_string(row) + "Col" + std::to_string(col) + "Hydro.csv";
	std::string outputPath = OutputDirectory_String + filename;

	ofstream outfile(outputPath);

	if (!outfile.good()) {
		cout << "Warning: Output folder does not exist or an output file in that folder is open and cannot be overwritten." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Correction: Confirm the HydroPlusConfig.xml parameter OutputFolder_Path and files in that folder are closed." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}


	//MapPixel_ID is map vector location, i.e., DataDrawer_ID, computed as a function of col_max and pixel row, column ID
	//Note: Conversion Eq: MapPixel_ID = row * col_max + col, with MapPixel_ID starting at 0, row & col starting at 1
	//Note: Conversion Eq: row = [int(MapPixel_ID/col_max)+1]; col = [mod(MapPixel_ID,col_max)+1]
	int col_max = input->LocationParams["nCols"];
	int MapPixel_ID = row * col_max + col;
	//outfile writes pixel specific data variables
	outfile << "Elevation_m," << input->DEM[row * col_max + col] << endl;
	outfile << "SlopeGround_rad," << input->SlopeGround_rad[row * col_max + col] << endl;
	outfile << "AspectGround_N_0_rad," << input->AspectGround_N_0_rad[row * col_max + col] << endl;
	outfile << "Topographic_Index," << input->TI_Value[row * col_max + col] << endl;
	outfile << "TI_Basin_Average," << input->TI_Value_Average << endl;
	outfile << "NLCD_Class," << input->NLCDLandCoverData[row * col_max + col] << endl;
	outfile << "Tree_Cover_fraction," << (input->TreeOnPervious[row * col_max + col] + input->TreeOnImpervious[row * col_max + col]) << endl;
	outfile << "Impervious_Cover_fraction," << (input->ImperviousNoTree[row * col_max + col] + input->TreeOnImpervious[row * col_max + col]) << endl;
	outfile << "Impervious_NoTree_fraction," << input->ImperviousNoTree[row * col_max + col] << endl;
	

	//outfile receives header names for time series variables
	outfile << "SimulationDate_GDH,";
	outfile << "Interception_RainSnow_TreeCanopy_m,";
	outfile << "Storage_RainSnow_TreeCanopy_m,";
	outfile << "EvaporationSublimation_TreeCanopy_m,";
	outfile << "ThroughFall_RainSnow_TreeCanopy_m,";
	outfile << "Interception_RainSnow_SVegCanopy_m,";
	outfile << "Storage_RainSnow_SVegCanopy_m,";
	outfile << "EvaporationSublimation_SVegCanopy_m,";
	outfile << "ThroughFall_RainSnow_SVegCanopy_m,";
	outfile << "Flux_to_ImperviousArea_Rain_SnowMelt_m,";
	outfile << "Flux_Impervious_to_Pervious_m,";
	outfile << "Flux_to_PerviousArea_Rain_SnowMelt_ImpRunon_m,";
	outfile << "Flux_to_WaterArea_Rain_SnowMelt_m,";
	outfile << "Evaporation_ImperviousPondedWater_m,";
	outfile << "Evaporation_PerviousPondedWater_m,";
	outfile << "Evaporation_WaterPondedWater_m,";
	outfile << "Storage_ImperviousPondedWater_m,";
	outfile << "Storage_PerviousPondedWater_m,";
	outfile << "Storage_WaterPondedWater_m,";
	outfile << "Flux_to_Infiltration_m,";
	outfile << "Drainage_macroPore_m,";
	outfile << "Infiltration_m,";
	//EvapoTranspiration_TS_Sum_m (m) has header EvapoTranspiration_SoilEvapZone_m, only between Soil_FieldCapacity_m3pm3 and Soil_WiltingPoint_m3pm3
	outfile << "EvapoTranspiration_SoilEvapZone_m,";
	//Storage_SoilEvapZone_TS_Sum_m3_p_m3 (m3/m3) has header Storage_SoilEvapZone_m3pm3, only between Soil_FieldCapacity_m3pm3 and Soil_WiltingPoint_m3pm3
	outfile << "Storage_SoilEvapZone_m3pm3,";
	//Storage_SoilEvapZone_TS_Sum_m (m) has header Storage_SoilEvapZone_m, only between Soil_FieldCapacity_m3pm3 and Soil_WiltingPoint_m3pm3
	outfile << "Storage_SoilEvapZone_m,";
	//Drainage_VadoseZone_TS_Sum_m (m3) has header Drainage_VadoseZone_m, only between Soil_SaturationPoint_m3pm3 and Soil_FieldCapacity_m3pm3
	outfile << "Drainage_VadoseZone_m,";
	//StorageDeficit_VadoseZone_TS_Sum_m (m) has header StorageDeficit_VadoseZone_m, is storage capacity for gravitational water across entire catchment vadose zone
	outfile << "StorageDeficit_VadoseZone_m,";
	outfile << "Groundwater_surficial_frac,";
	outfile << "Runoff_Impervious_m,";
	outfile << "Runoff_Pervious_m,";
	outfile << "Runoff_Water_m,";
	outfile << "Runoff_Surface_m,";
	outfile << "Runoff_Subsurface_m,";
	outfile << "Runoff_m";
	outfile << endl;
	outfile.close();
}

//TemperatureOutputWriter::writeToHydroRowColFile function writes cell-based Hydro time series for folder row col pair, aka RowColHydro or Row#Col#Hydro
void TemperatureOutputWriter::writeToHydroRowColFile(Inputs* input, DataFolder* folder, string OutputDirectory_String , int timeStep, int SimulationDate_GDH, int row, int col) {
	//row is decreased by one to account for C++ vectors originating at zero, not one  
	row = row - 1;
	//row is decreased by one to account for C++ vectors originating at zero, not one  
	col = col - 1;
	// Construct the filename using std::string and std::to_string
	std::string filename = "\\Row" + std::to_string(row) + "Col" + std::to_string(col) + "Hydro.csv";
	std::string outputPath = OutputDirectory_String + filename;

	// Open the output file in append mode
	std::ofstream outfile(outputPath, std::ios::app);

	if (!outfile.good()) {
		cout << "Cannot write Row#Col#Hydro.csv file" << endl;
		return;
	}
	
	//Folder_Type_ID equal to zero, which is BulkArea folder
	int Folder_Type_ID = 0;

	outfile << SimulationDate_GDH << ",";
	outfile << folder->VarDict["Interception_RainSnow_TreeCanopy_m"] << ",";
	outfile << folder->VarDict["Storage_RainSnow_TreeCanopy_m"] << ",";
	outfile << folder->VarDict["EvaporationSublimation_TreeCanopy_m"] << ",";
	outfile << folder->VarDict["ThroughFall_RainSnow_TreeCanopy_m"] << ",";
	outfile << folder->VarDict["Interception_RainSnow_SVegCanopy_m"] << ",";
	outfile << folder->VarDict["Storage_RainSnow_SVegCanopy_m"] << ",";
	outfile << folder->VarDict["EvaporationSublimation_SVegCanopy_m"] << ",";
	outfile << folder->VarDict["ThroughFall_RainSnow_SVegCanopy_m"] << ",";
	outfile << folder->VarDict["Flux_to_ImperviousArea_Rain_SnowMelt_m"] << ",";
	outfile << folder->VarDict["Flux_Impervious_to_Pervious_m"] << ",";
	outfile << folder->VarDict["Flux_to_PerviousArea_Rain_SnowMelt_ImpRunon_m"] << ",";
	outfile << folder->VarDict["Flux_to_WaterArea_Rain_SnowMelt_m"] << ",";
	outfile << folder->VarDict["Evaporation_ImperviousPondedWater_m"] << ",";
	outfile << folder->VarDict["Evaporation_PerviousPondedWater_m"] << ",";
	outfile << folder->VarDict["Evaporation_WaterPondedWater_m"] << ",";
	outfile << folder->VarDict["Storage_ImperviousPondedWater_m"] << ",";
	outfile << folder->VarDict["Storage_PerviousPondedWater_m"] << ",";
	outfile << folder->VarDict["Storage_WaterPondedWater_m"] << ",";
	outfile << folder->VarDict["Flux_to_Infiltration_m"] << ",";
	outfile << folder->VarDict["Drainage_macroPore_m"] << ",";
	outfile << folder->VarDict["Infiltration_m"] << ",";
	outfile << folder->VarDict["EvapoTranspiration_SoilEvapZone_m"] << ",";
	//Storage_SoilEvapZone_TS_Sum_m3_p_m3 (m3/m3) is only between Theta_FieldCapacity and Theta_WiltingPoint
	outfile << folder->VarDict["Storage_SoilEvapZone_m3_p_m3"] << ",";
	outfile << folder->VarDict["Storage_SoilEvapZone_m"] << ",";
	outfile << folder->VarDict["Drainage_VadoseZone_m"] << ",";
	outfile << input->RepoDict["StorageDeficit_VadoseZone_m"] << ",";
	outfile << folder->VarDict["Groundwater_surficial_frac"] << ",";
	outfile << folder->VarDict["Runoff_Impervious_m"] << ",";
	outfile << folder->VarDict["Runoff_Pervious_m"] << ",";
	outfile << folder->VarDict["Runoff_Water_m"] << ",";
	outfile << folder->VarDict["Runoff_Surface_m"] << ",";
	outfile << folder->VarDict["Runoff_Subsurface_m"] << ",";
	outfile << folder->VarDict["Runoff_m"];
	outfile << endl;
	//outfile close function
	outfile.close();
}

//createHeatRowColFile function creates Row#Col#Heat.csv file for writing during time step loop, aka RowColHeat or Row#Col#Heat
void TemperatureOutputWriter::createHeatRowColFile(int row, int col, string OutputDirectory_String , Inputs* input) {
	//row is decreased by one to account for C++ vectors originating at zero, not one  
	row = row - 1;
	//row is decreased by one to account for C++ vectors originating at zero, not one  
	col = col - 1;
	
	std::string filename = "\\Row" + std::to_string(row) + "Col" + std::to_string(col) + "Heat.csv";
	std::string outputPath = OutputDirectory_String + filename;

	ofstream outfile(outputPath);

	if (!outfile.good()) {
		cout << "Warning: Output folder does not exist or an output file in that folder is open and cannot be overwritten." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Correction: Confirm the HydroPlusConfig.xml parameter OutputFolder_Path and files in that folder are closed." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//MapPixel_ID is map vector location, i.e., DataDrawer_ID, computed as a function of col_max and pixel row, column ID
	//Note: Conversion Eq: MapPixel_ID = row * col_max + col, with MapPixel_ID starting at 0, row & col starting at 1
	//Note: Conversion Eq: row = [int(MapPixel_ID/col_max)+1]; col = [mod(MapPixel_ID,col_max)+1]
	int col_max = input->LocationParams["nCols"];
	int MapPixel_ID = row * col_max + col;
	//outfile writes pixel specific data variables
	outfile << "Elevation_m," << input->DEM[row * col_max + col] << endl;
	outfile << "SlopeGround_rad," << input->SlopeGround_rad[row * col_max + col] << endl;
	outfile << "AspectGround_N_0_rad," << input->AspectGround_N_0_rad[row * col_max + col] << endl;
	outfile << "Topographic_Index," << input->TI_Value[row * col_max + col] << endl;
	outfile << "TI_Basin_Average," << input->TI_Value_Average << endl;
	outfile << "NLCD_Class," << input->NLCDLandCoverData[row * col_max + col] << endl;
	outfile << "Tree_Cover_fraction," << (input->TreeOnPervious[row * col_max + col] + input->TreeOnImpervious[row * col_max + col]) << endl;
	outfile << "Impervious_Cover_fraction," << (input->ImperviousNoTree[row * col_max + col] + input->TreeOnImpervious[row * col_max + col]) << endl;
	outfile << "Impervious_NoTree_fraction," << input->ImperviousNoTree[row * col_max + col] << endl;

	//outfile receives header names for time series variables
	outfile << "SimulationDate_GDH,";
	outfile << "Tair_K,";
	outfile << "Tdew_K,";
	outfile << "HeatIndex_K,";
	outfile << "Twetbulbglobe_K,";
	outfile << "AbsHumidity_kg_p_m3,";
	outfile << "ImpNR_W_p_m2,";
	outfile << "ImpH_W_p_m2,";
	outfile << "ImpLE_W_p_m2,";
	outfile << "ImpDeltaQ_W_p_m2,";
	outfile << "Imp_AH_W_p_m2,";
	outfile << "TreeNR_W_p_m2,";
	outfile << "TreeH_W_p_m2,";
	outfile << "TreeLE_W_p_m2,";
	outfile << "TreeLEE_W_p_m2,";
	outfile << "TreeLET_W_p_m2,";
	outfile << "TreeDeltaQ_W_p_m2,";
	outfile << "Tree_AH_W_p_m2,";
	outfile << "SVegNR_W_p_m2,";
	outfile << "SVegH_W_p_m2,";
	outfile << "SVegLE_W_p_m2,";
	outfile << "SVegLEE_W_p_m2,";
	outfile << "SVegLET_W_p_m2,";
	outfile << "SVegDeltaQ_W_p_m2,";
	outfile << "SVeg_AH_W_p_m2,";
	outfile << "SoilNR_W_p_m2,";
	outfile << "SoilH_W_p_m2,";
	outfile << "SoilLE_W_p_m2,";
	outfile << "SoilDeltaQ_W_p_m2,";
	outfile << "Soil_AH_W_p_m2,";
	outfile << "WaterNR_W_p_m2,";
	outfile << "WaterH_W_p_m2,";
	outfile << "WaterLE_W_p_m2,";
	outfile << "WaterDeltaQ_W_p_m2,";
	outfile << "Water_AH_W_p_m2,";
	outfile << "NR_W_p_m2,";
	outfile << "H_total_W_p_m2,";
	outfile << "LE_total_W_p_m2,";
	outfile << "DeltaQ_W_p_m2,";
	outfile << "AnthropogenicHeat_total_W_p_m2,";
	outfile << "WindSpeed_canopy_to_mesolayer_m_p_s,";
	outfile << "WindSpeed_folder_m_p_s,";
	outfile << "Resistance_Aerodynamic_CanopyToMesoLayer_s_p_m,";
	outfile << "Resistance_Aerodynamic_Impervious_s_p_m,";
	outfile << "Resistance_Aerodynamic_Tree_s_p_m,";
	outfile << "Resistance_Aerodynamic_SVeg_s_p_m,";
	outfile << "Resistance_Aerodynamic_Soil_s_p_m,";
	outfile << "Resistance_Aerodynamic_Water_s_p_m,";
	outfile << "Resistance_Surface_Tree_s_p_m,";
	outfile << "Resistance_Surface_SVeg_s_p_m,";
	outfile << "Resistance_Surface_Soil_s_p_m,";
	outfile << "Tair_mesoScale_K,";
	outfile << "AbsHumidity_mesoScale_kg_p_m3,";

	outfile << "SurfaceSlopeAngle_rad,";
	outfile << "SurfaceAzimuthAngle_rad,";
	outfile << "LatitudeAngle_rad,";
	outfile << "DeclinationAngle_rad,";
	outfile << "HourAngle_Solar_rad,";
	outfile << "ZenithAngle_rad,";
	outfile << "Cos_SolarAzimuthAngle_Reda,";
	outfile << "Cos_IncidenceAngle_Reda";
	outfile << endl;
	outfile.close();
}

//TemperatureOutputWriter::writeToHeatRowColFile function writes cell-based Heat time series for folder row col pair, aka RowColHeat or Row#Col#Heat
void TemperatureOutputWriter::writeToHeatRowColFile(Inputs* input, DataFolder* folder, string OutputDirectory_String , int timeStep, int SimulationDate_GDH, int row, int col)
{
	//row is decreased by one to account for C++ vectors originating at zero, not one  
	row = row - 1;
	//row is decreased by one to account for C++ vectors originating at zero, not one  
	col = col - 1;
	//str character array created to contain upwards of 40 items
	char str[40];
	char str1[10];
	char str2[10];
	//strcpy function copies string source Row to character array destination str
	strcpy(str, "\\Row");
	//itoa function converts int row number to string in character array str1 using base 10 (e.g., supports decimal, negative)
	itoa(row, str1, 10);
	//itoa function converts int col number to string in character array str2 using base 10 (e.g., supports decimal, negative)
	itoa(col, str2, 10);
	//strcat function catenates str and str1
	strcat(str, str1);
	//strcat function catenates str and "Col" string
	strcat(str, "Col");
	//strcat function catenates str and str2
	strcat(str, str2);
	//str is catenated with Hydro.csv
	strcat(str, "Heat.csv");
	//ofstream opens outfile OutputDirectory_String  + str using ios::app function to append below existing content in outfile
	ofstream outfile(OutputDirectory_String  + str, ios::app);

	if (!outfile.good()) {
		cout << "Cannot write Row#Col#Heat.csv file" << endl;
		return;
	}

	//MapPixel_ID is map vector location, i.e., DataDrawer_ID, computed as a function of col_max and pixel row, column ID
	//Note: Conversion Eq: MapPixel_ID = row * col_max + col, with MapPixel_ID starting at 0, row & col starting at 1
	//Note: Conversion Eq: row = [int(MapPixel_ID/col_max)+1]; col = [mod(MapPixel_ID,col_max)+1]
	int col_max = input->LocationParams["nCols"];
	int MapPixel_ID = row * col_max + col;

	//outfile receives variables to cell-based output
	outfile << SimulationDate_GDH << ",";
	outfile << folder->VarDict["Tair_K"] << ",";
	outfile << folder->VarDict["Tdew_K"] << ",";
	outfile << folder->VarDict["HeatIndex_K"] << ",";
	outfile << folder->VarDict["Twetbulbglobe_K"] << ",";
	outfile << folder->VarDict["AbsHumidity_kg_p_m3"] << ",";
	outfile << folder->VarDict["ImpNR_W_p_m2"] << ",";
	outfile << folder->VarDict["ImpH_W_p_m2"] << ",";
	outfile << folder->VarDict["ImpLE_W_p_m2"] << ",";
	outfile << folder->VarDict["ImpDeltaQ_W_p_m2"] << ",";
	outfile << folder->VarDict["Imp_AH_W_p_m2"] << ",";
	outfile << folder->VarDict["TreeNR_W_p_m2"] << ",";
	outfile << folder->VarDict["TreeH_W_p_m2"] << ",";
	outfile << folder->VarDict["TreeLE_W_p_m2"] << ",";
	outfile << folder->VarDict["TreeLEE_W_p_m2"] << ",";
	outfile << folder->VarDict["TreeLET_W_p_m2"] << ",";
	outfile << folder->VarDict["TreeDeltaQ_W_p_m2"] << ",";
	outfile << folder->VarDict["Tree_AH_W_p_m2"] << ",";
	outfile << folder->VarDict["SVegNR_W_p_m2"] << ",";
	outfile << folder->VarDict["SVegH_W_p_m2"] << ",";
	outfile << folder->VarDict["SVegLE_W_p_m2"] << ",";
	outfile << folder->VarDict["SVegLEE_W_p_m2"] << ",";
	outfile << folder->VarDict["SVegLET_W_p_m2"] << ",";
	outfile << folder->VarDict["SVegDeltaQ_W_p_m2"] << ",";
	outfile << folder->VarDict["SVeg_AH_W_p_m2"] << ",";
	outfile << folder->VarDict["SoilNR_W_p_m2"] << ",";
	outfile << folder->VarDict["SoilH_W_p_m2"] << ",";
	outfile << folder->VarDict["SoilLE_W_p_m2"] << ",";
	outfile << folder->VarDict["SoilDeltaQ_W_p_m2"] << ",";
	outfile << folder->VarDict["Soil_AH_W_p_m2"] << ",";
	outfile << folder->VarDict["WaterNR_W_p_m2"] << ",";
	outfile << folder->VarDict["WaterH_W_p_m2"] << ",";
	outfile << folder->VarDict["WaterLE_W_p_m2"] << ",";
	outfile << folder->VarDict["WaterDeltaQ_W_p_m2"] << ",";
	outfile << folder->VarDict["Water_AH_W_p_m2"] << ",";
	outfile << folder->VarDict["NR_total_W_p_m2"] << ",";
	outfile << folder->VarDict["H_total_W_p_m2"] << ",";
	outfile << folder->VarDict["LE_total_W_p_m2"] << ",";
	outfile << folder->VarDict["DeltaQ_W_p_m2"] << ",";
	outfile << folder->VarDict["AH_total_W_p_m2"] << ",";
	outfile << folder->VarDict["WindSpeed_canopy_to_mesolayer_m_p_s"] << ",";
	outfile << folder->VarDict["WindSpeed_folder_m_p_s"] << ",";
	outfile << folder->VarDict["Resistance_Aerodynamic_CanopyToMesoLayer_s_p_m"] << ",";
	outfile << folder->VarDict["Resistance_Aerodynamic_Impervious_s_p_m"] << ",";
	outfile << folder->VarDict["Resistance_Aerodynamic_Tree_s_p_m"] << ",";
	outfile << folder->VarDict["Resistance_Aerodynamic_SVeg_s_p_m"] << ",";
	outfile << folder->VarDict["Resistance_Aerodynamic_Soil_s_p_m"] << ",";
	outfile << folder->VarDict["Resistance_Aerodynamic_Water_s_p_m"] << ",";
	outfile << folder->VarDict["Resistance_Surface_Tree_s_p_m"] << ",";
	outfile << folder->VarDict["Resistance_Surface_SVeg_s_p_m"] << ",";
	outfile << folder->VarDict["Resistance_Surface_Soil_s_p_m"] << ",";
	outfile << folder->VarDict["Tair_mesoScale_final_K"] << ",";
	outfile << folder->VarDict["AbsHumidity_mesoScale_final_kg_p_m3"] << ",";
	outfile << input->SlopeGround_rad[MapPixel_ID] << ",";
	outfile << input->AspectGround_N_0_rad[MapPixel_ID] << ",";
	outfile << input->Latitude_rad[timeStep] << ",";
	outfile << input->DeclinationAngle_Solar_rad[timeStep] << ",";
	outfile << input->HourAngle_Solar_rad[timeStep] << ",";
	outfile << folder->VarDict["ZenithAngle_Solar_rad"] << ",";
	outfile << cos(folder->VarDict["AzimuthAngle_Solar_N_0_rad"]) << ",";
	//Note: Confirming Equivalence between solar angles of Yang et al (2013) and Reda et al (2003) 
	//outfile << folder->VarDict["Cos_IncidenceAngle_Yang"] << ",";
	outfile << folder->VarDict["Cos_IncidenceAngle_Solar"];
	outfile << endl;
	outfile.close();
}

//TemperatureOutputWriter::createTimeStepResultFile function writes time-based map output header rows suitable for ArcGIS ASCII map .asc file
void TemperatureOutputWriter::createTimeStepResultFile(string outputVarName, int ts, Inputs* input)
{
	//string fName created by combining OutputDirectory_String , outputVarName, _, and to_string function of ts, ending with .asc
	string fName = input->SimulationStringParams["OutputFolder_Path"] + outputVarName + "_" + to_string(ts) + ".asc";
	//ofstream opens outfile fName to write header information
	ofstream outfile(fName);

	if (!outfile.good()) {
		cout << "Warning: Output folder does not exist or an output file in that folder is open and cannot be overwritten." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Correction: Confirm the HydroPlusConfig.xml parameter OutputFolder_Path and files in that folder are closed." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//ncols written to extend to length of ncols with width of 20, using width of 20 
	outfile << "ncols" << setw(20) << input->LocationParams["nCols"] << endl;
	//nrows written to extend to length of ncols with width of 20, using width of 20 
	outfile << "nrows" << setw(20) << input->LocationParams["nRows"] << endl;
	//xllcorner written to extend to length of ncols with width of 20, using width of 16
	outfile << "xllcorner" << setw(16) << input->LocationParams["xllcorner"] << endl;
	//yllcorner written to extend to length of ncols with width of 20, using width of 16 
	outfile << "yllcorner" << setw(16) << input->LocationParams["yllcorner"] << endl;
	//cellsize written to extend to length of ncols with width of 20, using width of 17 
	outfile << "cellsize" << setw(17) << input->LocationParams["cellsize"] << endl;
	//NODATA_value written to extend to length of ncols with width of 20, using width of 13 
	outfile << "NODATA_value" << setw(13) << input->LocationParams["NODATA"] << endl;
	outfile.close();
}

//TemperatureOutputWriter::writeToTimeStepResultFile function writes time-based map output values below header suitable for ArcGIS ASCII map .asc file
void TemperatureOutputWriter::writeToTimeStepResultFile(string outputVarName, int ts, string& OutputDirectory_String , DataOrganizer* organizer, Inputs* input)
{
	//string fName created by combining OutputDirectory_String , outputVarName, _, and to_string function of ts, ending with .asc
	//Note: The ASCII .asc file created with a header in createTimeStepResultFile function is re-opened here for writing values
	string fName = OutputDirectory_String  + outputVarName + "_" + to_string(ts) + ".asc";
	//ofstream opens outfile fName with ios::app function to append
	ofstream outfile(fName, ios::app);

	if (!outfile.good()) {
		cout << "Cannot write file: " + fName << endl;
		return;
	}
	//For Loop through the DataDrawers.size(), where the upper bound is based on the nrows * ncols, number of map pixels
	//Note: DataDrawers.size() is number of map pixels in dem.asc, equal to nrows * ncols for SpatialTemperatureHydro; 1 for StatisticalHydro 
	for (int DataDrawer_ID = 0; DataDrawer_ID < organizer->DataDrawers.size(); ++DataDrawer_ID)	{

		//DataFolder class creates pointer to folder
		//Note: Process required to access DataFolder class when folder cannot be passed to TemperatureOutputWriter.cpp functions
		DataFolder* folder;

		//If input vector NLCDLandCoverData at DataDrawer_ID is not equal to NODATA, then enter for analysis
		//Note: Conditional required to avoid entering for SpatialTemperatureHydro for NODATA, which has no DataFolder 
		if (input->NLCDLandCoverData[DataDrawer_ID] != input->LocationParams["NODATA"]) {
			//folder is pointer to new DataFolder class for SpatialTemperatureHydro model
			folder = organizer->DataDrawers[DataDrawer_ID][0];
		}

		//Note: Below the code is converting between map vector DataDrawer_ID and pixel row, col pair
		//Note: Conversion between DataDrawer_ID and row, column, with DataDrawer_ID starting at 0, row & col starting at 1
		//Note: Conversion Eq: DataDrawer_ID = (row * col_max) + col
		//Note: Conversion Eq: row = [int(DataDrawer_ID/col_max)+1]; col = [mod(DataDrawer_ID,col_max)+1]
		int row = (int)(DataDrawer_ID / (int)(input->LocationParams["nCols"]) + 1);
		int col = (DataDrawer_ID % (int)(input->LocationParams["nCols"]) + 1);
		//newLine is boolean variable set to false or true depending on outcome of ternary operator, testing condition col not equal to nCols
		bool newLine = (col != (int)(input->LocationParams["nCols"])) ? false : true;
		
		//If NLCDLandCoverData[DataDrawer_ID] equals NODATA value from HydroPlusConfig.xml then
		if (int(input->NLCDLandCoverData[DataDrawer_ID]) == input->LocationParams["NODATA"]) {
			//write NODATA to map
			outfile << "-9999";
		}
		//Else If write the HydroPlus variable to the map output
		else {
			//write VarDict outputVarName variable value to map
			//Note: Any HydroPlus variable can be written to map
			outfile << folder->VarDict[outputVarName];
		}
		//If newLine boolean is true then
		if (newLine) {
			//outfile receives endl function for end of line return
			outfile << endl;
		}
		//Else If then newLine boolen is false
		else {
			//outfile receives space between variables
			outfile << " ";
		}
	}
	//outfile uses close function
	outfile.close();
}

//TemperatureOutputWriter::generate_BlockGroup_Files function coordinates block group averages
void TemperatureOutputWriter::generate_BlockGroup_Files(map<int, DataFolder*> blockGroupIDToItsLocation, string OutputDirectory_String , bool needHourlyBGFiles, bool createDailyFiles)
{
	//For Loop from blockGroupIDToItsLocation.begin() through blockGroupIDToItsLocation.end() using iter as counter
	for (map<int, DataFolder*>::iterator iter = blockGroupIDToItsLocation.begin(); iter != blockGroupIDToItsLocation.end(); ++iter) {
		//If needHourlyBGFiles is true then
		if (needHourlyBGFiles) {
			//call createBlockGroupHourlyFile function
			createBlockGroupHourlyFile(OutputDirectory_String , iter->first);
		}
		//If createDailyFiles is true then
		if (createDailyFiles) {
			//call createBlockGroupDailyFile function
			createBlockGroupDailyFile(OutputDirectory_String , iter->first);
		}
	}
}

//TemperatureOutputWriter::createBlockGroupHourlyFile function writes hourly block group header
void TemperatureOutputWriter::createBlockGroupHourlyFile(string OutputDirectory_String , int blockGroupIndex)
{
	//FileName_str contains BlockGroup defined as string, appended to blockGroupIndex (converted to string) and .csv to create file name
	string FileName_str = "BlockGroup" + to_string(blockGroupIndex) + ".csv";
	ofstream outfile(OutputDirectory_String  + FileName_str);

	if (!outfile.good()) {
		cout << "Warning: Output folder does not exist or an output file in that folder is open and cannot be overwritten." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Correction: Confirm the HydroPlusConfig.xml parameter OutputFolder_Path and files in that folder are closed." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	//Header written as YYYYMMDDHH,Tair_K,Tdew_K,HeatIndex_K,Twetbulbglobe_K
	outfile << "YYYYMMDDHH,Tair_K,Tdew_K,HeatIndex_K,Twetbulbglobe_K" << endl;
	outfile.close();
}

//TemperatureOutputWriter::writeToBlockGroupHourlyFile function writes hourly block group variable values
void TemperatureOutputWriter::writeToBlockGroupHourlyFile(string OutputDirectory_String , string date, DataFolder* blockGroupFolder)
{
	int blockGroup_ID = static_cast<int>(blockGroupFolder->ParamDict["blockGroupIndex"]);
	//FileName_str contains BlockGroup defined as string, appended to blockGroup_ID (converted to string) and .csv to create file name
	string FileName_str = "BlockGroup" + to_string(blockGroup_ID) + ".csv";

	ofstream outfile(OutputDirectory_String  + FileName_str, ios::app);

	if (!outfile.good()) {
		cout << "Warning: Output folder does not exist or an output file in that folder is open and cannot be overwritten." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Correction: Confirm the HydroPlusConfig.xml parameter OutputFolder_Path and files in that folder are closed." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	outfile << date << ",";
	outfile << blockGroupFolder->VarDict["Tair_K"] / blockGroupFolder->ParamDict["folderCount"] << ",";
	outfile << blockGroupFolder->VarDict["Tdew_K"] / blockGroupFolder->ParamDict["folderCount"] << ",";
	outfile << blockGroupFolder->VarDict["HeatIndex_K"] / blockGroupFolder->ParamDict["folderCount"] << ",";
	outfile << blockGroupFolder->VarDict["Twetbulbglobe_K"] / blockGroupFolder->ParamDict["folderCount"];
	outfile << endl;
}

//TemperatureOutputWriter::createBlockGroupDailyFile function writes daily block group header
void TemperatureOutputWriter::createBlockGroupDailyFile(string OutputDirectory_String , int blockGroupIndex)
{
	//FileName_str contains BlockGroup defined as string, appended to blockGroupIndex (converted to string) and .csv to create file name
	string FileName_str = "BlockGroupDaily" + to_string(blockGroupIndex) + ".csv";
	ofstream outfile(OutputDirectory_String  + FileName_str);

	if (!outfile.good()) {
		cout << "Warning: Output folder does not exist or an output file in that folder is open and cannot be overwritten." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Correction: Confirm the HydroPlusConfig.xml parameter OutputFolder_Path and files in that folder are closed." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//HeatIndex terms are from Rothfusz, Lans P. National Weather Service Technical Attachment SR 90-23 (1990) 
	//HeatIndex_Lin terms are from Lin, S., M. Luo, R. J. Walker, X. Liu, S. A. Hwang and R. Chinery (2009)

	outfile << "YYYYMMDD,"; 
	outfile << "Tair_Max_K,"; 
	outfile << "Tair_Min_K,"; 
	outfile << "Tair_Avg_K,";
	outfile << "Tdew_Avg_K,";
	outfile << "HeatIndex_Max_K,";
	outfile << "HeatIndex_Avg_K,"; 
	outfile << "Twetbulbglobe_Max_K,";
	outfile << "Twetbulbglobe_Avg_K,";
	outfile << "HeatIndex_Lin_Max_K,";
	outfile << "HeatIndex_Lin_Avg_K,";
	outfile << "HeatIndex_Lin_AvgDaily_K,"; 
	outfile << "WindChill_Min_K,"; 
	outfile << "WindChill_Avg_K";
	outfile << endl;
	outfile.close();
}

//TemperatureOutputWriter::writeToBlockGroupDailyFile function writes daily block group variable value
void TemperatureOutputWriter::writeToBlockGroupDailyFile(Inputs* input, string OutputDirectory_String , string date, DataFolder* blockGroupFolder, int timeStep)
{
	int blockGroup_ID = static_cast<int>(blockGroupFolder->ParamDict["blockGroupIndex"]);
	//FileName_str contains BlockGroup defined as string, appended to blockGroup_ID (converted to string) and .csv to create file name
	string FileName_str = "BlockGroupDaily" + to_string(blockGroup_ID) + ".csv";
	ofstream outfile(OutputDirectory_String  + "//" + FileName_str, ios::app);

	if (!outfile.good()) {
		cout << "Warning: Output folder does not exist or an output file in that folder is open and cannot be overwritten." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Correction: Confirm the HydroPlusConfig.xml parameter OutputFolder_Path and files in that folder are closed." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//HeatIndex_Lin_blockGroupAverageDaily_K (K) terms are from Lin et al. (2009) calculated with daily average Tair_K and Tdew_K
	//HeatIndex_Lin_blockGroupAverage_K (K) by contrast uses average of hourly heat index values, within 0.1% of HeatIndex_Lin_blockGroupAverageDaily_K 
	double HeatIndex_Lin_blockGroupAverageDaily_K = -2.653 + 0.994 * ((blockGroupFolder->VarDict["Tair_blockGroupAverage_K"] / input->timeSteps_in_Day) - 273.15) + 0.0153 * pow(((blockGroupFolder->VarDict["Tdew_blockGroupAverage_K"] / input->timeSteps_in_Day) - 273.15), 2);

	//BlockGroupDaily written, with averages (e.g., Tair_blockGroupAverage_K) divided by timeSteps_in_Day, updated in BLockGroup Handler
	outfile << date << ","
		<< blockGroupFolder->VarDict["Tair_blockGroupMax_K"] << ","
		<< blockGroupFolder->VarDict["Tair_blockGroupMin_K"] << ","
		<< blockGroupFolder->VarDict["Tair_blockGroupAverage_K"] / input->timeSteps_in_Day << ","
		<< blockGroupFolder->VarDict["Tdew_blockGroupAverage_K"] / input->timeSteps_in_Day << ","
		<< blockGroupFolder->VarDict["HeatIndex_blockGroupMax_K"] << ","
		<< blockGroupFolder->VarDict["HeatIndex_blockGroupAverage_K"] / input->timeSteps_in_Day << ","
		<< blockGroupFolder->VarDict["Twetbulbglobe_blockGroupMax_K"] << ","
		<< blockGroupFolder->VarDict["Twetbulbglobe_blockGroupAverage_K"] / input->timeSteps_in_Day << ","
		<< blockGroupFolder->VarDict["HeatIndex_Lin_blockGroupMax_K"] << ","
		<< blockGroupFolder->VarDict["HeatIndex_Lin_blockGroupAverage_K"] / input->timeSteps_in_Day << ","
		<< (HeatIndex_Lin_blockGroupAverageDaily_K + 273.15) << ","
		<< blockGroupFolder->VarDict["WindChill_blockGroupMin_K"] << ","
		<< blockGroupFolder->VarDict["WindChill_blockGroupAverage_K"] / input->timeSteps_in_Day << endl;

	outfile.close();
}