#Python script to update elements in the HydroPlusConfig.xml file to the latest revision
#Created winter 2023 by T. Endreny, SUNY ESF, te@esf.edu 

#Inputs: Must define python variables: 
#path_root is the path to the folder where a recursive search for files begins
#Note: To only append lines below the target and not remove the target, set flag_replace_w_element_and_value = 1 and flag_append_to_target = 1
#Note: To remove the target, set flag_replace_w_element_and_value = 1 and flag_append_to_target = 0
#Note: To change the target and value, set flag_replace_w_element_and_value = 1 and flag_append_to_target = 1 and include str_out_line_01 for new target text

#flag_append_to_target is a flag to decide if existing element is kept when new elements appended or removed when new elements appended
#flag_append_differently_bulkArea_vs_GI is flag to decide if new elements appended are different for Bulk Area and GI folders
#str_element_old is existing element in HydroPlusConfig.xml, and is either: replaced or served as marker below new element
#str_element_new is new element for HydroPlusConfig.xml, and is either: replacing str_element_old or written above str_element_old
#cnt_out_lines defines number of new lines of data to add; there must be a str_out_line_0# to equal to cnt
#str_out_line_01 is the 1st element to add to HydroPlusConfig.xml
#str_out_line_01# is the cnt_out_lines element to add to HydroPlusConfig.xml

#Consider simple replace of text, to avoid replacing element values
# # Read in the file
# with open('file.txt', 'r') as file :
  # filedata = file.read()
# # Replace the target string
# filedata = filedata.replace('abcd', 'ram')
# # Write the file out again
# with open('file.txt', 'w') as file:
  # file.write(filedata)

#csv support tools
import csv
#directory support tools
import os
import re

#path_root defined as starting folder for recursive search
path_root = "C:\\iTree\\HydroPlus\\TestingFilesAndScript"
#path_root = "C:\\iTree\\a_prep_TIGER_DEM_NLCD\\a_scripts"
#path_root = "C:\\iTree\\projects"
#path_root = "C:\\iTree\\projects\Hydro"
#path_root = "C:\\iTree"\"
#path_root = "C:\\iTree\\projects\\CoolAir\\was_dc_21"

#ext defines extensions of files that will be queried for updates, where .tpl is PEST template file
ext = (".xml", ".tpl")
str_file_tpl = 'HydroPlusConfig.xml.tpl'
len_file_tpl = len(str_file_tpl)
str_file_xml = 'HydroPlusConfig.xml'
len_file_xml = len(str_file_xml)
str_file_template_xml = 'Template.xml'
len_file_template_xml = len(str_file_template_xml)

print("Path for recursive search is {}".format(path_root))
print("File is {}".format(str_file_xml))

#Note: To only append lines below the target and not remove the target, set flag_replace_w_element_and_value = 1 and flag_append_to_target = 1
#Note: To remove the target, set flag_replace_w_element_and_value = 1 and flag_append_to_target = 0
#Note: To change the target and value, set flag_replace_w_element_and_value = 1 and flag_append_to_target = 1 and include str_out_line_01 for new target text

#Default settings:
#flag_replace_w_element_and_value; =1 then target element string and value replaced if flag_append_to_target = 0, =0 then only target element string replaced
flag_replace_w_element_and_value = 1
#flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
flag_append_to_target = 1
#flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
#Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
flag_append_differently_bulkArea_vs_GI = 0

#define str_element_old list, which are existing elements that new elements will either a) replace or b) be written above
#Note: For some applications, it is best to use the full list of variables below, for others use a shortened list
str_element_old_list = ['<Flag_CoolAir_AnthropogenicHeat>']
#str_element_old_list = ['<Flag_CoolAir_AnthropogenicHeat>','<Vault_Thickness_m>','<Vault_Outlet_Delay_hr>','<Water_Diurnal_min_frac>','<a3_OHM_Water_W_p_m2>','<RefWeatherLocationImperviousCover_percent>','<OutputDirectory>','<Model>','<FlowPathAlgorithm>','<Infiltration>','<OutputTimeStep>','<CalibrationTimeStep>','<RefParamFolder>','StartDay>','EndDay>','Td_h>','Q0_mph>','T0_m2ph>','<m>','</m>','<!-- Scale parameter of soil transmissivity-->','<n>','DCIA_frac>','K0_mph>','WFS_m>','PAQ_RT_A_h>','PAQ_RT_B_h>','DCIAQ_RT_A_h>','DCIAQ_RT_B_h>','SSQ_RT_h>','LeafOnDay>','LeafOffDay>','LeafTransDays>','TreeLAI>','ShortVegLAI>','TreeBAI>','ShrubBAI>','EvergreenTreeCover_frac>','EverGreenShrubCover_frac>','<TreeLeafStorage_mm>','WeatherWindHeight>','Flag_UrbanCanyon>','Treeheight>','Treealbedo>','Treeemissivity>','TreeLeafWidth>','TreeMinimumResistance>','ShortVegetationheight>','ShortVegetationalbedo>','ShortVegetationemissivity>','ShortVegetationLeafWidth>','ShortVegetationMinimumResistance>','Soilheight>','Soilalbedo>','Soilemissivity>','Waterheight>','Wateralbedo>','Wateremissivity>','Urban21height>','Urban21roadwidth>','Urban21albedo>','Urban21emissivity>','Urban21buildingP>','Urban22height>','Urban22roadwidth>','Urban22albedo>','Urban22emissivity>','Urban22buildingP>','Urban23height>','Urban23roadwidth>','Urban23albedo>','Urban23emissivity>','Urban23buildingP>','Urban24height>','Urban24roadwidth>','Urban24albedo>','Urban24emissivity>','Urban24buildingP>','<RefWeatherLocationTopographicIndex>','<!-- Date format: YYYYMMDD; Must be date contained in meteorological input files-->','<!-- Date format: YYYYMMDD; Used to define TotalTimeSteps = [(EndDay+1)- StartDay] * 24-->','<Soil_Macropore_frac>','<Evapotranspiration_Depth_m>','<PerviousDepressionStorage_mm>','<ImperviousDepressionStorage_mm>','<Flag_TI_PerviousOnly>']

for str_element_old in str_element_old_list:

    if str_element_old == '<Flag_CoolAir_AnthropogenicHeat>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced if flag_append_to_target, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 1
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 4
        str_element_new = '<Flag_CoolAir_ImperviousCoverAdjust>'
        #define new element lines
        str_out_line_01 = "<Flag_CoolAir_ImperviousCoverAdjust>0</Flag_CoolAir_ImperviousCoverAdjust><!--Options: 0-3; 0=base;1=base+TCI;2=alternative;3=alternative+TCI; TCI=TreeCanopyCover_overImpervious_frac-->"
        str_out_line_02 = "<Scenario_CoolAir_Base_TCI_to_TC_frac>0.0</Scenario_CoolAir_Base_TCI_to_TC_frac><!--Options: Base case TreeCanopyCover_overImpervious_frac to compare with alternative-->"
        str_out_line_03 = "<Scenario_CoolAir_Alternative_TC_frac>0.0</Scenario_CoolAir_Alternative_TC_frac><!--Options: Alternative case tree cover minimum fractional value for urban NLCD classes-->"
        str_out_line_04 = "<Scenario_CoolAir_Alternative_IC_frac>1.0</Scenario_CoolAir_Alternative_IC_frac><!--Options: Alternative case impervious cover maximum fractional value for urban NLCD classes-->"
    if str_element_old == '<Vault_Thickness_m>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced if flag_append_to_target = 0, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 1
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 1
        str_element_new = '<Vault_StorageInitial_m>'
        #define new element lines
        str_out_line_01 = "<Vault_StorageInitial_m>0.0</Vault_StorageInitial_m><!--Options: >=0; initial vault storage level; units m-->"
    if str_element_old == '<Vault_Outlet_Delay_hr>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced if flag_append_to_target = 0, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 1
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 1
        str_element_new = '<Vault_ExfiltrationRate_mph>'
        #define new element lines
        str_out_line_01 = "<Vault_ExfiltrationRate_mph>0.0</Vault_ExfiltrationRate_mph><!--Options: >= 0; alternative to BulkArea Soil_Ksat_mph for seepage from vault; units m/hr-->"
    if str_element_old == '<Water_Diurnal_min_frac>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced if flag_append_to_target = 0, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 1
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 1
        str_element_new = '<SVeg_in_Gap_DevelopedArea_frac>'
        #define new element lines
        str_out_line_01 = "<SVeg_in_Gap_DevelopedArea_frac>0.5</SVeg_in_Gap_DevelopedArea_frac><!--Options: 0 to 1; SVeg if TC+IC leave a gap else Soil; units frac-->"
    if str_element_old == '<a3_OHM_Water_W_p_m2>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced if flag_append_to_target = 0, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 1
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 3
        str_element_new = '<Flag_Water_Diurnal_pattern>'
        #define new element lines
        str_out_line_01 = "<Flag_Water_Diurnal_pattern>0</Flag_Water_Diurnal_pattern><!--Options: 0 or 1; 0=unregulated; 1=regulates evaporative flux-->"
        str_out_line_02 = "<Water_Diurnal_max_frac>0.15</Water_Diurnal_max_frac><!--Options: 0 to 1; regulates evaporative flux; units frac-->"
        str_out_line_03 = "<Water_Diurnal_min_frac>0.01</Water_Diurnal_min_frac><!--Options: 0 to Water_Diurnal_max_frac; regulates evaporative flux; units frac-->"
    if str_element_old == '<RefWeatherLocationImperviousCover_percent>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 1
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 1
        str_element_new = '<RefWeatherLocationAnthropogenicHeatMax_Wpm2>'
        #define new element lines
        str_out_line_01 = "<RefWeatherLocationAnthropogenicHeatMax_Wpm2>2.0</RefWeatherLocationAnthropogenicHeatMax_Wpm2><!--Options: >=0; Anthropogenic heat max; W/m2-->"
    if str_element_old == '<OutputDirectory>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'OutputFolder_Path>'
    if str_element_old == '<Model>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Model_Selection>'
    if str_element_old == '<Infiltration>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Algorithm_SoilKsatDecay>'
    if str_element_old == '<FlowPathAlgorithm>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Algorithm_FlowDirection>'

    if str_element_old == '<OutputTimeStep>':
        #Note: Replace with TimeStep_CompareToObservedRunoff_str; Change HydroPlus.sln keywords Daily, Hourly, Weekly, Monthly to drop "ly"
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 0
        str_element_new = '<NotNeeded>'
    if str_element_old == '<CalibrationTimeStep>':
        #Note: Replace with Flag_CompareToObservedRunoff
        #Note: Must search "CalibrationTimeStep" and replace associated instances of "NoCalibration" with "0", and in the convertToTimeStep function, replace associated instances of "CalibrationTimeStep" with "OutputTimeStep"
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 0
        str_element_new = '<NotNeeded>'
    if str_element_old == '<RefParamFolder>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        str_new = '<ReferenceParameters_DrawerID_FolderID>'
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 3
        str_element_new = '<Flag_CompareToObservedRunoff>'
        #define new element lines
        str_out_line_01 = "<Flag_CompareToObservedRunoff>0</Flag_CompareToObservedRunoff><!--Options: Compare 0=Nothing; 1=Routed runoff; 2=Unrouted runoff-->"
        str_out_line_02 = "<TimeStep_CompareToObservedRunoff_str>Hour</TimeStep_CompareToObservedRunoff_str><!--Options: Hour | Day | Week | Month-->"
        str_out_line_03 = "<ReferenceParameters_DrawerID_FolderID>0,0</ReferenceParameters_DrawerID_FolderID><!--Options: >=0; ID of DataDrawer & DataFolder w/ Reference parameters-->"
        
    if str_element_old == 'StartDay>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'StartDate_YYYYMMDD>'
    if str_element_old == 'EndDay>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'StopDate_YYYYMMDD>'
    if str_element_old == '<!-- Date format: YYYYMMDD; Must be date contained in meteorological input files-->':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = '<!--Options: YYYYMMDD or YYYYMMDDHH; Must agree with meteorological input files-->'
    if str_element_old == '<!-- Date format: YYYYMMDD; Used to define TotalTimeSteps = [(EndDay+1)- StartDay] * 24-->':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = '<!--Options: YYYYMMDD or YYYYMMDDHH; Stop date is included in TotalTimeSteps; -->'

    if str_element_old == 'Td_h>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'VadoseZone_Drainage_Delay_h>'
    if str_element_old == 'Q0_mph>':
        #Note: C++ change input->RefParams["Q0_mph"]
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Discharge_Subsurface_Initial_mph>'
    if str_element_old == 'T0_m2ph>':
        #Note: C++ change input->RefParams["T0_m2ph"]
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'VadoseZone_Transmissivity_Max_m2ph>'
    if str_element_old == '<m>':
        #Note: C++ change input->RefParams["m"]
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = '<Parameter_m_KsatExpDecay>'
    if str_element_old == '</m>':
        #Note: C++ change input->RefParams["m"]
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = '</Parameter_m_KsatExpDecay>'
    if str_element_old == '<!-- Scale parameter of soil transmissivity-->':
        #Note: C++ change input->RefParams["m"]
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = '<!--Options: >0; Scale parameter for decay of hydraulic conductivity with depth; unitless-->'    
    if str_element_old == '<n>':
        #Note: C++ change input->RefParams["n"] double N
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 1
        str_element_new = '<Parameter_n_KsatPowerDecay>'
        #define new element lines
        str_out_line_01 = "<Parameter_n_KsatPowerDecay>2</Parameter_n_KsatPowerDecay><!--Options: 1 or 2; Power for decay of hydraulic conductivity with depth; unitless-->"
        
    if str_element_old == 'DCIA_frac>':
        #Note: C++ change input->RefParams["m"]
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'DirectlyConnectedImperviousArea_frac>'
    if str_element_old == 'K0_mph>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Soil_Ksat_mph>'
    if str_element_old == 'WFS_m>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Soil_WettingFront_Suction_m>'
    if str_element_old == 'PAQ_RT_A_h>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Routing_PerviousArea_Alpha_h>'
    if str_element_old == 'PAQ_RT_B_h>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Routing_PerviousArea_Beta_h>'
    if str_element_old == 'DCIAQ_RT_A_h>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Routing_ImperviousArea_Alpha_h>'
    if str_element_old == 'DCIAQ_RT_B_h>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Routing_ImperviousArea_Beta_h>'
    if str_element_old == 'SSQ_RT_h>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Routing_Subsurface_Beta_h>'
    if str_element_old == 'LeafOnDay>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'LeafOnDate_JDay>'
    if str_element_old == 'LeafOffDay>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'LeafOffDate_JDay>'
    if str_element_old == 'LeafTransDays>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'LeafTransition_Day>'

    if str_element_old == 'TreeLAI>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'LeafAreaIndex_Tree_m2pm2>'
    if str_element_old == 'ShortVegLAI>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'LeafAreaIndex_SVeg_m2pm2>'
    if str_element_old == 'TreeBAI>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'BarkAreaIndex_Tree_m2pm2>'    
    if str_element_old == 'ShrubBAI>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'BarkAreaIndex_SVeg_m2pm2>'
    if str_element_old == 'EvergreenTreeCover_frac>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'TreeCover_Evergreen_frac>'
    if str_element_old == 'EverGreenShrubCover_frac>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'SVegCover_Evergreen_frac>'
        
    if str_element_old == '<TreeLeafStorage_mm>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 4
        str_element_new = '<LeafAreaStorage_Tree_mm>'
        #define new element lines
        str_out_line_01 = "<LeafAreaStorage_Tree_mm>0.2</LeafAreaStorage_Tree_mm><!--Options: >=0; liquid storage depth maximum-->"
        str_out_line_02 = "<BarkAreaStorage_Tree_mm>0.2</BarkAreaStorage_Tree_mm><!--Options: >=0; liquid storage depth maximum-->"
        str_out_line_03 = "<LeafAreaStorage_SVeg_mm>0.2</LeafAreaStorage_SVeg_mm><!--Options: >=0; liquid storage depth maximum-->"
        str_out_line_04 = "<BarkAreaStorage_SVeg_mm>0.2</BarkAreaStorage_SVeg_mm><!--Options: >=0; liquid storage depth maximum-->"

    if str_element_old == 'WeatherWindHeight>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Sensor_Wind_m>'
    if str_element_old == 'Flag_UrbanCanyon>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Flag_UrbanCanyonResistance>'
        
    if str_element_old == 'Treeheight>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Avg_TreeCover_m>'
    if str_element_old == 'Treealbedo>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Albedo_Tree_frac>'
    if str_element_old == 'Treeemissivity>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Emissivity_Tree_frac>'
    if str_element_old == 'TreeLeafWidth>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Width_Leaf_Tree_m>'
    if str_element_old == 'TreeMinimumResistance>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Resistance_Tree_Min_spm>'
        
    if str_element_old == 'ShortVegetationheight>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Avg_SVegCover_m>'
    if str_element_old == 'ShortVegetationalbedo>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Albedo_SVeg_frac>'
    if str_element_old == 'ShortVegetationemissivity>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Emissivity_SVeg_frac>'
    if str_element_old == 'ShortVegetationLeafWidth>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Width_Leaf_SVeg_m>'
    if str_element_old == 'ShortVegetationMinimumResistance>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Resistance_SVeg_Min_spm>'
        
    if str_element_old == 'Soilheight>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Avg_SoilCover_m>'
    if str_element_old == 'Soilalbedo>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Albedo_Soil_frac>'
    if str_element_old == 'Soilemissivity>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Emissivity_Soil_frac>'

    if str_element_old == 'Waterheight>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Avg_WaterCover_m>'
    if str_element_old == 'Wateralbedo>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Albedo_Water_frac>'
    if str_element_old == 'Wateremissivity>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Emissivity_Water_frac>'

    if str_element_old == 'Urban21height>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Avg_Urban21_m>'
    if str_element_old == 'Urban21roadwidth>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Width_Road_Urban21_m>'
    if str_element_old == 'Urban21albedo>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Albedo_Urban21_frac>'
    if str_element_old == 'Urban21emissivity>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Emissivity_Urban21_frac>'
    if str_element_old == 'Urban21buildingP>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Area_Roof_Urban21_frac>'

    if str_element_old == 'Urban22height>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Avg_Urban22_m>'
    if str_element_old == 'Urban22roadwidth>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Width_Road_Urban22_m>'
    if str_element_old == 'Urban22albedo>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Albedo_Urban22_frac>'
    if str_element_old == 'Urban22emissivity>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Emissivity_Urban22_frac>'
    if str_element_old == 'Urban22buildingP>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Area_Roof_Urban22_frac>'

    if str_element_old == 'Urban23height>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Avg_Urban23_m>'
    if str_element_old == 'Urban23roadwidth>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Width_Road_Urban23_m>'
    if str_element_old == 'Urban23albedo>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Albedo_Urban23_frac>'
    if str_element_old == 'Urban23emissivity>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Emissivity_Urban23_frac>'
    if str_element_old == 'Urban23buildingP>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Area_Roof_Urban23_frac>'

    if str_element_old == 'Urban24height>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Height_Avg_Urban24_m>'
    if str_element_old == 'Urban24roadwidth>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Width_Road_Urban24_m>'
    if str_element_old == 'Urban24albedo>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Albedo_Urban24_frac>'
    if str_element_old == 'Urban24emissivity>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Emissivity_Urban24_frac>'
    if str_element_old == 'Urban24buildingP>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 0
        str_new = 'Area_Roof_Urban24_frac>'

    
    if str_element_old == '<RefWeatherLocationTopographicIndex>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 1
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 3
        str_element_new = '<RefWeatherLocationNLCD_Class>'
        #define new element lines
        str_out_line_01 = "<RefWeatherLocationNLCD_Class>21</RefWeatherLocationNLCD_Class><!--Options: 'Map' or >=0; defines NLCD Class w Map or element value; unitless-->"
        str_out_line_02 = "<RefWeatherLocationTreeCover_percent>0</RefWeatherLocationTreeCover_percent><!--Options: 'Map' or 0-100; defines tree canopy cover area w Map or element value; percent-->"
        str_out_line_03 = "<RefWeatherLocationImperviousCover_percent>25</RefWeatherLocationImperviousCover_percent><!--Options: 'Map' or 0-100; defines impervious cover area w Map or element value; percent-->"


    if str_element_old == '<Soil_Macropore_frac>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 3
        str_element_new = '<Soil_Macropore_TreeCover_frac>'
        #define new element lines
        str_out_line_01 = "<Soil_Macropore_TreeCover_frac>0.1</Soil_Macropore_TreeCover_frac><!--Options: 0 to 1; determines fraction of infiltration bypassing rootzone-->"
        str_out_line_02 = "<Soil_Macropore_SVegCover_frac>0.05</Soil_Macropore_SVegCover_frac><!--Options: 0 to 1; determines fraction of infiltration bypassing rootzone-->"
        str_out_line_03 = "<Soil_Macropore_SoilCover_frac>0.01</Soil_Macropore_SoilCover_frac><!--Options: 0 to 1; determines fraction of infiltration bypassing rootzone-->"

    if str_element_old == '<Evapotranspiration_Depth_m>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 3
        str_element_new = '<EvapotranspirationDepth_TreeCover_m>'
        #define new element lines
        str_out_line_01 = "<EvapotranspirationDepth_TreeCover_m>1.0</EvapotranspirationDepth_TreeCover_m><!--Options: >=0; depth accessible to evapotranspiration demand; units m-->"
        str_out_line_02 = "<EvapotranspirationDepth_SVegCover_m>0.5</EvapotranspirationDepth_SVegCover_m><!--Options: >=0; depth accessible to evapotranspiration demand; units m-->"
        str_out_line_03 = "<EvapotranspirationDepth_SoilCover_m>0.2</EvapotranspirationDepth_SoilCover_m><!--Options: >=0; depth accessible to evapotranspiration demand; units m-->"

    if str_element_old == '<PerviousDepressionStorage_mm>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 3
        str_element_new = '<PerviousDepressionStorage_TreeCover_mm>'
        #define new element lines
        str_out_line_01 = "<PerviousDepressionStorage_TreeCover_mm>5.0</PerviousDepressionStorage_TreeCover_mm><!--Options: >=0; average ponding depth; units mm-->"
        str_out_line_02 = "<PerviousDepressionStorage_SVegCover_mm>2.5</PerviousDepressionStorage_SVegCover_mm><!--Options: >=0; average ponding depth; units mm-->"
        str_out_line_03 = "<PerviousDepressionStorage_SoilCover_mm>1.5</PerviousDepressionStorage_SoilCover_mm><!--Options: >=0; average ponding depth; units mm-->"
        
    if str_element_old == '<ImperviousDepressionStorage_mm>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 0
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 2
        str_element_new = '<ImperviousDepressionStorage_noTreeCover_mm>' 
        #define new element lines
        str_out_line_01 = "<ImperviousDepressionStorage_noTreeCover_mm>1.5</ImperviousDepressionStorage_noTreeCover_mm><!--Options: >=0; average ponding depth; units mm-->"
        str_out_line_02 = "<ImperviousDepressionStorage_TreeCover_mm>2.5</ImperviousDepressionStorage_TreeCover_mm><!--Options: >=0; average ponding depth; units mm-->"

    # if str_element_old == '<TreeLeafStorage_mm>':
        # #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        # flag_replace_w_element_and_value = 1
        # #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        # flag_append_to_target = 1
        # #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        # #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        # flag_append_differently_bulkArea_vs_GI = 1
        # cnt_out_lines = 3
        # str_element_new = '<Height_TreeCover_Avg_m>'
        # #define new element lines
        # str_out_line_01 = "<Height_TreeCover_Avg_m>12</Height_TreeCover_Avg_m><!-- Options: >0; Average tree cover height; units m-->"
        # str_out_line_02 = "<Height_SVegCover_Avg_m>0.1</Height_SVegCover_Avg_m><!-- Options: > 0; Average short veg cover height; units m-->"
        # str_out_line_03 = "<Height_Sensor_Wind_m>10.0</Height_Sensor_Wind_m><!-- Options: > 0; height at which wind speed measured; units m-->"

    if str_element_old == '<Flag_TI_PerviousOnly>':
        #flag_replace_w_element_and_value; =1 then target element string and value replaced, =0 then only target element string replaced
        flag_replace_w_element_and_value = 1
        #flag_append_to_target; =1 then new elements appended below target in HydroPlusConfig.xml, =0 then target removed and elements appended
        flag_append_to_target = 1
        #flag_append_differently_bulkArea_vs_GI; =1 then new elements appended are different for Bulk Area and GI 
        #Note: If equal 1, then go change output rules for section below where icnt_Folder == 1
        flag_append_differently_bulkArea_vs_GI = 0
        cnt_out_lines = 1
        str_element_new = '<Flag_ReadEvaporationData>' 
        #define new element lines
        str_out_line_01 = "<Flag_ReadEvaporationData>1</Flag_ReadEvaporationData><!--Options: 0 or 1; 0=Evaporation calculated in model; 1=Evaporation.csv provided-->"

    print("Searching for {}.".format(str_element_old))
    
    if (flag_replace_w_element_and_value == 0):
        print("Replacing {} string with {} string.".format(str_element_old, str_new))

    if (flag_replace_w_element_and_value == 1):
        print("Appending {} as part of {} lines appended.".format(str_element_new, cnt_out_lines))
        print("flag_append_to_target set to {}.".format(flag_append_to_target))
        print("flag_append_differently_bulkArea_vs_GI set to {}.".format(flag_append_to_target))

        #define start of str_element_old
        str_start = '<'
        #len_element_old is length of str_element_old variable, that which exists and will be replaced or serve as location for new element
        len_element_old = len(str_element_old)        
        #len_element_new is length of str_element_new variable, that which replaces or appends above existing element
        len_element_new = len(str_element_new)


        #define whitespace
        str_whitespace_1 = ' '
        str_whitespace_2 = '  '
        str_whitespace_3 = '   '
        str_whitespace_4 = '    '
        str_whitespace_5 = '     '
        str_whitespace_6 = '      '
        str_whitespace_7 = '       '
        str_whitespace_8 = '        '
        str_whitespace_8 = '         '


    #find_files_recursively function will search the folder_directory for the files of the defined extensions, or any sub-directories, and return their path
    def find_files_recursively(folder_directory):
        #for loop using os.scandir function at folder_directory
        for file_in_directory in os.scandir(folder_directory):
            #if file_in_directory satisfies is_file function and file_in_directory satisfies name.endswith function for ext variable then proceed
            if file_in_directory.is_file() and file_in_directory.name.endswith(ext):
                #return file name
                #yield file_in_directory.name
                #return path name
                yield file_in_directory.path
            #else if file_in_directory is a directory and not a file, the call find_files_recursively function and return item
            elif file_in_directory.is_dir():
                yield from find_files_recursively(file_in_directory.path)
            #else:
                #print("Neither a file, nor a dir: {}".format(file_in_directory.path))
       
    #for loop through function find_files_recursively, getting File_path for each match
    for File_path in find_files_recursively(path_root):
        #print(File_path)
        #file_name_tpl is last 23 characters of path
        file_name_tpl = File_path.rstrip()[-len_file_tpl:]
        #file_name_xml is last 19 characters of path
        file_name_xml = File_path.rstrip()[-len_file_xml:]
        #file_name_template_xml is last 12 characters of path
        file_name_template_xml = File_path.rstrip()[-len_file_template_xml:]
        #if file_name_tpl or file_name_xml equals target then proceed
        if (file_name_tpl == str_file_tpl or file_name_xml == str_file_xml or file_name_template_xml == str_file_template_xml):
            print('File_path = {}'.format(File_path))
            # if (file_name_tpl == str_file_tpl):
                # print("file_name_tpl = {}".format(file_name_tpl))
            # if (file_name_xml == str_file_xml):
                # print("file_name_xml = {}".format(file_name_xml))
                
            #If flag_replace_w_element_and_value =0 then target element string only replaced
            if (flag_replace_w_element_and_value == 0):

                #with open HydroPlusConfig.xml file, use "r" read function, named in_file
                with open(File_path, "r") as HydroPlus_in_file:
                    #Config_buffer takes all content in HydroPlusConfig.xml file
                    Config_buffer = HydroPlus_in_file.read()

                #Config_buffer is updated with replace function, replacing str_element_old with str_new
                Config_buffer = Config_buffer.replace(str_element_old, str_new)

                #with open HydroPlusConfig.xml file, use "w" write function, named out_file
                with open(File_path, "w") as HydroPlus_out_file:
                    HydroPlus_out_file.write(Config_buffer)

            #If flag_replace_w_element_and_value =1 then target element string and value replaced
            if (flag_replace_w_element_and_value == 1):

                #with open HydroPlusConfig.xml file, use "r" read function, named in_file
                with open(File_path, "r") as HydroPlus_in_file:
                    #Config_buffer takes all content in HydroPlusConfig.xml file
                    Config_buffer = HydroPlus_in_file.readlines()

                #reset counters
                icnt = 0
                icnt_target = 0
                icnt_check = 0
                
                #search if file needs to be updated
                for line in Config_buffer:
                    icnt = icnt + 1
                    #str_line is line trimmed of edge white space on left and truncated to first len_element_old columns using .lstrip() and [:len_element_old] functions
                    str_line_a = line.lstrip()[:len_element_old]
                    #str_line is line trimmed of edge white space on left and truncated to first len_element_new columns using .lstrip() and [:len_element_new] functions
                    str_line_b = line.lstrip()[:len_element_new]
                                        
                    #if str_line_a equals str_element_old then enter to prepare for and write new elements
                    if str_line_a == str_element_old:
                        icnt_target = icnt
                        #len_line is length of line
                        len_line = len(line)
                        #len_line is length of line stripped of left white space
                        len_line_lstrip = len(line.lstrip())
                        #len_whitespace is difference in len_line and len_line_lstrip
                        len_whitespace = len_line - len_line_lstrip
                        #print("len_line = {} - len_line_lstrip = {} = Whitespace = {}".format(len_line,len_line_lstrip,len_whitespace))
                        #if len_whitespace is < 3 then it is likely a tab, and set to 4
                        if (len_whitespace < 3):
                            len_whitespace = 4
                        # #match defines where str_start is within line
                        # match = (re.search(str_start,line))
                        # print("line starts at {}".format(match.start()))
                        # print("line starts at {}".format(line.rfind('<')))
                    #if str_line_b equals str_element_old then enter to save icnt value, but not to write new elements
                    if str_line_b == str_element_new:
                        icnt_check = icnt
                        #print("icnt_check is {}".format(icnt_check))

                #if icnt_check > 0 and (icnt_check - icnt_target == 1) then str_element_new is already after str_element_old, so do not write new file
                if (icnt_check > 0 and (icnt_check - icnt_target) == 1) or (icnt_check > 0 and icnt_target == 0):
                    print("File already updated with icnt_check = {}.".format(icnt_check))
                #elif icnt_check equals zero (icnt_check - icnt_target) not equal to 1, then file not already updated and updating needed
                elif ((icnt_check == 0 and (icnt_check - icnt_target) != 1 and icnt_target > 0)):
                    print("File is updating with icnt_target = {}.".format(icnt_target))
                    icnt_Folder = 0
                    #with open HydroPlusConfig.xml file, use "w" write function, named out_file
                    with open(File_path, "w") as HydroPlus_out_file:
                        for line in Config_buffer:
                            #str_line is line trimmed of edge white space and truncated to first 20 columns using .strip() and [:20] functions
                            str_line = line.strip()[:len_element_old]
                            #if str_line equals str_element_old then enter
                            if str_line == str_element_old:
                                #if len_whitespace equals a value then determine str_whitespace value for adding white space to left side of line
                                if len_whitespace == 1:
                                    str_whitespace = str_whitespace_1
                                elif len_whitespace == 2:
                                    str_whitespace = str_whitespace_2
                                elif len_whitespace == 3:
                                    str_whitespace = str_whitespace_3
                                elif len_whitespace == 4:
                                    str_whitespace = str_whitespace_4
                                elif len_whitespace == 5:
                                    str_whitespace = str_whitespace_5
                                elif len_whitespace == 6:
                                    str_whitespace = str_whitespace_6
                                elif len_whitespace == 7:
                                    str_whitespace = str_whitespace_7
                                elif len_whitespace == 8:
                                    str_whitespace = str_whitespace_8
                                elif len_whitespace == 9:
                                    str_whitespace = str_whitespace_9
                                #if icnt_Folder equals zero then in BulkArea folder
                                if (icnt_Folder == 0):
                                    #If flag_append_to_target equals 1 then write original line and append new elements
                                    if (flag_append_to_target == 1):
                                        #line appended to include original line, plus str_out_ lines
                                        if (cnt_out_lines == 0):
                                            line = line + "\n"
                                        if (cnt_out_lines == 1):
                                            line = line + str_whitespace + str_out_line_01 + "\n"
                                        if (cnt_out_lines == 2):
                                            line = line + str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n"
                                        if (cnt_out_lines == 3):
                                            line = line + str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n"
                                        if (cnt_out_lines == 4):
                                            line = line + str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n" + str_whitespace + str_out_line_04 + "\n"
                                    #Else If flag_append_to_target equals 0 then do not write original line, it is removed, and append new elements
                                    else:
                                        #line appended to include original line, plus three str_out_ lines
                                        if (cnt_out_lines == 0):
                                            line = ""
                                        if (cnt_out_lines == 1):
                                            line = str_whitespace + str_out_line_01 + "\n"
                                        if (cnt_out_lines == 2):
                                            line = str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n"
                                        if (cnt_out_lines == 3):
                                            line = str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n"
                                        if (cnt_out_lines == 4):
                                            line = str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n" + str_whitespace + str_out_line_04 + "\n"
                                    
                                #if icnt_Folder equals one then in GI folder
                                if (icnt_Folder == 1):
                                    #if flag_append_differently_bulkArea_vs_GI equals 1 then new elements appended are different for Bulk Area and GI folders
                                    if (flag_append_differently_bulkArea_vs_GI == 1):
                                        #If flag_append_to_target equals 1 then write original line and append new elements
                                        if (flag_append_to_target == 1):
                                            #line appended to include original line, plus two str_out_ lines
                                            #Note: don't write 3rd line to GI, as there is only one weather station
                                            if (cnt_out_lines == 0):
                                                line = line + "\n"
                                            if (cnt_out_lines == 1):
                                                line = line
                                            if (cnt_out_lines == 2):
                                                line = line + str_whitespace + str_out_line_01 + "\n"
                                            if (cnt_out_lines == 3):
                                                line = line + str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n"
                                            if (cnt_out_lines == 4):
                                                line = line + str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n"
                                        #Else If flag_append_to_target equals 0 then do not write original line, it is removed, and append new elements
                                        else:
                                            #line appended to include original line, plus two str_out_ lines
                                            #Note: don't write 3rd line to GI, as there is only one weather station
                                            if (cnt_out_lines == 2):
                                                line = str_whitespace + str_out_line_01 + "\n"
                                            if (cnt_out_lines == 3):
                                                line = str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n"
                                            if (cnt_out_lines == 4):
                                                line = str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n"
                                    #Else If flag_append_differently_bulkArea_vs_GI equals 0 then new elements appended are same for Bulk Area and GI folders
                                    else:
                                        #If flag_append_to_target equals 1 then write original line and append new elements
                                        if (flag_append_to_target == 1):
                                            #line appended to include original line, plus three str_out_ lines
                                            if (cnt_out_lines == 0):
                                                line = line + "\n"
                                            if (cnt_out_lines == 1):
                                                line = line + str_whitespace + str_out_line_01 + "\n"
                                            if (cnt_out_lines == 2):
                                                line = line + str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n"
                                            if (cnt_out_lines == 3):
                                                line = line + str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n"
                                            if (cnt_out_lines == 4):
                                                line = line + str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n" + str_whitespace + str_out_line_04 + "\n"
                                        #Else If flag_append_to_target equals 0 then do not write original line, it is removed, and append new elements
                                        else:
                                            #line appended to include original line, plus three str_out_ lines
                                            if (cnt_out_lines == 0):
                                                line = ""
                                            if (cnt_out_lines == 1):
                                                line = str_whitespace + str_out_line_01 + "\n"
                                            if (cnt_out_lines == 2):
                                                line = str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n"
                                            if (cnt_out_lines == 3):
                                                line = str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n"
                                            if (cnt_out_lines == 4):
                                                line = str_whitespace + str_out_line_01 + "\n" + str_whitespace + str_out_line_02 + "\n" + str_whitespace + str_out_line_03 + "\n" + str_whitespace + str_out_line_04 + "\n"
                                    
                                #icnt_Folder increments to indicate if BulkArea of GI
                                icnt_Folder = icnt_Folder + 1
                            #HydroPlus_out_file has line written, adding all lines plus additional lines
                            HydroPlus_out_file.write(line)
                #Else If file was updated or already updated, meaning an error likely occurred and has no target
                else:
                    print("File is not a target, check inputs.")
            
#Write end of process
print("Program has completed. Check the HydroPlusConfig.xml.")


#Appendix of prior targets and output lines 
#Fall 2022
# #define str_element_old
# str_element_old = '<TreeLeafStorage_mm>'#'<Flag_TI_PerviousOnly>' #'<TreeLeafStorage_mm>'
# len_element_old = len(str_element_old)
# str_element_new = '<Treeheight>' #'<Flag_ReadEvaporationData>' #'<Treeheight>'
# len_element_new = len(str_element_new)
# #define start of str_element_old
# str_start = '<'

# #define str_out lines
# cnt_out_lines = 3 #1 #3
# #Note: HydroPlusConfig uses input->RefParams[""] for BulkArea parameters
# if (cnt_out_lines == 3):
    # str_out_line_01 = "<Treeheight>12</Treeheight><!-- Options: > 0; roughness height for evaporation resistance; units m-->"
    # str_out_line_02 = "<ShortVegetationheight>0.1</ShortVegetationheight><!-- Options: > 0; roughness height for evaporation resistance; units m-->"
    # str_out_line_03 = "<WeatherWindHeight>10.0</WeatherWindHeight><!-- Options: > 0; height at which wind speed measured; units m-->"
# if (cnt_out_lines == 1):
    # str_out_line_01 = "<Flag_ReadEvaporationData>1</Flag_ReadEvaporationData><!--Options: 0 or 1; 0=Evaporation calculated in model; 1=Evaporation.csv provided-->"
